@@ -35,6 +35,7 @@
 #include "core.h"
 #include "debug.h"
 #include "prpl.h"
+#include "notify.h"
 
 /* XXX */
 #include "gtkconv.h"
@@ -157,7 +158,7 @@ gaim_remote_handle_uri(const char *uri)
 		}
 
 		gaim_debug(GAIM_DEBUG_MISC, "gaim_remote_handle_uri", "who: %s\n", who);
-		show_add_buddy(gc, who, group, NULL);
+		gaim_blist_request_add_buddy(gc->account, who, group, NULL);
 		g_free(who);
 		if (group)
 			g_free(group);
@@ -313,7 +314,7 @@ meta_handler(struct UI *ui, guchar subtype, gchar *data)
 			g_source_remove(ui->inpa);
 			g_free(ui);
 		}
-		gaim_core_quit();
+		g_timeout_add(0, gaim_core_quit_cb, NULL);
 		break;
 	case CUI_META_DETACH:
 		uis = g_slist_remove(uis, ui);
@@ -434,7 +435,7 @@ message_handler(struct UI *ui, guchar subtype, gchar *data)
 			pos += len;
 
 			memcpy(&flags, data + pos, sizeof(flags));
-			serv_send_im(gc, who, msg, -1, flags);
+			serv_send_im(gc, who, msg, flags);
 
 			g_free(who);
 			g_free(msg);
@@ -458,7 +459,7 @@ gaim_recv(GIOChannel *source, gchar *buf, gint len)
 	GError *error = NULL;
 
 	while (total < len) {
-		if (g_io_channel_read_chars(source, buf + total, len - total, &cur, &error) != G_IO_STATUS_NORMAL) {
+		if (g_io_channel_read_chars(source, buf + total, len - total, (gsize *) &cur, &error) != G_IO_STATUS_NORMAL) {
 			if (error)
 				g_error_free(error);
 			return -1;
@@ -501,7 +502,7 @@ UI_readable(GIOChannel *source, GIOCondition cond, gpointer data)
 
 	gchar type;
 	gchar subtype;
-	guint32 len;
+	gint len;
 
 	GError *error = NULL;
 
@@ -628,7 +629,7 @@ socket_readable(GIOChannel *source, GIOCondition cond, gpointer data)
 }
 
 static gint
-open_socket()
+open_socket(char **error)
 {
 	struct sockaddr_un saddr;
 	gint fd;
@@ -647,9 +648,12 @@ open_socket()
 		if (bind(fd, (struct sockaddr *)&saddr, sizeof(saddr)) != -1)
 			listen(fd, 100);
 		else {
+			*error = g_strdup_printf(_("Failed to assign %s to a socket:\n%s"),
+					   saddr.sun_path, strerror(errno));
 			g_log(NULL, G_LOG_LEVEL_CRITICAL,
 			      "Failed to assign %s to a socket (Error: %s)",	
 			      saddr.sun_path, strerror(errno));
+			umask(m);
 			return -1;
 		}
 		umask(m);
@@ -664,9 +668,13 @@ plugin_load(GaimPlugin *plugin)
 {
 #ifndef _WIN32
 	GIOChannel *channel;
+	char *buf;
 
-	if ((UI_fd = open_socket()) < 0)
+	if ((UI_fd = open_socket(&buf)) < 0) {
+		gaim_notify_error(NULL, NULL, _("Unable to open socket"), buf);
+		g_free(buf);
 		return FALSE;
+	}
 
 	channel = g_io_channel_unix_new(UI_fd);
 	watcher = g_io_add_watch(channel, G_IO_IN, socket_readable, NULL);
@@ -703,7 +711,7 @@ plugin_unload(GaimPlugin *plugin)
 
 static GaimPluginInfo info =
 {
-	2,                                                /**< api_version    */
+	GAIM_PLUGIN_API_VERSION,                          /**< api_version    */
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	GAIM_GTK_PLUGIN_TYPE,                             /**< ui_requirement */
 	0,                                                /**< flags          */
@@ -718,7 +726,7 @@ static GaimPluginInfo info =
 	                                                  /**  description    */
 	N_("Gives Gaim the ability to be remote-controlled through third-party "
 	   "applications or through the gaim-remote tool."),
-	"Christian Hammond <chipx86@gnupdate.org>",       /**< author         */
+	"Sean Egan <sean.egan@binghamton.edu>",
 	GAIM_WEBSITE,                                     /**< homepage       */
 
 	plugin_load,                                      /**< load           */
@@ -726,7 +734,9 @@ static GaimPluginInfo info =
 	NULL,                                             /**< destroy        */
 
 	NULL,                                             /**< ui_info        */
-	NULL                                              /**< extra_info     */
+	NULL,                                             /**< extra_info     */
+	NULL,
+	NULL
 };
 
 static void
