@@ -1,7 +1,9 @@
 /*
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +27,9 @@
 #include "plugin.h"
 #include "prefs.h"
 #include "prpl.h"
+#include "status.h"
 #include "util.h"
+#include "request.h"
 
 /* XXX CORE/UI: Until we can get rid of the message queue stuff... */
 #include "gaim.h"
@@ -56,7 +60,7 @@ static void dequeue_message(GtkTreeIter *iter)
 
 	gtk_tree_model_get(GTK_TREE_MODEL(awayqueuestore), iter, 0, &name, -1);
 
-	gaim_debug(GAIM_DEBUG_INFO, "away", "Unqueueing messages from %s.\n",
+	gaim_debug(GAIM_DEBUG_INFO, "away", "Dequeueing messages from %s.\n",
 			   name);
 
 	templist = message_queue;
@@ -77,7 +81,7 @@ static void dequeue_message(GtkTreeIter *iter)
 				else
 					gaim_conversation_set_account(cnv, account);
 
-				gaim_im_write(GAIM_IM(cnv), NULL, qm->message, qm->len,
+				gaim_conv_im_write(GAIM_CONV_IM(cnv), NULL, qm->message,
 						qm->flags, qm->tm);
 				g_free(qm->message);
 				g_free(qm);
@@ -117,7 +121,7 @@ void purge_away_queue(GSList **queue)
 		else
 			gaim_conversation_set_account(cnv, account);
 
-		gaim_im_write(GAIM_IM(cnv), NULL, qm->message, -1, qm->flags, qm->tm);
+		gaim_conv_im_write(GAIM_CONV_IM(cnv), NULL, qm->message, qm->flags, qm->tm);
 
 		g_free(qm->message);
 		g_free(qm);
@@ -135,7 +139,7 @@ gint dequeue_cb(GtkWidget *treeview, GdkEventButton *event, gpointer data) {
 	GtkTreeSelection *select;
 
 	if(!(event->type == GDK_2BUTTON_PRESS && event->button == 1))
-		return FALSE; /* Double clicking on the list will unqueue that user's messages. */
+		return FALSE; /* Double clicking on the list will dequeue that user's messages. */
 
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
 	if(gtk_tree_selection_get_selected(select, NULL, &iter))
@@ -204,7 +208,7 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 	if (strlen(a->name))
 		gtk_window_set_title(GTK_WINDOW(imaway), a->name);
 	else
-		gtk_window_set_title(GTK_WINDOW(imaway), _("Gaim - Away!"));
+		gtk_window_set_title(GTK_WINDOW(imaway), _("Away!"));
 	g_signal_connect(G_OBJECT(imaway), "destroy",
 			G_CALLBACK(do_im_back), imaway);
 	gtk_widget_realize(imaway);
@@ -216,7 +220,7 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 
 	sw = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER,
-			GTK_POLICY_ALWAYS);
+			GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
 	gtk_widget_set_size_request(sw, 245, 120);
 	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
@@ -227,10 +231,10 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 	gaim_setup_imhtml(awaytext);
 	gtk_widget_show(awaytext);
 	buf = stylize(a->message, BUF_LONG);
-	gtk_imhtml_append_text(GTK_IMHTML(awaytext), buf, -1, GTK_IMHTML_NO_TITLE |
+	gtk_imhtml_append_text(GTK_IMHTML(awaytext), buf, GTK_IMHTML_NO_TITLE |
 			GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
 	g_free(buf);
-	gtk_imhtml_append_text(GTK_IMHTML(awaytext), "<BR>", -1,
+	gtk_imhtml_append_text(GTK_IMHTML(awaytext), "<BR>",
 			GTK_IMHTML_NO_TITLE | GTK_IMHTML_NO_COMMENTS |
 			GTK_IMHTML_NO_SCROLL);
 
@@ -241,7 +245,7 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 			GTK_SHADOW_IN);
 	gtk_box_pack_start(GTK_BOX(vbox), awayqueuesw, TRUE, TRUE, 0);
 
-	awayqueuestore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	awayqueuestore = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	awayqueue = gtk_tree_view_new_with_model(GTK_TREE_MODEL(awayqueuestore));
 	renderer = gtk_cell_renderer_text_new();
 
@@ -254,6 +258,10 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 			"text", 1,
 			NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
+	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
+			"text", 2,
+			NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(awayqueue), column);
 
 	gtk_container_add(GTK_CONTAINER(awayqueuesw), awayqueue);
 
@@ -277,12 +285,26 @@ void do_away_message(GtkWidget *w, struct away_message *a)
 	serv_set_away_all(awaymessage->message);
 }
 
-void rem_away_mess(GtkWidget *w, struct away_message *a)
+void do_rem_away_mess(gchar *name)
 {
+	struct away_message *a = NULL;
 	struct away_message *default_away = NULL;
 	const char *default_away_name;
 	GSList *l;
 
+	/* Lookup the away message based on the title */
+	for (l = away_messages; l != NULL; l = l->next) {
+		a = l->data;
+		if (!strcmp(a->name, name))
+			break;
+	}
+	g_free(name);
+
+	if (l == NULL || a == NULL) {
+		/* Could not find away message! */
+		return;
+	}
+
 	default_away_name = gaim_prefs_get_string("/core/away/default_message");
 
 	for(l = away_messages; l; l = l->next) {
@@ -298,6 +320,20 @@ void rem_away_mess(GtkWidget *w, struct away_message *a)
 	away_messages = g_slist_remove(away_messages, a);
 	g_free(a);
 	do_away_menu();
+	gaim_status_sync();
+}
+
+void rem_away_mess(GtkWidget *w, struct away_message *a)
+{
+	gchar *text;
+
+	text = g_strdup_printf(_("Are you sure you want to remove the away message \"%s\"?"), a->name);
+
+	gaim_request_action(NULL, NULL, _("Remove Away Message"), text, -1, g_strdup(a->name), 2,
+						_("Remove"), G_CALLBACK(do_rem_away_mess),
+						_("Cancel"), G_CALLBACK(g_free));
+
+	g_free(text);
 }
 
 static void set_gc_away(GObject *obj, GaimConnection *gc)
@@ -360,6 +396,7 @@ void do_away_menu()
 			l = l->next;
 		}
 
+		g_list_free(l);
 
 		remmenu = gtk_menu_new();
 
