@@ -4,7 +4,9 @@
  *
  * gaim
  *
- * Copyright (C) 2002-2003, Christian Hammond <chipx86@gnupdate.org>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,12 +31,12 @@
 /** Data Structures                                                       */
 /**************************************************************************/
 
-typedef struct _GaimWindowUiOps       GaimWindowUiOps;
-typedef struct _GaimWindow            GaimWindow;
+typedef struct _GaimConvWindowUiOps   GaimConvWindowUiOps;
+typedef struct _GaimConvWindow        GaimConvWindow;
 typedef struct _GaimConversationUiOps GaimConversationUiOps;
 typedef struct _GaimConversation      GaimConversation;
-typedef struct _GaimIm                GaimIm;
-typedef struct _GaimChat              GaimChat;
+typedef struct _GaimConvIm            GaimConvIm;
+typedef struct _GaimConvChat          GaimConvChat;
 
 /**
  * A type of conversation.
@@ -66,14 +68,14 @@ typedef enum
 typedef enum
 {
 	GAIM_CONV_UPDATE_ADD = 0, /**< The buddy associated with the conversation
-							       was added.   */
+	                               was added.   */
 	GAIM_CONV_UPDATE_REMOVE,  /**< The buddy associated with the conversation
-								   was removed. */
+	                               was removed. */
 	GAIM_CONV_UPDATE_ACCOUNT, /**< The gaim_account was changed. */
 	GAIM_CONV_UPDATE_TYPING,  /**< The typing state was updated. */
 	GAIM_CONV_UPDATE_UNSEEN,  /**< The unseen state was updated. */
 	GAIM_CONV_UPDATE_LOGGING, /**< Logging for this conversation was
-								   enabled or disabled. */
+	                               enabled or disabled. */
 	GAIM_CONV_UPDATE_TOPIC,   /**< The topic for a chat was updated. */
 
 	/*
@@ -82,7 +84,10 @@ typedef enum
 	 */
 	GAIM_CONV_ACCOUNT_ONLINE,  /**< One of the user's accounts went online.  */
 	GAIM_CONV_ACCOUNT_OFFLINE, /**< One of the user's accounts went offline. */
-	GAIM_CONV_UPDATE_AWAY      /**< The other user went away.                */
+	GAIM_CONV_UPDATE_AWAY,     /**< The other user went away.                */
+	GAIM_CONV_UPDATE_ICON,     /**< The other user's buddy icon changed.     */
+	GAIM_CONV_UPDATE_TITLE,
+	GAIM_CONV_UPDATE_CHATLEFT
 
 } GaimConvUpdateType;
 
@@ -102,43 +107,47 @@ typedef enum
  */
 typedef enum
 {
-	GAIM_MESSAGE_SEND      = 0x0001, /**< Outgoing message.     */
-	GAIM_MESSAGE_RECV      = 0x0002, /**< Incoming message.     */
-	GAIM_MESSAGE_SYSTEM    = 0x0004, /**< System message.       */
-	GAIM_MESSAGE_AUTO_RESP = 0x0008, /**< Auto response.        */
-	GAIM_MESSAGE_COLORIZE  = 0x0010, /**< Colorize nicks.       */
-	GAIM_MESSAGE_NICK      = 0x0020, /**< Contains your nick.   */
-	GAIM_MESSAGE_NO_LOG    = 0x0040, /**< Do not log.           */
-	GAIM_MESSAGE_WHISPER   = 0x0080  /**< Whispered message.    */
+	GAIM_MESSAGE_SEND      = 0x0001, /**< Outgoing message.        */
+	GAIM_MESSAGE_RECV      = 0x0002, /**< Incoming message.        */
+	GAIM_MESSAGE_SYSTEM    = 0x0004, /**< System message.          */
+	GAIM_MESSAGE_AUTO_RESP = 0x0008, /**< Auto response.           */
+	GAIM_MESSAGE_COLORIZE  = 0x0010, /**< Colorize nicks.          */
+	GAIM_MESSAGE_NICK      = 0x0020, /**< Contains your nick.      */
+	GAIM_MESSAGE_NO_LOG    = 0x0040, /**< Do not log.              */
+	GAIM_MESSAGE_WHISPER   = 0x0080, /**< Whispered message.       */
+	GAIM_MESSAGE_IMAGES    = 0x0100, /**< Message contains images. */
+	GAIM_MESSAGE_ERROR     = 0x0200  /**< Error message.           */
 } GaimMessageFlags;
 
 #include "account.h"
+#include "log.h"
+#include "buddyicon.h"
 #include "server.h"
 
 /**
  * Conversation window operations.
  *
- * Any UI representing a window must assign a filled-out gaim_window_ops
- * structure to the GaimWindow.
+ * Any UI representing a window must assign a filled-out gaim_conv_window_ops
+ * structure to the GaimConvWindow.
  */
-struct _GaimWindowUiOps
+struct _GaimConvWindowUiOps
 {
 	GaimConversationUiOps *(*get_conversation_ui_ops)(void);
 
-	void (*new_window)(GaimWindow *win);
-	void (*destroy_window)(GaimWindow *win);
+	void (*new_window)(GaimConvWindow *win);
+	void (*destroy_window)(GaimConvWindow *win);
 
-	void (*show)(GaimWindow *win);
-	void (*hide)(GaimWindow *win);
-	void (*raise)(GaimWindow *win);
-	void (*flash)(GaimWindow *win);
+	void (*show)(GaimConvWindow *win);
+	void (*hide)(GaimConvWindow *win);
+	void (*raise)(GaimConvWindow *win);
+	void (*flash)(GaimConvWindow *win);
 
-	void (*switch_conversation)(GaimWindow *win, unsigned int index);
-	void (*add_conversation)(GaimWindow *win, GaimConversation *conv);
-	void (*remove_conversation)(GaimWindow *win, GaimConversation *conv);
-	void (*move_conversation)(GaimWindow *win, GaimConversation *conv,
-							  unsigned int newIndex);
-	int (*get_active_index)(const GaimWindow *win);
+	void (*switch_conversation)(GaimConvWindow *win, unsigned int index);
+	void (*add_conversation)(GaimConvWindow *win, GaimConversation *conv);
+	void (*remove_conversation)(GaimConvWindow *win, GaimConversation *conv);
+	void (*move_conversation)(GaimConvWindow *win, GaimConversation *conv,
+	                          unsigned int newIndex);
+	int (*get_active_index)(const GaimConvWindow *win);
 };
 
 /**
@@ -151,21 +160,22 @@ struct _GaimConversationUiOps
 {
 	void (*destroy_conversation)(GaimConversation *conv);
 	void (*write_chat)(GaimConversation *conv, const char *who,
-					   const char *message, GaimMessageFlags flags, time_t mtime);
+	                   const char *message, GaimMessageFlags flags,
+	                   time_t mtime);
 	void (*write_im)(GaimConversation *conv, const char *who,
-					 const char *message, size_t len, GaimMessageFlags flags, time_t mtime);
+	                 const char *message, GaimMessageFlags flags,
+	                 time_t mtime);
 	void (*write_conv)(GaimConversation *conv, const char *who,
-					   const char *message, size_t length, GaimMessageFlags flags,
-					   time_t mtime);
+	                   const char *message, GaimMessageFlags flags,
+	                   time_t mtime);
 
 	void (*chat_add_user)(GaimConversation *conv, const char *user);
 	void (*chat_add_users)(GaimConversation *conv, GList *users);
 	void (*chat_rename_user)(GaimConversation *conv,
-							 const char *old_name, const char *new_name);
+	                         const char *old_name, const char *new_name);
 	void (*chat_remove_user)(GaimConversation *conv, const char *user);
 	void (*chat_remove_users)(GaimConversation *conv, GList *users);
 
-	void (*set_title)(GaimConversation *conv, const char *title);
 	void (*update_progress)(GaimConversation *conv, float percent);
 
 	/* Events */
@@ -176,19 +186,19 @@ struct _GaimConversationUiOps
  * A core representation of a graphical window containing one or more
  * conversations.
  */
-struct _GaimWindow
+struct _GaimConvWindow
 {
 	GList *conversations;              /**< The conversations in the window. */
 	size_t conversation_count;         /**< The number of conversations.     */
 
-	GaimWindowUiOps *ui_ops;           /**< UI-specific window operations.   */
+	GaimConvWindowUiOps *ui_ops;       /**< UI-specific window operations.   */
 	void *ui_data;                     /**< UI-specific data.                */
 };
 
 /**
  * Data specific to Instant Messages.
  */
-struct _GaimIm
+struct _GaimConvIm
 {
 	GaimConversation *conv;            /**< The parent conversation.     */
 
@@ -198,12 +208,14 @@ struct _GaimIm
 	guint  type_again_timeout;         /**< The type again timer handle. */
 
 	GSList *images;                    /**< A list of images in the IM.  */
+
+	GaimBuddyIcon *icon;               /**< The buddy icon.              */
 };
 
 /**
  * Data specific to Chats.
  */
-struct _GaimChat
+struct _GaimConvChat
 {
 	GaimConversation *conv;          /**< The parent conversation.      */
 
@@ -212,20 +224,23 @@ struct _GaimChat
 	char  *who;                      /**< The person who set the topic. */
 	char  *topic;                    /**< The topic.                    */
 	int    id;                       /**< The chat ID.                  */
+	char *nick;                      /**< Your nick in this chat.       */
+
+	gboolean left;                   /**< We left the chat and kept the window open */
 };
 
 /**
  * A core representation of a conversation between two or more people.
  *
  * The conversation can be an IM or a chat. Each conversation is kept
- * in a GaimWindow and has a UI representation.
+ * in a GaimConvWindow and has a UI representation.
  */
 struct _GaimConversation
 {
 	GaimConversationType type;  /**< The type of conversation.          */
 
 	GaimAccount *account;       /**< The user using this conversation.  */
-	GaimWindow *window;         /**< The parent window.                 */
+	GaimConvWindow *window;     /**< The parent window.                 */
 
 	int conversation_pos;       /**< The position in the window's list. */
 
@@ -234,6 +249,8 @@ struct _GaimConversation
 
 	gboolean logging;           /**< The status of logging.             */
 
+	GaimLog *log;               /**< This conversation's log            */
+
 	GList *send_history;        /**< The send history.                  */
 	GString *history;           /**< The conversation history.          */
 
@@ -241,8 +258,8 @@ struct _GaimConversation
 
 	union
 	{
-		GaimIm   *im;           /**< IM-specific data.                  */
-		GaimChat *chat;         /**< Chat-specific data.                */
+		GaimConvIm   *im;       /**< IM-specific data.                  */
+		GaimConvChat *chat;     /**< Chat-specific data.                */
 		void *misc;             /**< Misc. data.                        */
 
 	} u;
@@ -268,46 +285,46 @@ extern "C" {
  * Creates a new conversation window.
  *
  * This window is added to the list of windows, but is not shown until
- * gaim_window_show() is called.
+ * gaim_conv_window_show() is called.
  *
  * @return The new conversation window.
  */
-GaimWindow *gaim_window_new(void);
+GaimConvWindow *gaim_conv_window_new(void);
 
 /**
  * Destroys the specified conversation window and all conversations in it.
  *
  * @param win The window to destroy.
  */
-void gaim_window_destroy(GaimWindow *win);
+void gaim_conv_window_destroy(GaimConvWindow *win);
 
 /**
  * Shows the specified conversation window.
  *
  * @param win The window.
  */
-void gaim_window_show(GaimWindow *win);
+void gaim_conv_window_show(GaimConvWindow *win);
 
 /**
  * Hides the specified conversation window.
  *
  * @param win The window.
  */
-void gaim_window_hide(GaimWindow *win);
+void gaim_conv_window_hide(GaimConvWindow *win);
 
 /**
  * Raises the specified conversation window.
  *
  * @param win The window.
  */
-void gaim_window_raise(GaimWindow *win);
+void gaim_conv_window_raise(GaimConvWindow *win);
 
 /**
  * Causes the window to flash for IM notification, if the UI supports this.
  *
  * @param win The window.
  */
-void gaim_window_flash(GaimWindow *win);
+void gaim_conv_window_flash(GaimConvWindow *win);
 
 /**
  * Sets the specified window's UI window operations structure.
@@ -315,7 +332,8 @@ void gaim_window_flash(GaimWindow *win);
  * @param win The window.
  * @param ops The UI window operations structure.
  */
-void gaim_window_set_ui_ops(GaimWindow *win, GaimWindowUiOps *ops);
+void gaim_conv_window_set_ui_ops(GaimConvWindow *win,
+								 GaimConvWindowUiOps *ops);
 
 /**
  * Returns the specified window's UI window operations structure.
@@ -324,7 +342,7 @@ void gaim_window_set_ui_ops(GaimWindow *win, GaimWindowUiOps *ops);
  *
  * @return The UI window operations structure.
  */
-GaimWindowUiOps *gaim_window_get_ui_ops(const GaimWindow *win);
+GaimConvWindowUiOps *gaim_conv_window_get_ui_ops(const GaimConvWindow *win);
 
 /**
  * Adds a conversation to this window.
@@ -336,7 +354,8 @@ GaimWindowUiOps *gaim_window_get_ui_ops(const GaimWindow *win);
  *
  * @return The new index of the conversation in the window.
  */
-int gaim_window_add_conversation(GaimWindow *win, GaimConversation *conv);
+int gaim_conv_window_add_conversation(GaimConvWindow *win,
+									  GaimConversation *conv);
 
 /**
  * Removes the conversation at the specified index from the window.
@@ -348,8 +367,8 @@ int gaim_window_add_conversation(GaimWindow *win, GaimConversation *conv);
  *
  * @return The conversation removed.
  */
-GaimConversation *gaim_window_remove_conversation(GaimWindow *win,
-												  unsigned int index);
+GaimConversation *gaim_conv_window_remove_conversation(GaimConvWindow *win,
+													   unsigned int index);
 
 /**
  * Moves the conversation at the specified index in a window to a new index.
@@ -358,8 +377,9 @@ GaimConversation *gaim_window_remove_conversation(GaimWindow *win,
  * @param index     The index of the conversation to move.
  * @param new_index The new index.
  */
-void gaim_window_move_conversation(GaimWindow *win, unsigned int index,
-								   unsigned int new_index);
+void gaim_conv_window_move_conversation(GaimConvWindow *win,
+										unsigned int index,
+										unsigned int new_index);
 
 /**
  * Returns the conversation in the window at the specified index.
@@ -371,8 +391,8 @@ void gaim_window_move_conversation(GaimWindow *win, unsigned int index,
  *
  * @return The conversation at the specified index.
  */
-GaimConversation *gaim_window_get_conversation_at(const GaimWindow *win,
-												  unsigned int index);
+GaimConversation *gaim_conv_window_get_conversation_at(
+		const GaimConvWindow *win, unsigned int index);
 
 /**
  * Returns the number of conversations in the window.
@@ -381,7 +401,7 @@ GaimConversation *gaim_window_get_conversation_at(const GaimWindow *win,
  *
  * @return The number of conversations.
  */
-size_t gaim_window_get_conversation_count(const GaimWindow *win);
+size_t gaim_conv_window_get_conversation_count(const GaimConvWindow *win);
 
 /**
  * Switches the active conversation to the one at the specified index.
@@ -391,7 +411,8 @@ size_t gaim_window_get_conversation_count(const GaimWindow *win);
  * @param win   The window.
  * @param index The new index.
  */
-void gaim_window_switch_conversation(GaimWindow *win, unsigned int index);
+void gaim_conv_window_switch_conversation(GaimConvWindow *win,
+										  unsigned int index);
 
 /**
  * Returns the active conversation in the window.
@@ -400,7 +421,8 @@ void gaim_window_switch_conversation(GaimWindow *win, unsigned int index);
  *
  * @return The active conversation.
  */
-GaimConversation *gaim_window_get_active_conversation(const GaimWindow *win);
+GaimConversation *gaim_conv_window_get_active_conversation(
+		const GaimConvWindow *win);
 
 /**
  * Returns the list of conversations in the specified window.
@@ -409,7 +431,7 @@ GaimConversation *gaim_window_get_active_conversation(const GaimWindow *win);
  *
  * @return The list of conversations.
  */
-GList *gaim_window_get_conversations(const GaimWindow *win);
+GList *gaim_conv_window_get_conversations(const GaimConvWindow *win);
 
 /**
  * Returns a list of all windows.
@@ -425,7 +447,7 @@ GList *gaim_get_windows(void);
  *
  * @return The window if found, or @c NULL if not found.
  */
-GaimWindow *gaim_get_first_window_with_type(GaimConversationType type);
+GaimConvWindow *gaim_get_first_window_with_type(GaimConversationType type);
 
 /**
  * Returns the last window containing a conversation of the specified type.
@@ -434,7 +456,7 @@ GaimWindow *gaim_get_first_window_with_type(GaimConversationType type);
  *
  * @return The window if found, or @c NULL if not found.
  */
-GaimWindow *gaim_get_last_window_with_type(GaimConversationType type);
+GaimConvWindow *gaim_get_last_window_with_type(GaimConversationType type);
 
 /*@}*/
 
@@ -446,10 +468,10 @@ GaimWindow *gaim_get_last_window_with_type(GaimConversationType type);
 /**
  * Creates a new conversation of the specified type.
  *
- * @param type The type of conversation.
- * @param user The account opening the conversation window on the gaim
- *             user's end.
- * @param name The name of the conversation.
+ * @param type    The type of conversation.
+ * @param account The account opening the conversation window on the gaim
+ *                user's end.
+ * @param name    The name of the conversation.
  *
  * @return The new conversation.
  */
@@ -542,7 +564,7 @@ void gaim_conversation_set_title(GaimConversation *conv, const char *title);
 /**
  * Returns the specified conversation's title.
  *
- * @param win The conversation.
+ * @param conv The conversation.
  *
  * @return The title.
  */
@@ -581,10 +603,18 @@ void gaim_conversation_set_unseen(GaimConversation *conv,
  *
  * @param conv The conversation.
  *
- * @param The conversation's unseen state.
+ * @return The conversation's unseen state.
  */
 GaimUnseenState gaim_conversation_get_unseen(const GaimConversation *conv);
 
+/**
+ * Sets the specified conversation's name.
+ *
+ * @param conv The conversation.
+ * @param name The conversation's name.
+ */
+void gaim_conversation_set_name(GaimConversation *conv, const char *name);
+
 /**
  * Returns the specified conversation's name.
  *
@@ -597,13 +627,16 @@ const char *gaim_conversation_get_name(const GaimConversation *conv);
 /**
  * Enables or disables logging for this conversation.
  *
- * @param log @c TRUE if logging should be enabled, or @c FALSE otherwise.
+ * @param conv The conversation.
+ * @param log  @c TRUE if logging should be enabled, or @c FALSE otherwise.
  */
 void gaim_conversation_set_logging(GaimConversation *conv, gboolean log);
 
 /**
  * Returns whether or not logging is enabled for this conversation.
  *
+ * @param conv The conversation.
+ *
  * @return @c TRUE if logging is enabled, or @c FALSE otherwise.
  */
 gboolean gaim_conversation_is_logging(const GaimConversation *conv);
@@ -641,7 +674,7 @@ GString *gaim_conversation_get_history(const GaimConversation *conv);
  *
  * @return The conversation's parent window.
  */
-GaimWindow *gaim_conversation_get_window(const GaimConversation *conv);
+GaimConvWindow *gaim_conversation_get_window(const GaimConversation *conv);
 
 /**
  * Returns the specified conversation's IM-specific data.
@@ -652,9 +685,9 @@ GaimWindow *gaim_conversation_get_window(const GaimConversation *conv);
  *
  * @return The IM-specific data.
  */
-GaimIm *gaim_conversation_get_im_data(const GaimConversation *conv);
+GaimConvIm *gaim_conversation_get_im_data(const GaimConversation *conv);
 
-#define GAIM_IM(c) (gaim_conversation_get_im_data(c))
+#define GAIM_CONV_IM(c) (gaim_conversation_get_im_data(c))
 
 /**
  * Returns the specified conversation's chat-specific data.
@@ -665,9 +698,9 @@ GaimIm *gaim_conversation_get_im_data(const GaimConversation *conv);
  *
  * @return The chat-specific data.
  */
-GaimChat *gaim_conversation_get_chat_data(const GaimConversation *conv);
+GaimConvChat *gaim_conversation_get_chat_data(const GaimConversation *conv);
 
-#define GAIM_CHAT(c) (gaim_conversation_get_chat_data(c))
+#define GAIM_CONV_CHAT(c) (gaim_conversation_get_chat_data(c))
 
 /**
  * Sets extra data for a conversation.
@@ -736,7 +769,7 @@ GaimConversation *gaim_find_conversation_with_account(
  * Writes to a conversation window.
  *
  * This function should not be used to write IM or chat messages. Use
- * gaim_im_write() and gaim_chat_write() instead. Those functions will
+ * gaim_conv_im_write() and gaim_conv_chat_write() instead. Those functions will
  * most likely call this anyway, but they may do their own formatting,
  * sound playback, etc.
  *
@@ -746,15 +779,14 @@ GaimConversation *gaim_find_conversation_with_account(
  * @param conv    The conversation.
  * @param who     The user who sent the message.
  * @param message The message.
- * @param length  The length of the message.
  * @param flags   The message flags.
  * @param mtime   The time the message was sent.
  *
- * @see gaim_im_write()
- * @see gaim_chat_write()
+ * @see gaim_conv_im_write()
+ * @see gaim_conv_chat_write()
  */
 void gaim_conversation_write(GaimConversation *conv, const char *who,
-							 const char *message, size_t length, GaimMessageFlags flags,
+							 const char *message, GaimMessageFlags flags,
 							 time_t mtime);
 
 /**
@@ -798,7 +830,29 @@ void gaim_conversation_foreach(void (*func)(GaimConversation *conv));
  *
  * @return The parent conversation.
  */
-GaimConversation *gaim_im_get_conversation(const GaimIm *im);
+GaimConversation *gaim_conv_im_get_conversation(const GaimConvIm *im);
+
+/**
+ * Sets the IM's buddy icon.
+ *
+ * This should only be called from within Gaim. You probably want to
+ * call gaim_buddy_icon_set_data().
+ *
+ * @param im   The IM.
+ * @param icon The buddy icon.
+ *
+ * @see gaim_buddy_icon_set_data()
+ */
+void gaim_conv_im_set_icon(GaimConvIm *im, GaimBuddyIcon *icon);
+
+/**
+ * Returns the IM's buddy icon.
+ *
+ * @param im The IM.
+ *
+ * @return The buddy icon.
+ */
+GaimBuddyIcon *gaim_conv_im_get_icon(const GaimConvIm *im);
 
 /**
  * Sets the IM's typing state.
@@ -806,7 +860,7 @@ GaimConversation *gaim_im_get_conversation(const GaimIm *im);
  * @param im    The IM.
  * @param state The typing state.
  */
-void gaim_im_set_typing_state(GaimIm *im, int state);
+void gaim_conv_im_set_typing_state(GaimConvIm *im, GaimTypingState state);
 
 /**
  * Returns the IM's typing state.
@@ -815,7 +869,7 @@ void gaim_im_set_typing_state(GaimIm *im, int state);
  *
  * @return The IM's typing state.
  */
-int gaim_im_get_typing_state(const GaimIm *im);
+GaimTypingState gaim_conv_im_get_typing_state(const GaimConvIm *im);
 
 /**
  * Starts the IM's typing timeout.
@@ -823,14 +877,14 @@ int gaim_im_get_typing_state(const GaimIm *im);
  * @param im      The IM.
  * @param timeout The timeout.
  */
-void gaim_im_start_typing_timeout(GaimIm *im, int timeout);
+void gaim_conv_im_start_typing_timeout(GaimConvIm *im, int timeout);
 
 /**
  * Stops the IM's typing timeout.
  *
  * @param im The IM.
  */
-void gaim_im_stop_typing_timeout(GaimIm *im);
+void gaim_conv_im_stop_typing_timeout(GaimConvIm *im);
 
 /**
  * Returns the IM's typing timeout.
@@ -839,7 +893,7 @@ void gaim_im_stop_typing_timeout(GaimIm *im);
  *
  * @return The timeout.
  */
-guint gaim_im_get_typing_timeout(const GaimIm *im);
+guint gaim_conv_im_get_typing_timeout(const GaimConvIm *im);
 
 /**
  * Sets the IM's time until it should send another typing notification.
@@ -847,7 +901,7 @@ guint gaim_im_get_typing_timeout(const GaimIm *im);
  * @param im  The IM.
  * @param val The time.
  */
-void gaim_im_set_type_again(GaimIm *im, time_t val);
+void gaim_conv_im_set_type_again(GaimConvIm *im, time_t val);
 
 /**
  * Returns the IM's time until it should send another typing notification.
@@ -856,21 +910,21 @@ void gaim_im_set_type_again(GaimIm *im, time_t val);
  *
  * @return The time.
  */
-time_t gaim_im_get_type_again(const GaimIm *im);
+time_t gaim_conv_im_get_type_again(const GaimConvIm *im);
 
 /**
  * Starts the IM's type again timeout.
  *
  * @param im      The IM.
  */
-void gaim_im_start_type_again_timeout(GaimIm *im);
+void gaim_conv_im_start_type_again_timeout(GaimConvIm *im);
 
 /**
  * Stops the IM's type again timeout.
  *
  * @param im The IM.
  */
-void gaim_im_stop_type_again_timeout(GaimIm *im);
+void gaim_conv_im_stop_type_again_timeout(GaimConvIm *im);
 
 /**
  * Returns the IM's type again timeout interval.
@@ -879,31 +933,27 @@ void gaim_im_stop_type_again_timeout(GaimIm *im);
  *
  * @return The type again timeout interval.
  */
-guint gaim_im_get_type_again_timeout(const GaimIm *im);
+guint gaim_conv_im_get_type_again_timeout(const GaimConvIm *im);
 
 /**
  * Updates the visual typing notification for an IM conversation.
  *
  * @param im The IM.
  */
-void gaim_im_update_typing(GaimIm *im);
+void gaim_conv_im_update_typing(GaimConvIm *im);
 
 /**
  * Writes to an IM.
  *
- * The @a len parameter is used for writing binary data, such as an
- * image. If @c message is text, specify -1 for @a len.
- *
  * @param im      The IM.
  * @param who     The user who sent the message.
  * @param message The message to write.
- * @param len     The length of the message, or -1 to specify the length
- *                of @a message.
  * @param flags   The message flags.
  * @param mtime   The time the message was sent.
  */
-void gaim_im_write(GaimIm *im, const char *who,
-				   const char *message, size_t len, GaimMessageFlags flags, time_t mtime);
+void gaim_conv_im_write(GaimConvIm *im, const char *who,
+						const char *message, GaimMessageFlags flags,
+						time_t mtime);
 
 /**
  * Sends a message to this IM conversation.
@@ -911,7 +961,7 @@ void gaim_im_write(GaimIm *im, const char *who,
  * @param im      The IM.
  * @param message The message to send.
  */
-void gaim_im_send(GaimIm *im, const char *message);
+void gaim_conv_im_send(GaimConvIm *im, const char *message);
 
 /*@}*/
 
@@ -928,21 +978,21 @@ void gaim_im_send(GaimIm *im, const char *message);
  *
  * @return The parent conversation.
  */
-GaimConversation *gaim_chat_get_conversation(const GaimChat *chat);
+GaimConversation *gaim_conv_chat_get_conversation(const GaimConvChat *chat);
 
 /**
  * Sets the list of users in the chat room.
  *
  * @note Calling this function will not update the display of the users.
- *       Please use gaim_chat_add_user(), gaim_chat_add_users(),
- *       gaim_chat_remove_user(), and gaim_chat_remove_users() instead.
+ *       Please use gaim_conv_chat_add_user(), gaim_conv_chat_add_users(),
+ *       gaim_conv_chat_remove_user(), and gaim_conv_chat_remove_users() instead.
  *
  * @param chat  The chat.
  * @param users The list of users.
  *
  * @return The list passed.
  */
-GList *gaim_chat_set_users(GaimChat *chat, GList *users);
+GList *gaim_conv_chat_set_users(GaimConvChat *chat, GList *users);
 
 /**
  * Returns a list of users in the chat room.
@@ -951,7 +1001,7 @@ GList *gaim_chat_set_users(GaimChat *chat, GList *users);
  *
  * @return The list of users.
  */
-GList *gaim_chat_get_users(const GaimChat *chat);
+GList *gaim_conv_chat_get_users(const GaimConvChat *chat);
 
 /**
  * Ignores a user in a chat room.
@@ -959,7 +1009,7 @@ GList *gaim_chat_get_users(const GaimChat *chat);
  * @param chat The chat.
  * @param name The name of the user.
  */
-void gaim_chat_ignore(GaimChat *chat, const char *name);
+void gaim_conv_chat_ignore(GaimConvChat *chat, const char *name);
 
 /**
  * Unignores a user in a chat room.
@@ -967,7 +1017,7 @@ void gaim_chat_ignore(GaimChat *chat, const char *name);
  * @param chat The chat.
  * @param name The name of the user.
  */
-void gaim_chat_unignore(GaimChat *chat, const char *name);
+void gaim_conv_chat_unignore(GaimConvChat *chat, const char *name);
 
 /**
  * Sets the list of ignored users in the chat room.
@@ -977,7 +1027,7 @@ void gaim_chat_unignore(GaimChat *chat, const char *name);
  *
  * @return The list passed.
  */
-GList *gaim_chat_set_ignored(GaimChat *chat, GList *ignored);
+GList *gaim_conv_chat_set_ignored(GaimConvChat *chat, GList *ignored);
 
 /**
  * Returns the list of ignored users in the chat room.
@@ -986,7 +1036,7 @@ GList *gaim_chat_set_ignored(GaimChat *chat, GList *ignored);
  *
  * @return The list of ignored users.
  */
-GList *gaim_chat_get_ignored(const GaimChat *chat);
+GList *gaim_conv_chat_get_ignored(const GaimConvChat *chat);
 
 /**
  * Returns the actual name of the specified ignored user, if it exists in
@@ -1002,8 +1052,8 @@ GList *gaim_chat_get_ignored(const GaimChat *chat);
  * @return The ignored user if found, complete with prefixes, or @c NULL
  *         if not found.
  */
-const char *gaim_chat_get_ignored_user(const GaimChat *chat,
-									   const char *user);
+const char *gaim_conv_chat_get_ignored_user(const GaimConvChat *chat,
+											const char *user);
 
 /**
  * Returns @c TRUE if the specified user is ignored.
@@ -1013,8 +1063,8 @@ const char *gaim_chat_get_ignored_user(const GaimChat *chat,
  *
  * @return @c TRUE if the user is in the ignore list; @c FALSE otherwise.
  */
-gboolean gaim_chat_is_user_ignored(const GaimChat *chat,
-								   const char *user);
+gboolean gaim_conv_chat_is_user_ignored(const GaimConvChat *chat,
+										const char *user);
 
 /**
  * Sets the chat room's topic.
@@ -1023,8 +1073,8 @@ gboolean gaim_chat_is_user_ignored(const GaimChat *chat,
  * @param who   The user that set the topic.
  * @param topic The topic.
  */
-void gaim_chat_set_topic(GaimChat *chat, const char *who,
-						 const char *topic);
+void gaim_conv_chat_set_topic(GaimConvChat *chat, const char *who,
+							  const char *topic);
 
 /**
  * Returns the chat room's topic.
@@ -1033,7 +1083,7 @@ void gaim_chat_set_topic(GaimChat *chat, const char *who,
  *
  * @return The chat's topic.
  */
-const char *gaim_chat_get_topic(const GaimChat *chat);
+const char *gaim_conv_chat_get_topic(const GaimConvChat *chat);
 
 /**
  * Sets the chat room's ID.
@@ -1041,7 +1091,7 @@ const char *gaim_chat_get_topic(const GaimChat *chat);
  * @param chat The chat.
  * @param id   The ID.
  */
-void gaim_chat_set_id(GaimChat *chat, int id);
+void gaim_conv_chat_set_id(GaimConvChat *chat, int id);
 
 /**
  * Returns the chat room's ID.
@@ -1050,7 +1100,7 @@ void gaim_chat_set_id(GaimChat *chat, int id);
  *
  * @return The ID.
  */
-int gaim_chat_get_id(const GaimChat *chat);
+int gaim_conv_chat_get_id(const GaimConvChat *chat);
 
 /**
  * Writes to a chat.
@@ -1061,8 +1111,9 @@ int gaim_chat_get_id(const GaimChat *chat);
  * @param flags   The flags.
  * @param mtime   The time the message was sent.
  */
-void gaim_chat_write(GaimChat *chat, const char *who,
-					 const char *message, GaimMessageFlags flags, time_t mtime);
+void gaim_conv_chat_write(GaimConvChat *chat, const char *who,
+						  const char *message, GaimMessageFlags flags,
+						  time_t mtime);
 
 /**
  * Sends a message to this chat conversation.
@@ -1070,7 +1121,7 @@ void gaim_chat_write(GaimChat *chat, const char *who,
  * @param chat    The chat.
  * @param message The message to send.
  */
-void gaim_chat_send(GaimChat *chat, const char *message);
+void gaim_conv_chat_send(GaimConvChat *chat, const char *message);
 
 /**
  * Adds a user to a chat.
@@ -1079,8 +1130,8 @@ void gaim_chat_send(GaimChat *chat, const char *message);
  * @param user      The user to add.
  * @param extra_msg An extra message to display with the join message.
  */
-void gaim_chat_add_user(GaimChat *chat, const char *user,
-						const char *extra_msg);
+void gaim_conv_chat_add_user(GaimConvChat *chat, const char *user,
+							 const char *extra_msg);
 
 /**
  * Adds a list of users to a chat.
@@ -1091,7 +1142,7 @@ void gaim_chat_add_user(GaimChat *chat, const char *user,
  * @param chat      The chat.
  * @param users     The list of users to add.
  */
-void gaim_chat_add_users(GaimChat *chat, GList *users);
+void gaim_conv_chat_add_users(GaimConvChat *chat, GList *users);
 
 /**
  * Renames a user in a chat.
@@ -1100,8 +1151,8 @@ void gaim_chat_add_users(GaimChat *chat, GList *users);
  * @param old_user The old username.
  * @param new_user The new username.
  */
-void gaim_chat_rename_user(GaimChat *chat, const char *old_user,
-						   const char *new_user);
+void gaim_conv_chat_rename_user(GaimConvChat *chat, const char *old_user,
+								const char *new_user);
 
 /**
  * Removes a user from a chat, optionally with a reason.
@@ -1112,8 +1163,8 @@ void gaim_chat_rename_user(GaimChat *chat, const char *old_user,
  * @param user   The user that is being removed.
  * @param reason The optional reason given for the removal. Can be @c NULL.
  */
-void gaim_chat_remove_user(GaimChat *chat, const char *user,
-						   const char *reason);
+void gaim_conv_chat_remove_user(GaimConvChat *chat, const char *user,
+								const char *reason);
 
 /**
  * Removes a list of users from a chat, optionally with a single reason.
@@ -1122,14 +1173,31 @@ void gaim_chat_remove_user(GaimChat *chat, const char *user,
  * @param users  The users that are being removed.
  * @param reason The optional reason given for the removal. Can be @c NULL.
  */
-void gaim_chat_remove_users(GaimChat *chat, GList *users, const char *reason);
+void gaim_conv_chat_remove_users(GaimConvChat *chat, GList *users,
+								 const char *reason);
 
 /**
  * Clears all users from a chat.
  *
  * @param chat The chat.
  */
-void gaim_chat_clear_users(GaimChat *chat);
+void gaim_conv_chat_clear_users(GaimConvChat *chat);
+
+/**
+ * Sets your nickname (used for hilighting) for a chat.
+ *
+ * @param chat The chat.
+ * @param nick The nick.
+ */
+void gaim_conv_chat_set_nick(GaimConvChat *chat, const char *nick);
+
+/**
+ * Gets your nickname (used for hilighting) for a chat.
+ *
+ * @param chat The chat.
+ * @return  The nick.
+ */
+const char *gaim_conv_chat_get_nick(GaimConvChat *chat);
 
 /**
  * Finds a chat with the specified chat ID.
@@ -1141,10 +1209,29 @@ void gaim_chat_clear_users(GaimChat *chat);
  */
 GaimConversation *gaim_find_chat(const GaimConnection *gc, int id);
 
+/**
+ * Lets the core know we left a chat, without destroying it.
+ * Called from serv_got_chat_left().
+ *
+ * @param chat The chat.
+ */
+void gaim_conv_chat_left(GaimConvChat *chat);
+
+/**
+ * Returns true if we're no longer in this chat,
+ * and just left the window open.
+ *
+ * @param chat The chat.
+ *
+ * @return @c TRUE if we left the chat already, @c FALSE if
+ * we're still there.
+ */
+gboolean gaim_conv_chat_has_left(GaimConvChat *chat);
+
 /*@}*/
 
 /**************************************************************************/
-/** @name Conversation Placement Functions                                */
+/** @name Conversation Placement API                                      */
 /**************************************************************************/
 /*@{*/
 
@@ -1159,6 +1246,7 @@ GList *gaim_conv_placement_get_options(void);
 /**
  * Adds a conversation placement function to the list of possible functions.
  *
+ * @param id   The unique ID of the placement function.
  * @param name The name of the function.
  * @param fnc  A pointer to the function.
  */
@@ -1193,6 +1281,20 @@ const char *gaim_conv_placement_get_name(const char *id);
  */
 GaimConvPlacementFunc gaim_conv_placement_get_fnc(const char *id);
 
+/**
+ * Sets the current conversation placement function.
+ *
+ * @param func The new conversation placement function.
+ */
+void gaim_conv_placement_set_current_func(GaimConvPlacementFunc func);
+
+/**
+ * Returns the current conversation placement function.
+ *
+ * @return The current conversation placement function.
+ */
+GaimConvPlacementFunc gaim_conv_placement_get_current_func(void);
+
 /**
  * Returns the id of the specified conversation placement function.
  *
@@ -1216,15 +1318,15 @@ const char *gaim_conv_placement_get_fnc_id(GaimConvPlacementFunc fnc);
  *
  * @param ops The UI operations structure.
  */
-void gaim_set_win_ui_ops(GaimWindowUiOps *ops);
+void gaim_conversations_set_win_ui_ops(GaimConvWindowUiOps *ops);
 
 /**
  * Returns the gaim window UI operations structure to be used in
  * new windows.
  *
- * @return A filled-out GaimWindowUiOps structure.
+ * @return A filled-out GaimConvWindowUiOps structure.
  */
-GaimWindowUiOps *gaim_get_win_ui_ops(void);
+GaimConvWindowUiOps *gaim_conversations_get_win_ui_ops(void);
 
 /*@}*/
 
