@@ -4,7 +4,9 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +24,8 @@
  */
 #include "gtkinternal.h"
 
+#include <gdk/gdkkeysyms.h>
+
 #include "connection.h"
 #include "debug.h"
 #include "prefs.h"
@@ -64,6 +68,12 @@ email_response_cb(GtkDialog *dialog, gint id, GaimNotifyMailData *data)
 	gaim_notify_close(GAIM_NOTIFY_EMAILS, data);
 }
 
+static void
+formatted_close_cb(GtkWidget *win, GdkEvent *event, void *user_data)
+{
+	gaim_notify_close(GAIM_NOTIFY_FORMATTED, win);
+}
+
 static void *
 gaim_gtk_notify_message(GaimNotifyMsgType type, const char *title,
 						const char *primary, const char *secondary,
@@ -76,7 +86,8 @@ gaim_gtk_notify_message(GaimNotifyMsgType type, const char *title,
 	char label_text[2048];
 	const char *icon_name = NULL;
 
-	switch (type) {
+	switch (type)
+	{
 		case GAIM_NOTIFY_MSG_ERROR:
 			icon_name = GAIM_STOCK_DIALOG_ERROR;
 			break;
@@ -94,14 +105,18 @@ gaim_gtk_notify_message(GaimNotifyMsgType type, const char *title,
 			break;
 	}
 
-	if (icon_name != NULL) {
+	if (icon_name != NULL)
+	{
 		img = gtk_image_new_from_stock(icon_name, GTK_ICON_SIZE_DIALOG);
 		gtk_misc_set_alignment(GTK_MISC(img), 0, 0);
 	}
 
-	dialog = gtk_dialog_new_with_buttons(title, NULL, 0,
-										 GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-										 NULL);
+	dialog = gtk_dialog_new_with_buttons(title ? title : GAIM_ALERT_TITLE,
+					     NULL, 0, GTK_STOCK_OK,
+					     GTK_RESPONSE_ACCEPT, NULL);
+
+	gtk_window_set_role(GTK_WINDOW(dialog), "notify_dialog");
+
 	g_signal_connect(G_OBJECT(dialog), "response",
 					 G_CALLBACK(message_response_cb), dialog);
 
@@ -199,21 +214,31 @@ gaim_gtk_notify_emails(size_t count, gboolean detailed,
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
 	/* Descriptive label */
-	detail_text = g_strdup_printf(
-		ngettext("%s has %d new message.", "%s has %d new messages.", count),
-		*tos, count);
+	detail_text = g_strdup_printf(ngettext("%s has %d new message.",
+										   "%s has %d new messages.",
+										   (int)count),
+								  *tos, (int)count);
 
-	if (count == 1) {
+	if (count == 1)
+	{
 		char *from_text = NULL, *subject_text = NULL;
 
-		if (froms != NULL) {
+		if (froms != NULL)
+		{
+			char *from_enc;
+			from_enc = g_markup_escape_text(*froms, -1);
 			from_text = g_strdup_printf(
-				_("<span weight=\"bold\">From:</span> %s\n"), *froms);
+				_("<span weight=\"bold\">From:</span> %s\n"), from_enc);
+			g_free(from_enc);
 		}
 
-		if (subjects != NULL) {
+		if (subjects != NULL)
+		{
+			char *subject_enc;
+			subject_enc = g_markup_escape_text(*subjects, -1);
 			subject_text = g_strdup_printf(
-				_("<span weight=\"bold\">Subject:</span> %s\n"), *subjects);
+				_("<span weight=\"bold\">Subject:</span> %s\n"), subject_enc);
+			g_free(subject_enc);
 		}
 
 		label_text = g_strdup_printf(
@@ -230,7 +255,8 @@ gaim_gtk_notify_emails(size_t count, gboolean detailed,
 		if (subject_text != NULL)
 			g_free(subject_text);
 	}
-	else {
+	else
+	{
 		label_text = g_strdup_printf(
 			_("<span weight=\"bold\" size=\"larger\">You have mail!</span>"
 			"\n\n%s"), detail_text);
@@ -253,6 +279,19 @@ gaim_gtk_notify_emails(size_t count, gboolean detailed,
 	return data;
 }
 
+static gboolean
+formatted_input_cb(GtkWidget *dialog, GdkEventKey *event, gpointer data)
+{
+	if (event->keyval == GDK_Escape)
+	{
+		gtk_widget_destroy(dialog);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void *
 gaim_gtk_notify_formatted(const char *title, const char *primary,
 						  const char *secondary, const char *text,
@@ -266,11 +305,16 @@ gaim_gtk_notify_formatted(const char *title, const char *primary,
 	GtkWidget *sw;
 	int options = 0;
 	char label_text[2048];
+	char *linked_text;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_title(GTK_WINDOW(window), title);
 	gtk_window_set_type_hint(GTK_WINDOW(window), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_container_set_border_width(GTK_CONTAINER(window), 12);
 
+	g_signal_connect(G_OBJECT(window), "delete_event",
+					 G_CALLBACK(formatted_close_cb), NULL);
+
 	/* Setup the main vbox */
 	vbox = gtk_vbox_new(FALSE, 12);
 	gtk_container_add(GTK_CONTAINER(window), vbox);
@@ -278,8 +322,10 @@ gaim_gtk_notify_formatted(const char *title, const char *primary,
 
 	/* Setup the descriptive label */
 	g_snprintf(label_text, sizeof(label_text),
-		   "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-		   primary, (secondary ? secondary : ""));
+			   "<span weight=\"bold\" size=\"larger\">%s</span>%s%s",
+			   primary,
+			   (secondary ? "\n" : ""),
+			   (secondary ? secondary : ""));
 
 	label = gtk_label_new(NULL);
 
@@ -302,6 +348,7 @@ gaim_gtk_notify_formatted(const char *title, const char *primary,
 
 	/* Now build that gtkimhtml */
 	imhtml = gtk_imhtml_new(NULL, NULL);
+	gtk_widget_set_name(imhtml, "gaim_gtknotify_imhtml");
 	gtk_container_add(GTK_CONTAINER(sw), imhtml);
 	gtk_widget_show(imhtml);
 	gaim_setup_imhtml(imhtml);
@@ -311,25 +358,38 @@ gaim_gtk_notify_formatted(const char *title, const char *primary,
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
-	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(gtk_widget_destroy), window);
+	g_signal_connect_swapped(G_OBJECT(button), "clicked",
+							 G_CALLBACK(formatted_close_cb), window);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+					 G_CALLBACK(formatted_input_cb), NULL);
+
 
 	/* Add the text to the gtkimhtml */
+#if 1 /* PREFSLASH04 */
 	if (gaim_prefs_get_bool("/gaim/gtk/conversations/ignore_colors"))
 		options ^= GTK_IMHTML_NO_COLOURS;
-
 	if (gaim_prefs_get_bool("/gaim/gtk/conversations/ignore_fonts"))
 		options ^= GTK_IMHTML_NO_FONTS;
-
 	if (gaim_prefs_get_bool("/gaim/gtk/conversations/ignore_font_sizes"))
 		options ^= GTK_IMHTML_NO_SIZES;
+#else
+	if (gaim_prefs_get_bool("/gaim/gtk/conversations/ignore_formatting")) {
+		options ^= GTK_IMHTML_NO_COLOURS;
+		options ^= GTK_IMHTML_NO_FONTS;
+		options ^= GTK_IMHTML_NO_SIZES;
+	}
+#endif
 
 	options ^= GTK_IMHTML_NO_COMMENTS;
 	options ^= GTK_IMHTML_NO_TITLE;
 	options ^= GTK_IMHTML_NO_NEWLINE;
 	options ^= GTK_IMHTML_NO_SCROLL;
 
-	gtk_imhtml_append_text(GTK_IMHTML(imhtml), text, -1, options);
+	/* Make sure URLs are clickable */
+	linked_text = gaim_markup_linkify(text);
+	gtk_imhtml_append_text(GTK_IMHTML(imhtml), linked_text,
+									   options);
+	g_free(linked_text);
 
 	/* Show the window */
 	gtk_widget_show(window);
@@ -340,7 +400,8 @@ gaim_gtk_notify_formatted(const char *title, const char *primary,
 static void
 gaim_gtk_close_notify(GaimNotifyType type, void *ui_handle)
 {
-	if (type == GAIM_NOTIFY_EMAIL || type == GAIM_NOTIFY_EMAILS) {
+	if (type == GAIM_NOTIFY_EMAIL || type == GAIM_NOTIFY_EMAILS)
+	{
 		GaimNotifyMailData *data = (GaimNotifyMailData *)ui_handle;
 
 		gtk_widget_destroy(data->dialog);
@@ -352,6 +413,198 @@ gaim_gtk_close_notify(GaimNotifyType type, void *ui_handle)
 		gtk_widget_destroy(GTK_WIDGET(ui_handle));
 }
 
+#ifndef _WIN32
+static gint
+uri_command(const char *command, gboolean sync)
+{
+	GError *error = NULL;
+	gint ret = 0;
+
+	gaim_debug_misc("gtknotify", "Executing %s\n", command);
+
+	if (!gaim_program_is_valid(command))
+	{
+		gchar *tmp;
+
+		tmp = g_strdup_printf(_("The browser command \"%s\" is invalid."),
+							  command);
+
+		gaim_notify_error(NULL, NULL, _("Unable to open URL"), tmp);
+
+		g_free(tmp);
+
+	}
+	else if (sync)
+	{
+		gint status;
+
+		if (!g_spawn_command_line_sync(command, NULL, NULL, &status, &error))
+		{
+			char *tmp = g_strdup_printf(_("Error launching \"%s\": %s"),
+										command, error->message);
+
+			gaim_notify_error(NULL, NULL, _("Unable to open URL"), tmp);
+
+			g_free(tmp);
+			g_error_free(error);
+		}
+		else
+			ret = status;
+	}
+	else
+	{
+		if (!g_spawn_command_line_async(command, &error))
+		{
+			char *tmp = g_strdup_printf(_("Error launching \"%s\": %s"),
+										command, error->message);
+
+			gaim_notify_error(NULL, NULL, _("Unable to open URL"), tmp);
+
+			g_free(tmp);
+			g_error_free(error);
+		}
+	}
+
+	return ret;
+}
+#endif /* _WIN32 */
+
+static void *
+gaim_gtk_notify_uri(const char *uri)
+{
+#ifndef _WIN32
+	char *command = NULL;
+	char *remote_command = NULL;
+	const char *web_browser;
+	int place;
+
+	web_browser = gaim_prefs_get_string("/gaim/gtk/browsers/browser");
+	place = gaim_prefs_get_int("/gaim/gtk/browsers/place");
+
+	if (!strcmp(web_browser, "netscape"))
+	{
+		command = g_strdup_printf("netscape \"%s\"", uri);
+
+		if (place == GAIM_BROWSER_NEW_WINDOW)
+		{
+			remote_command = g_strdup_printf("netscape -remote "
+											 "\"openURL(\"%s\",new-window)\"",
+											 uri);
+		}
+		else if (place == GAIM_BROWSER_CURRENT)
+		{
+			remote_command = g_strdup_printf("netscape -remote "
+											 "\"openURL(\"%s\")\"", uri);
+		}
+	}
+	else if (!strcmp(web_browser, "opera"))
+	{
+		if (place == GAIM_BROWSER_NEW_WINDOW)
+			command = g_strdup_printf("opera -newwindow \"%s\"", uri);
+		else if (place == GAIM_BROWSER_NEW_TAB)
+			command = g_strdup_printf("opera -newpage \"%s\"", uri);
+		else if (place == GAIM_BROWSER_CURRENT)
+		{
+			remote_command = g_strdup_printf("opera -remote "
+											 "\"openURL(\"%s\")\"", uri);
+			command = g_strdup_printf("opera \"%s\"", uri);
+		}
+		else
+			command = g_strdup_printf("opera \"%s\"", uri);
+
+	}
+	else if (!strcmp(web_browser, "kfmclient"))
+	{
+		command = g_strdup_printf("kfmclient openURL \"%s\"", uri);
+		/*
+		 * Does Konqueror have options to open in new tab
+		 * and/or current window?
+		 */
+	}
+	else if (!strcmp(web_browser, "gnome-open"))
+	{
+		command = g_strdup_printf("gnome-open \"%s\"", uri);
+	}
+	else if (!strcmp(web_browser, "galeon"))
+	{
+		if (place == GAIM_BROWSER_NEW_WINDOW)
+			command = g_strdup_printf("galeon -w \"%s\"", uri);
+		else if (place == GAIM_BROWSER_NEW_TAB)
+			command = g_strdup_printf("galeon -n \"%s\"", uri);
+		else
+			command = g_strdup_printf("galeon \"%s\"", uri);
+	}
+	else if (!strcmp(web_browser, "mozilla") ||
+			 !strcmp(web_browser, "mozilla-firebird") ||
+			 !strcmp(web_browser, "firefox"))
+	{
+		command = g_strdup_printf("%s \"%s\"", web_browser, uri);
+
+		if (place == GAIM_BROWSER_NEW_WINDOW)
+		{
+			remote_command = g_strdup_printf("%s -remote \"openURL(\"%s\","
+											 "new-window)\"",
+											 web_browser, uri);
+		}
+		else if (place == GAIM_BROWSER_NEW_TAB)
+		{
+			remote_command = g_strdup_printf("%s -remote \"openURL(\"%s\","
+											 "new-tab)\"",
+											 web_browser, uri);
+		}
+		else if (place == GAIM_BROWSER_CURRENT)
+		{
+			remote_command = g_strdup_printf("%s -remote \"openURL(\"%s\")\"",
+											 web_browser, uri);
+		}
+	}
+	else if (!strcmp(web_browser, "custom"))
+	{
+		const char *web_command;
+
+		web_command = gaim_prefs_get_string("/gaim/gtk/browsers/command");
+
+		if (web_command == NULL || *web_command == '\0')
+		{
+			gaim_notify_error(NULL, NULL, _("Unable to open URL"),
+							  _("The 'Manual' browser command has been "
+								"chosen, but no command has been set."));
+			return NULL;
+		}
+
+		if (strstr(web_command, "%s"))
+			command = gaim_strreplace(web_command, "%s", uri);
+		else
+		{
+			/*
+			 * There is no "%s" in the browser command.  Assume the user
+			 * wanted the URL tacked on to the end of the command.
+			 */
+			command = g_strdup_printf("%s %s", web_command, uri);
+		}
+	}
+
+	if (remote_command != NULL)
+	{
+		/* try the remote command first */
+		if (uri_command(remote_command, TRUE) != 0)
+			uri_command(command, FALSE);
+
+		g_free(remote_command);
+
+	}
+	else
+		uri_command(command, FALSE);
+
+	g_free(command);
+
+#else /* !_WIN32 */
+	ShellExecute(NULL, NULL, uri, NULL, ".\\", 0);
+#endif /* !_WIN32 */
+
+	return NULL;
+}
+
 static GaimNotifyUiOps ops =
 {
 	gaim_gtk_notify_message,
@@ -363,7 +616,7 @@ static GaimNotifyUiOps ops =
 };
 
 GaimNotifyUiOps *
-gaim_get_gtk_notify_ui_ops(void)
+gaim_gtk_notify_get_ui_ops(void)
 {
 	return &ops;
 }
