@@ -4,8 +4,10 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -32,11 +34,15 @@
 
 #ifdef USE_GTKSPELL
 # include <gtkspell/gtkspell.h>
+# ifdef _WIN32
+#  include "wspell.h"
+# endif
 #endif
 
 #include <gdk/gdkkeysyms.h>
 
 #include "debug.h"
+#include "imgstore.h"
 #include "notify.h"
 #include "prefs.h"
 #include "prpl.h"
@@ -48,29 +54,43 @@
 #include "gtkutils.h"
 #include "ui.h"
 
-#ifdef _WIN32
-#include "wspell.h"
-#endif
+guint accels_save_timer = 0;
+
+static gboolean
+url_clicked_idle_cb(gpointer data)
+{
+	gaim_notify_uri(NULL, data);
+	g_free(data);
+	return FALSE;
+}
 
 static void
 url_clicked_cb(GtkWidget *w, const char *uri)
 {
-	gaim_notify_uri(NULL, uri);
+	g_idle_add(url_clicked_idle_cb, g_strdup(uri));
 }
 
+GtkIMHtmlFuncs gtkimhtml_cbs = {
+	(GtkIMHtmlGetImageFunc)gaim_imgstore_get,
+	(GtkIMHtmlGetImageDataFunc)gaim_imgstore_get_data,
+	(GtkIMHtmlGetImageSizeFunc)gaim_imgstore_get_size,
+	(GtkIMHtmlGetImageFilenameFunc)gaim_imgstore_get_filename,
+	gaim_imgstore_ref,
+	gaim_imgstore_unref,
+};
+
 void
 gaim_setup_imhtml(GtkWidget *imhtml)
 {
 	g_return_if_fail(imhtml != NULL);
 	g_return_if_fail(GTK_IS_IMHTML(imhtml));
 
-	if (!gaim_prefs_get_bool("/gaim/gtk/conversations/show_smileys"))
-		gtk_imhtml_show_smileys(GTK_IMHTML(imhtml), FALSE);
-
 	g_signal_connect(G_OBJECT(imhtml), "url_clicked",
 					 G_CALLBACK(url_clicked_cb), NULL);
 
 	smiley_themeize(imhtml);
+
+	gtk_imhtml_set_funcs(GTK_IMHTML(imhtml), &gtkimhtml_cbs);
 }
 
 void
@@ -81,173 +101,12 @@ toggle_sensitive(GtkWidget *widget, GtkWidget *to_toggle)
 	gtk_widget_set_sensitive(to_toggle, !sensitivity);
 }
 
-static void
-gaim_gtk_remove_tags(GaimGtkConversation *gtkconv, const char *tag)
-{
-	GtkTextIter start, end, m_start, m_end;
-
-	if (gtkconv == NULL || tag == NULL)
-		return;
-
-	if (!gtk_text_buffer_get_selection_bounds(gtkconv->entry_buffer,
-											  &start, &end))
-		return;
-
-	/* FIXMEif (strstr(tag, "<FONT SIZE=")) {
-		while ((t = strstr(t, "<FONT SIZE="))) {
-			if (((t - s) < finish) && ((t - s) >= start)) {
-				gtk_editable_delete_text(GTK_EDITABLE(entry), (t - s),
-							 (t - s) + strlen(tag));
-				g_free(s);
-				s = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
-				t = s;
-			} else
-				t++;
-		}
-	} else*/ {
-		while (gtk_text_iter_forward_search(&start, tag, 0, &m_start,
-											&m_end, &end)) {
-
-			gtk_text_buffer_delete(gtkconv->entry_buffer, &m_start, &m_end);
-			gtk_text_buffer_get_selection_bounds(gtkconv->entry_buffer,
-												 &start, &end);
-		}
-	}
-}
-
 void
-gaim_gtk_surround(GaimGtkConversation *gtkconv,
-				  const char *pre, const char *post)
+gaim_gtk_set_sensitive_if_input(GtkWidget *entry, GtkWidget *dialog)
 {
-	GtkTextIter start, end;
-	GtkTextMark *mark_start, *mark_end;
-	GtkTextBuffer *entry_buffer;
-
-	if (gtkconv == NULL || pre == NULL || post == NULL)
-		return;
-
-	entry_buffer = gtkconv->entry_buffer;
-
-	if (gtk_text_buffer_get_selection_bounds(entry_buffer,
-											 &start, &end)) {
-		gaim_gtk_remove_tags(gtkconv, pre);
-		gaim_gtk_remove_tags(gtkconv, post);
-
-		mark_start = gtk_text_buffer_create_mark(entry_buffer, "m1",
-												 &start, TRUE);
-		mark_end = gtk_text_buffer_create_mark(entry_buffer, "m2",
-											   &end, FALSE);
-		gtk_text_buffer_insert(entry_buffer, &start, pre, -1);
-		gtk_text_buffer_get_selection_bounds(entry_buffer, &start, &end);
-		gtk_text_buffer_insert(entry_buffer, &end, post, -1);
-		gtk_text_buffer_get_iter_at_mark(entry_buffer, &start, mark_start);
-		gtk_text_buffer_move_mark_by_name(entry_buffer, "selection_bound",
-										  &start);
-	} else {
-		gtk_text_buffer_insert(entry_buffer, &start, pre, -1);
-		gtk_text_buffer_insert(entry_buffer, &start, post, -1);
-		mark_start = gtk_text_buffer_get_insert(entry_buffer);
-		gtk_text_buffer_get_iter_at_mark(entry_buffer, &start, mark_start);
-		gtk_text_iter_backward_chars(&start, strlen(post));
-		gtk_text_buffer_place_cursor(entry_buffer, &start);
-	}
-
-	gtk_widget_grab_focus(gtkconv->entry);
-}
-
-static gboolean
-invert_tags(GtkTextBuffer *buffer, const char *s1, const char *s2,
-			gboolean really)
-{
-	GtkTextIter start1, start2, end1, end2;
-	char *b1, *b2;
-
-	if (gtk_text_buffer_get_selection_bounds(buffer, &start1, &end2)) {
-		start2 = start1;
-		end1 = end2;
-
-		if (!gtk_text_iter_forward_chars(&start2, strlen(s1)))
-			return FALSE;
-
-		if (!gtk_text_iter_backward_chars(&end1, strlen(s2)))
-			return FALSE;
-
-		b1 = gtk_text_buffer_get_text(buffer, &start1, &start2, FALSE);
-		b2 = gtk_text_buffer_get_text(buffer, &end1, &end2, FALSE);
-
-		if (!g_ascii_strncasecmp(b1, s1, strlen(s1)) &&
-		    !g_ascii_strncasecmp(b2, s2, strlen(s2))) {
-
-			if (really) {
-				GtkTextMark *m_end1, *m_end2;
- 
-				m_end1= gtk_text_buffer_create_mark(buffer, "m1", &end1, TRUE);
-				m_end2= gtk_text_buffer_create_mark(buffer, "m2", &end2, TRUE);
-
-				gtk_text_buffer_delete(buffer, &start1, &start2);
-				gtk_text_buffer_get_iter_at_mark(buffer, &end1, m_end1);
-				gtk_text_buffer_get_iter_at_mark(buffer, &end2, m_end2);
-				gtk_text_buffer_delete(buffer, &end1, &end2);
-				gtk_text_buffer_delete_mark(buffer, m_end1);
-				gtk_text_buffer_delete_mark(buffer, m_end2);
-			}
-
-			g_free(b1);
-			g_free(b2);
-
-			return TRUE;
-		}
-
-		g_free(b1);
-		g_free(b2);
-	}
-
-	return FALSE;
-}
-
-void
-gaim_gtk_advance_past(GaimGtkConversation *gtkconv,
-					  const char *pre, const char *post)
-{
-	GtkTextIter current_pos, start, end;
-
-	if (invert_tags(gtkconv->entry_buffer, pre, post, TRUE))
-		return;
-
-	gtk_text_buffer_get_iter_at_mark(gtkconv->entry_buffer, &current_pos,
-			gtk_text_buffer_get_insert(gtkconv->entry_buffer));
-
-	if (gtk_text_iter_forward_search(&current_pos, post, 0,
-									 &start, &end, NULL))
-		gtk_text_buffer_place_cursor(gtkconv->entry_buffer, &end);
-	else
-		gtk_text_buffer_insert_at_cursor(gtkconv->entry_buffer, post, -1);
-
-	gtk_widget_grab_focus(gtkconv->entry);
-}
-
-void
-gaim_gtk_set_font_face(GaimGtkConversation *gtkconv,
-					   const char *font)
-{
-	char *pre_fontface;
-
-	if (gtkconv == NULL || font == NULL)
-		return;
-
-	strncpy(gtkconv->fontface,
-			(font && *font ? font : DEFAULT_FONT_FACE),
-			sizeof(gtkconv->fontface));
-
-	gtkconv->has_font = TRUE;
-
-	pre_fontface = g_strconcat("<FONT FACE=\"",
-							   gtkconv->fontface, "\">", NULL);
-	gaim_gtk_surround(gtkconv, pre_fontface, "</FONT>");
-
-	gtk_widget_grab_focus(gtkconv->entry);
-
-	g_free(pre_fontface);
+	const char *text = gtk_entry_get_text(GTK_ENTRY(entry));
+	gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK,
+									  (*text != '\0'));
 }
 
 static int
@@ -266,7 +125,11 @@ do_save_icon(GtkObject *obj, GaimConversation *c)
 	GaimGtkConversation *gtkconv;
 	FILE *file;
 	const char *f;
-	
+	GaimBuddyIcon *icon;
+	size_t len;
+	const void *data;
+
+
 	gtkconv = GAIM_GTK_CONVERSATION(c);
 
 	f = gtk_file_selection_get_filename(
@@ -275,15 +138,11 @@ do_save_icon(GtkObject *obj, GaimConversation *c)
 	if (gaim_gtk_check_if_dir(f, GTK_FILE_SELECTION(gtkconv->u.im->save_icon)))
 		return;
 
-	if ((file = fopen(f, "w")) != NULL) {
-		int len;
-		void *data = get_icon_data(gaim_conversation_get_gc(c),
-								   normalize(gaim_conversation_get_name(c)),
-								   &len);
-
-		if (data)
-			fwrite(data, 1, len, file);
+	icon = gaim_conv_im_get_icon(GAIM_CONV_IM(c));
+	data = gaim_buddy_icon_get_data(icon, &len);
 
+	if ((len > 0) && (data != NULL) && (file = fopen(f, "wb")) != NULL) {
+		fwrite(data, 1, len, file);
 		fclose(file);
 	} else {
 		gaim_notify_error(NULL, NULL,
@@ -322,7 +181,7 @@ gaim_gtk_save_icon_dialog(GtkObject *obj, GaimConversation *conv)
 		return;
 	}
 
-	gtkconv->u.im->save_icon = gtk_file_selection_new(_("Gaim - Save Icon"));
+	gtkconv->u.im->save_icon = gtk_file_selection_new(_("Save Icon"));
 
 	gtk_file_selection_hide_fileop_buttons(
 		GTK_FILE_SELECTION(gtkconv->u.im->save_icon));
@@ -421,6 +280,18 @@ gtk_toggle_sensitive_array(GtkWidget *w, GPtrArray *data)
 	}
 }
 
+void
+gaim_gtk_toggle_showhide(GtkWidget *widget, GtkWidget *to_toggle)
+{
+	if (to_toggle == NULL)
+		return;
+
+	if (GTK_WIDGET_VISIBLE(to_toggle))
+		gtk_widget_hide(to_toggle);
+	else
+		gtk_widget_show(to_toggle);
+}
+
 void gaim_separator(GtkWidget *menu)
 {
 	GtkWidget *menuitem;
@@ -449,6 +320,7 @@ GtkWidget *gaim_new_item(GtkWidget *menu, const char *str)
 	gtk_widget_add_accelerator(menuitem, "activate", accel, str[0],
 				   GDK_MOD1_MASK, GTK_ACCEL_LOCKED);
 */
+	gaim_set_accessible_label (menuitem, label);
 	return menuitem;
 }
 
@@ -461,11 +333,11 @@ GtkWidget *gaim_new_check_item(GtkWidget *menu, const char *str,
 	if (menu)
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), checked);
+
 	if (sf)
 		g_signal_connect(G_OBJECT(menuitem), "activate", sf, data);
 
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), checked);
-
 	gtk_widget_show_all(menuitem);
 
 	return menuitem;
@@ -525,6 +397,7 @@ gaim_pixbuf_button_from_stock(const char *text, const char *icon,
 		gtk_label_set_text_with_mnemonic(GTK_LABEL(label), text);
 		gtk_label_set_mnemonic_widget(GTK_LABEL(label), button);
 		gtk_box_pack_start(GTK_BOX(lbox), label, FALSE, FALSE, 0);
+		gaim_set_accessible_label (button, label);
 	}
 
 	gtk_widget_show_all(bbox);
@@ -594,6 +467,7 @@ gaim_gtk_make_frame(GtkWidget *parent, const char *title)
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
+	gaim_set_accessible_label (vbox, label);
 
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -615,22 +489,22 @@ protocol_menu_cb(GtkWidget *optmenu, GCallback cb)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
-	GaimProtocol protocol;
+	const char *protocol;
 	gpointer user_data;
 
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenu));
 	item = gtk_menu_get_active(GTK_MENU(menu));
 
-	protocol = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "protocol"));
+	protocol = g_object_get_data(G_OBJECT(item), "protocol");
 	user_data = (g_object_get_data(G_OBJECT(optmenu), "user_data"));
 
 	if (cb != NULL)
-		((void (*)(GtkWidget *, GaimProtocol, gpointer))cb)(item, protocol,
+		((void (*)(GtkWidget *, const char *, gpointer))cb)(item, protocol,
 															user_data);
 }
 
 GtkWidget *
-gaim_gtk_protocol_option_menu_new(GaimProtocol protocol, GCallback cb,
+gaim_gtk_protocol_option_menu_new(const char *id, GCallback cb,
 								  gpointer user_data)
 {
 	GaimPluginProtocolInfo *prpl_info;
@@ -708,13 +582,13 @@ gaim_gtk_protocol_option_menu_new(GaimProtocol protocol, GCallback cb,
 		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 		gtk_widget_show(label);
 
-		g_object_set_data(G_OBJECT(item), "protocol",
-						 GINT_TO_POINTER(prpl_info->protocol));
+		g_object_set_data(G_OBJECT(item), "protocol", plugin->info->id);
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		gtk_widget_show(item);
+		gaim_set_accessible_label (item, label);
 
-		if (prpl_info->protocol == protocol)
+		if (id != NULL && !strcmp(plugin->info->id, id))
 			selected_index = i;
 	}
 
@@ -731,6 +605,14 @@ gaim_gtk_protocol_option_menu_new(GaimProtocol protocol, GCallback cb,
 	return optmenu;
 }
 
+GaimAccount *
+gaim_gtk_account_option_menu_get_selected(GtkWidget *optmenu)
+{
+	GtkWidget *menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenu));
+	GtkWidget *item = gtk_menu_get_active(GTK_MENU(menu));
+	return g_object_get_data(G_OBJECT(item), "account");
+}
+
 static void
 account_menu_cb(GtkWidget *optmenu, GCallback cb)
 {
@@ -752,7 +634,7 @@ account_menu_cb(GtkWidget *optmenu, GCallback cb)
 
 static void
 create_account_menu(GtkWidget *optmenu, GaimAccount *default_account,
-					GaimCheckAccountFunc check_account_func, gboolean show_all)
+					GaimFilterAccountFunc filter_func, gboolean show_all)
 {
 	GaimAccount *account;
 	GtkWidget *menu;
@@ -792,10 +674,12 @@ create_account_menu(GtkWidget *optmenu, GaimAccount *default_account,
 			account = gaim_connection_get_account(gc);
 		}
 
-		if (check_account_func && !check_account_func(account))
+		if (filter_func && !filter_func(account)) {
+			i--;
 			continue;
+		}
 
-		plugin = gaim_find_prpl(gaim_account_get_protocol(account));
+		plugin = gaim_find_prpl(gaim_account_get_protocol_id(account));
 
 		if (plugin != NULL)
 			prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
@@ -814,14 +698,19 @@ create_account_menu(GtkWidget *optmenu, GaimAccount *default_account,
 			g_snprintf(buf, sizeof(buf), "%s.png", proto_name);
 
 			filename = g_build_filename(DATADIR, "pixmaps", "gaim", "status",
-										"default", buf, NULL);
+			                            "default", buf, NULL);
 			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
 			g_free(filename);
 
 			if (pixbuf != NULL) {
 				/* Scale and insert the image */
 				scale = gdk_pixbuf_scale_simple(pixbuf, 16, 16,
-												GDK_INTERP_BILINEAR);
+				                                GDK_INTERP_BILINEAR);
+
+				if (!gaim_account_is_connected(account) && show_all &&
+						gaim_connections_get_all())
+					gdk_pixbuf_saturate_and_pixelate(scale, scale, 0.0, FALSE);
+
 				image = gtk_image_new_from_pixbuf(scale);
 
 				g_object_unref(G_OBJECT(pixbuf));
@@ -853,6 +742,7 @@ create_account_menu(GtkWidget *optmenu, GaimAccount *default_account,
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		gtk_widget_show(item);
+		gaim_set_accessible_label (item, label);
 
 		if (default_account != NULL && account == default_account)
 			selected_index = i;
@@ -868,13 +758,13 @@ create_account_menu(GtkWidget *optmenu, GaimAccount *default_account,
 }
 
 static void
-account_menu_sign_on_off_cb(GaimConnection *gc, GtkWidget *optmenu)
+regenerate_account_menu(GtkWidget *optmenu)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
 	gboolean show_all;
 	GaimAccount *account;
-	GaimCheckAccountFunc check_account_func;
+	GaimFilterAccountFunc filter_func;
 
 	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(optmenu));
 	item = gtk_menu_get_active(GTK_MENU(menu));
@@ -883,12 +773,24 @@ account_menu_sign_on_off_cb(GaimConnection *gc, GtkWidget *optmenu)
 	show_all = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(optmenu),
 												 "show_all"));
 
-	check_account_func = g_object_get_data(G_OBJECT(item),
-										   "check_account_func");
+	filter_func = g_object_get_data(G_OBJECT(optmenu),
+										   "filter_func");
 
 	gtk_option_menu_remove_menu(GTK_OPTION_MENU(optmenu));
 
-	create_account_menu(optmenu, account, check_account_func, show_all);
+	create_account_menu(optmenu, account, filter_func, show_all);
+}
+
+static void
+account_menu_sign_on_off_cb(GaimConnection *gc, GtkWidget *optmenu)
+{
+	regenerate_account_menu(optmenu);
+}
+
+static void
+account_menu_added_removed_cb(GaimAccount *account, GtkWidget *optmenu)
+{
+	regenerate_account_menu(optmenu);
 }
 
 static gboolean
@@ -903,7 +805,7 @@ account_menu_destroyed_cb(GtkWidget *optmenu, GdkEvent *event,
 GtkWidget *
 gaim_gtk_account_option_menu_new(GaimAccount *default_account,
 								 gboolean show_all, GCallback cb,
-								 GaimCheckAccountFunc check_account_func,
+								 GaimFilterAccountFunc filter_func,
 								 gpointer user_data)
 {
 	GtkWidget *optmenu;
@@ -922,15 +824,21 @@ gaim_gtk_account_option_menu_new(GaimAccount *default_account,
 	gaim_signal_connect(gaim_connections_get_handle(), "signed-off",
 						optmenu, GAIM_CALLBACK(account_menu_sign_on_off_cb),
 						optmenu);
+	gaim_signal_connect(gaim_accounts_get_handle(), "account-added",
+						optmenu, GAIM_CALLBACK(account_menu_added_removed_cb),
+						optmenu);
+	gaim_signal_connect(gaim_accounts_get_handle(), "account-removed",
+						optmenu, GAIM_CALLBACK(account_menu_added_removed_cb),
+						optmenu);
 
 	/* Set some data. */
 	g_object_set_data(G_OBJECT(optmenu), "user_data", user_data);
 	g_object_set_data(G_OBJECT(optmenu), "show_all", GINT_TO_POINTER(show_all));
-	g_object_set_data(G_OBJECT(optmenu), "chck_account_func",
-					  check_account_func);
+	g_object_set_data(G_OBJECT(optmenu), "filter_func",
+					  filter_func);
 
 	/* Create and set the actual menu. */
-	create_account_menu(optmenu, default_account, check_account_func, show_all);
+	create_account_menu(optmenu, default_account, filter_func, show_all);
 
 	/* And now the last callback. */
 	g_signal_connect(G_OBJECT(optmenu), "changed",
@@ -967,90 +875,512 @@ char *stylize(const gchar *text, int length)
 	buf = g_malloc(length);
 	g_snprintf(buf, length, "%s", text);
 
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_bold")) {
-		g_snprintf(tmp, length, "<B>%s</B>", buf);
+	if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_formatting")) {
+		const char *font;
+		GdkColor fg_color, bg_color;
+
+		if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_bold")) {
+			g_snprintf(tmp, length, "<B>%s</B>", buf);
+			strcpy(buf, tmp);
+		}
+
+		if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_italic")) {
+			g_snprintf(tmp, length, "<I>%s</I>", buf);
+			strcpy(buf, tmp);
+		}
+
+		if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_underline")) {
+			g_snprintf(tmp, length, "<U>%s</U>", buf);
+			strcpy(buf, tmp);
+		}
+
+		font = gaim_prefs_get_string("/gaim/gtk/conversations/font_face");
+
+		if (strcmp(font, "") != 0) {
+			g_snprintf(tmp, length, "<FONT FACE=\"%s\">%s</FONT>", font, buf);
+			strcpy(buf, tmp);
+		}
+
+		g_snprintf(tmp, length, "<FONT SIZE=\"%d\">%s</FONT>",
+			gaim_prefs_get_int("/gaim/gtk/conversations/font_size"), buf);
 		strcpy(buf, tmp);
-	}
 
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_italic")) {
-		g_snprintf(tmp, length, "<I>%s</I>", buf);
-		strcpy(buf, tmp);
-	}
+		if(strcmp(gaim_prefs_get_string("/gaim/gtk/conversations/fgcolor"), "") != 0)
+		{
+			gdk_color_parse(gaim_prefs_get_string("/gaim/gtk/conversations/fgcolor"),
+							&fg_color);
 
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_underline")) {
-		g_snprintf(tmp, length, "<U>%s</U>", buf);
-		strcpy(buf, tmp);
-	}
+			g_snprintf(tmp, length, "<FONT COLOR=\"#%02X%02X%02X\">%s</FONT>",
+					   fg_color.red/256, fg_color.green/256,
+					   fg_color.blue/256, buf);
+			strcpy(buf, tmp);
+		}
 
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/send_strikethrough")) {
-		g_snprintf(tmp, length, "<S>%s</S>", buf);
-		strcpy(buf, tmp);
-	}
+		if(strcmp(gaim_prefs_get_string("/gaim/gtk/conversations/bgcolor"), "") != 0)
+		{
+			gdk_color_parse(gaim_prefs_get_string("/gaim/gtk/conversations/bgcolor"),
+							&bg_color);
 
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/use_custom_font")) {
-		const char *fontface;
+			g_snprintf(tmp, length, "<BODY BGCOLOR=\"#%02X%02X%02X\">%s</BODY>",
+					   bg_color.red/256, bg_color.green/256,
+					   bg_color.blue/256, buf);
+			strcpy(buf, tmp);
+		}
 
-		fontface = gaim_prefs_get_string("/gaim/gtk/conversations/font_face");
-
-		g_snprintf(tmp, length, "<FONT FACE=\"%s\">%s</FONT>", fontface, buf);
-		strcpy(buf, tmp);
-	}
-
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/use_custom_size")) {
-		int fontsize = gaim_prefs_get_int("/gaim/gtk/conversations/font_size");
-
-		g_snprintf(tmp, length, "<FONT SIZE=\"%d\">%s</FONT>", fontsize, buf);
-		strcpy(buf, tmp);
-	}
-
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/use_custom_fgcolor")) {
-		GdkColor fgcolor;
-
-		gdk_color_parse(
-			gaim_prefs_get_string("/gaim/gtk/conversations/fgcolor"),
-			&fgcolor);
-
-		g_snprintf(tmp, length, "<FONT COLOR=\"#%02X%02X%02X\">%s</FONT>",
-				   fgcolor.red/256, fgcolor.green/256, fgcolor.blue/256, buf);
-		strcpy(buf, tmp);
-	}
-
-	if (gaim_prefs_get_bool("/gaim/gtk/conversations/use_custom_bgcolor")) {
-		GdkColor bgcolor;
-
-		gdk_color_parse(
-			gaim_prefs_get_string("/gaim/gtk/conversations/bgcolor"),
-			&bgcolor);
-
-		g_snprintf(tmp, length, "<BODY BGCOLOR=\"#%02X%02X%02X\">%s</BODY>",
-				   bgcolor.red/256, bgcolor.green/256, bgcolor.blue/256, buf);
-		strcpy(buf, tmp);
 	}
 
 	g_free(tmp);
 	return buf;
 }
 
-void show_usage(int mode, const char *name)
+void
+gaim_gtk_setup_gtkspell(GtkTextView *textview)
 {
-	switch (mode) {
-	case 0:		/* full help text */
-		printf(_("Gaim %s\n"
-		       "Usage: %s [OPTION]...\n\n"
-		       "  -a, --acct          display account editor window\n"
-		       "  -w, --away[=MESG]   make away on signon (optional argument MESG specifies\n"
-		       "                      name of away message to use)\n"
-		       "  -l, --login[=NAME]  automatically login (optional argument NAME specifies\n"
-		       "                      account(s) to use, seperated by commas)\n"
-		       "  -n, --loginwin      don't automatically login; show login window\n"
-		       "  -u, --user=NAME     use account NAME\n"
-		       "  -f, --file=FILE     use FILE as config\n"
-		       "  -d, --debug         print debugging messages to stdout\n"
-		       "  -v, --version       display the current version and exit\n"
-		       "  -h, --help          display this help and exit\n"), VERSION, name);
-		break;
-	case 1:		/* short message */
-		printf(_("Gaim %s. Try `%s -h' for more information.\n"), VERSION, name);
-		break;
+#ifdef USE_GTKSPELL
+	GError *error = NULL;
+	char *locale = NULL;
+
+	g_return_if_fail(textview != NULL);
+	g_return_if_fail(GTK_IS_TEXT_VIEW(textview));
+
+	if (gtkspell_new_attach(textview, locale, &error) == NULL && error)
+	{
+		gaim_debug_warning("gtkspell", "Failed to setup GtkSpell: %s\n",
+						   error->message);
+		g_error_free(error);
+	}
+#endif /* USE_GTKSPELL */
+}
+
+void
+gaim_gtk_save_accels_cb(GtkAccelGroup *accel_group, guint arg1,
+														 GdkModifierType arg2, GClosure *arg3,
+														 gpointer data)
+{
+	gaim_debug(GAIM_DEBUG_MISC, "accels", "accel changed, scheduling save.\n");
+
+	if (!accels_save_timer)
+		accels_save_timer = g_timeout_add(5000, gaim_gtk_save_accels, NULL);
+}
+
+gboolean
+gaim_gtk_save_accels(gpointer data)
+{
+	char *filename = NULL;
+
+	filename = g_build_filename(gaim_user_dir(), G_DIR_SEPARATOR_S,
+															"accels", NULL);
+	gaim_debug(GAIM_DEBUG_MISC, "accels", "saving accels to %s\n", filename);
+	gtk_accel_map_save(filename);
+	g_free(filename);
+
+	accels_save_timer = 0;
+	return FALSE;
+}
+
+void
+gaim_gtk_load_accels(gpointer data)
+{
+	char *filename = NULL;
+
+	filename = g_build_filename(gaim_user_dir(), G_DIR_SEPARATOR_S,
+															"accels", NULL);
+	gtk_accel_map_load(filename);
+	g_free(filename);
+}
+
+gboolean
+gaim_gtk_parse_x_im_contact(const char *msg, gboolean all_accounts,
+							GaimAccount **ret_account, char **ret_protocol,
+							char **ret_username, char **ret_alias)
+{
+	char *protocol = NULL;
+	char *username = NULL;
+	char *alias    = NULL;
+	char *str;
+	char *c, *s;
+	gboolean valid;
+
+	g_return_val_if_fail(msg          != NULL, FALSE);
+	g_return_val_if_fail(ret_protocol != NULL, FALSE);
+	g_return_val_if_fail(ret_username != NULL, FALSE);
+
+	s = str = g_strdup(msg);
+
+	while (*s != '\r' && *s != '\n' && *s != '\0')
+	{
+		char *key, *value;
+
+		key = s;
+
+		/* Grab the key */
+		while (*s != '\r' && *s != '\n' && *s != '\0' && *s != ' ')
+			s++;
+
+		if (*s == '\r') s++;
+
+		if (*s == '\n')
+		{
+			s++;
+			continue;
+		}
+
+		if (*s != '\0') *s++ = '\0';
+
+		/* Clear past any whitespace */
+		while (*s != '\0' && *s == ' ')
+			s++;
+
+		/* Now let's grab until the end of the line. */
+		value = s;
+
+		while (*s != '\r' && *s != '\n' && *s != '\0')
+			s++;
+
+		if (*s == '\r') *s++ = '\0';
+		if (*s == '\n') *s++ = '\0';
+
+		if ((c = strchr(key, ':')) != NULL)
+		{
+			if (!g_ascii_strcasecmp(key, "X-IM-Username:"))
+				username = g_strdup(value);
+			else if (!g_ascii_strcasecmp(key, "X-IM-Protocol:"))
+				protocol = g_strdup(value);
+			else if (!g_ascii_strcasecmp(key, "X-IM-Alias:"))
+				alias = g_strdup(value);
+		}
+	}
+
+	if (username != NULL && protocol != NULL)
+	{
+		valid = TRUE;
+
+		*ret_username = username;
+		*ret_protocol = protocol;
+
+		if (ret_alias != NULL)
+			*ret_alias = alias;
+
+		/* Check for a compatible account. */
+		if (ret_account != NULL)
+		{
+			GList *list;
+			GaimAccount *account = NULL;
+			GList *l;
+			const char *protoname;
+
+			if (all_accounts)
+				list = gaim_accounts_get_all();
+			else
+				list = gaim_connections_get_all();
+
+			for (l = list; l != NULL; l = l->next)
+			{
+				GaimConnection *gc;
+				GaimPluginProtocolInfo *prpl_info = NULL;
+				GaimPlugin *plugin;
+
+				if (all_accounts)
+				{
+					account = (GaimAccount *)l->data;
+
+					plugin = gaim_plugins_find_with_id(
+						gaim_account_get_protocol_id(account));
+
+					if (plugin == NULL)
+					{
+						account = NULL;
+
+						continue;
+					}
+
+					prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
+				}
+				else
+				{
+					gc = (GaimConnection *)l->data;
+					account = gaim_connection_get_account(gc);
+
+					prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+				}
+
+				protoname = prpl_info->list_icon(account, NULL);
+
+				if (!strcmp(protoname, protocol))
+					break;
+
+				account = NULL;
+			}
+
+			/* Special case for AIM and ICQ */
+			if (account == NULL && (!strcmp(protocol, "aim") ||
+									!strcmp(protocol, "icq")))
+			{
+				for (l = list; l != NULL; l = l->next)
+				{
+					GaimConnection *gc;
+					GaimPluginProtocolInfo *prpl_info = NULL;
+					GaimPlugin *plugin;
+
+					if (all_accounts)
+					{
+						account = (GaimAccount *)l->data;
+
+						plugin = gaim_plugins_find_with_id(
+							gaim_account_get_protocol_id(account));
+
+						if (plugin == NULL)
+						{
+							account = NULL;
+
+							continue;
+						}
+
+						prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
+					}
+					else
+					{
+						gc = (GaimConnection *)l->data;
+						account = gaim_connection_get_account(gc);
+
+						prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(gc->prpl);
+					}
+
+					protoname = prpl_info->list_icon(account, NULL);
+
+					if (!strcmp(protoname, "aim") || !strcmp(protoname, "icq"))
+						break;
+
+					account = NULL;
+				}
+			}
+
+			*ret_account = account;
+		}
+	}
+	else
+	{
+		valid = FALSE;
+
+		if (username != NULL) g_free(username);
+		if (protocol != NULL) g_free(protocol);
+		if (alias    != NULL) g_free(alias);
+	}
+
+	g_free(str);
+
+	return valid;
+}
+
+void
+gaim_set_accessible_label (GtkWidget *w, GtkWidget *l)
+{
+	AtkObject *acc, *label;
+	AtkObject *rel_obj[1];
+	AtkRelationSet *set;
+	AtkRelation *relation;
+	const gchar *label_text;
+	const gchar *existing_name;
+
+	acc = gtk_widget_get_accessible (w);
+	label = gtk_widget_get_accessible (l);
+
+	/* If this object has no name, set it's name with the label text */
+	existing_name = atk_object_get_name (acc);
+	if (!existing_name) {
+		label_text = gtk_label_get_text (GTK_LABEL(l));
+		if (label_text)
+			atk_object_set_name (acc, label_text);
+	}
+
+	/* Create the labeled-by relation */
+	set = atk_object_ref_relation_set (acc);
+	rel_obj[0] = label;
+	relation = atk_relation_new (rel_obj, 1, ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (set, relation);
+	g_object_unref (relation);
+
+	/* Create the label-for relation */
+	set = atk_object_ref_relation_set (label);
+	rel_obj[0] = acc;
+	relation = atk_relation_new (rel_obj, 1, ATK_RELATION_LABEL_FOR);
+	atk_relation_set_add (set, relation);
+	g_object_unref (relation);
+}
+
+#if GTK_CHECK_VERSION(2,2,0)
+static void
+gaim_gtk_menu_position_func(GtkMenu *menu,
+							gint *x,
+							gint *y,
+							gboolean *push_in,
+							gpointer data)
+{
+	GtkWidget *widget;
+	GtkRequisition requisition;
+	GdkScreen *screen;
+	GdkRectangle monitor;
+	gint monitor_num;
+	gint space_left, space_right, space_above, space_below;
+	gint needed_width;
+	gint needed_height;
+	gint xthickness;
+	gint ythickness;
+	gboolean rtl;
+
+	g_return_if_fail(GTK_IS_MENU(menu));
+
+	widget     = GTK_WIDGET(menu);
+	screen     = gtk_widget_get_screen(widget);
+	xthickness = widget->style->xthickness;
+	ythickness = widget->style->ythickness;
+	rtl        = (gtk_widget_get_direction(widget) == GTK_TEXT_DIR_RTL);
+
+	/*
+	 * We need the requisition to figure out the right place to
+	 * popup the menu. In fact, we always need to ask here, since
+	 * if a size_request was queued while we weren't popped up,
+	 * the requisition won't have been recomputed yet.
+	 */
+	gtk_widget_size_request (widget, &requisition);
+
+	monitor_num = gdk_screen_get_monitor_at_point (screen, *x, *y);
+
+	push_in = FALSE;
+  
+	/*
+	 * The placement of popup menus horizontally works like this (with
+	 * RTL in parentheses)
+	 *
+	 * - If there is enough room to the right (left) of the mouse cursor,
+	 *   position the menu there.
+	 * 
+	 * - Otherwise, if if there is enough room to the left (right) of the 
+	 *   mouse cursor, position the menu there.
+	 * 
+	 * - Otherwise if the menu is smaller than the monitor, position it
+	 *   on the side of the mouse cursor that has the most space available
+	 *
+	 * - Otherwise (if there is simply not enough room for the menu on the
+	 *   monitor), position it as far left (right) as possible.
+	 *
+	 * Positioning in the vertical direction is similar: first try below
+	 * mouse cursor, then above.
+	 */
+	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+	space_left = *x - monitor.x;
+	space_right = monitor.x + monitor.width - *x - 1;
+	space_above = *y - monitor.y;
+	space_below = monitor.y + monitor.height - *y - 1;
+
+	/* position horizontally */
+
+	/* the amount of space we need to position the menu. Note the
+	 * menu is offset "xthickness" pixels 
+	 */
+	needed_width = requisition.width - xthickness;
+
+	if (needed_width <= space_left ||
+	    needed_width <= space_right)
+	{
+		if ((rtl  && needed_width <= space_left) ||
+		    (!rtl && needed_width >  space_right))
+		{
+			/* position left */
+			*x = *x + xthickness - requisition.width + 1;
+		}
+		else
+		{
+			/* position right */
+			*x = *x - xthickness;
+		}
+
+		/* x is clamped on-screen further down */
+	}
+	else if (requisition.width <= monitor.width)
+	{
+		/* the menu is too big to fit on either side of the mouse
+		 * cursor, but smaller than the monitor. Position it on
+		 * the side that has the most space
+		 */
+		if (space_left > space_right)
+		{
+			/* left justify */
+			*x = monitor.x;
+		}
+		else
+		{
+			/* right justify */
+			*x = monitor.x + monitor.width - requisition.width;
+		}
+	}
+	else /* menu is simply too big for the monitor */
+	{
+		if (rtl)
+		{
+			/* right justify */
+			*x = monitor.x + monitor.width - requisition.width;
+		}
+		else
+		{
+			/* left justify */
+			*x = monitor.x;
+		}
+	}
+
+	/* Position vertically. The algorithm is the same as above, but
+	 * simpler because we don't have to take RTL into account.
+	 */
+	needed_height = requisition.height - ythickness;
+
+	if (needed_height <= space_above ||
+	    needed_height <= space_below)
+	{
+		if (needed_height <= space_below)
+			*y = *y - ythickness;
+		else
+			*y = *y + ythickness - requisition.height + 1;
+	  
+		*y = CLAMP (*y, monitor.y,
+			   monitor.y + monitor.height - requisition.height);
+	}
+	else if (needed_height > space_below && needed_height > space_above)
+	{
+		if (space_below >= space_above)
+			*y = monitor.y + monitor.height - requisition.height;
+		else
+			*y = monitor.y;
+	}
+	else
+	{
+		*y = monitor.y;
 	}
 }
+
+#endif
+
+void
+gaim_gtk_treeview_popup_menu_position_func(GtkMenu *menu,
+										   gint *x,
+										   gint *y,
+										   gboolean *push_in,
+										   gpointer data)
+{
+	GtkWidget *widget = GTK_WIDGET(data);
+	GtkTreeView *tv = GTK_TREE_VIEW(data);
+	GtkTreePath *path;
+	GtkTreeViewColumn *col;
+	GdkRectangle rect;
+	gint ythickness = GTK_WIDGET(menu)->style->ythickness;
+
+	gdk_window_get_origin (widget->window, x, y);
+	gtk_tree_view_get_cursor (tv, &path, &col);
+	gtk_tree_view_get_cell_area (tv, path, col, &rect);
+
+	*x += rect.x+rect.width;
+	*y += rect.y+rect.height+ythickness;
+#if GTK_CHECK_VERSION(2,2,0)
+	gaim_gtk_menu_position_func (menu, x, y, push_in, data);
+#endif
+}
