@@ -1,7 +1,9 @@
 /*
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +27,7 @@
 #include "conversation.h"
 #include "core.h"
 #include "debug.h"
+#include "eventloop.h"
 #include "ft.h"
 #include "log.h"
 #include "notify.h"
@@ -40,12 +43,15 @@
 #include "gtkconn.h"
 #include "gtkconv.h"
 #include "gtkdebug.h"
+#include "gtkeventloop.h"
 #include "gtkft.h"
 #include "gtknotify.h"
+#include "gtkplugin.h"
 #include "gtkpounce.h"
 #include "gtkprefs.h"
 #include "gtkprivacy.h"
 #include "gtkrequest.h"
+#include "gtkroomlist.h"
 #include "gtksound.h"
 #include "gtkutils.h"
 #include "stock.h"
@@ -56,25 +62,31 @@
 # include <signal.h>
 #endif
 
-#include "locale.h"
 #include <getopt.h>
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+# define SN_API_NOT_YET_FROZEN
+# include <libsn/sn-launchee.h>
+# include <gdk/gdkx.h>
+#endif
+
 extern void load_prefs();
 extern void load_pounces();
 
 static GtkWidget *name;
 static GtkWidget *pass;
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+static SnLauncheeContext *sn_context = NULL;
+static SnDisplay *sn_display = NULL;
+#endif
+
 GtkWidget *mainwindow = NULL;
 
 int opt_away = 0;
 int docklet_count = 0;
 char *opt_away_arg = NULL;
-char *opt_rcfile_arg = NULL;
 int opt_debug = 0;
-#ifdef _WIN32
-int opt_gdebug = 0;
-#endif
 
 #if HAVE_SIGNAL_H
 /*
@@ -110,13 +122,11 @@ static gboolean sound_timeout(gpointer data)
  * oscar.c, after the buddy list is made and serv_finish_login is called */
 void gaim_setup(GaimConnection *gc)
 {
-	if (gaim_prefs_get_bool("/gaim/gtk/sound/enabled/login")
-			&& gaim_prefs_get_bool("/gaim/gtk/sound/silent_signon")) {
-		if(snd_tmout) {
+	if (gaim_prefs_get_bool("/gaim/gtk/sound/enabled/login")) {
+		if (snd_tmout)
 			g_source_remove(snd_tmout);
-		}
 		gaim_gtk_sound_set_login_mute(TRUE);
-		snd_tmout = g_timeout_add(10000, sound_timeout, NULL);
+		snd_tmout = gaim_timeout_add(10000, sound_timeout, NULL);
 	}
 }
 
@@ -133,23 +143,15 @@ static gboolean domiddleclick(GtkWidget *w, GdkEventButton *event, gpointer null
 static void dologin(GtkWidget *widget, GtkWidget *w)
 {
 	GaimAccount *account;
-	const char *username = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(name)->entry));
+	GtkWidget *item;
 	const char *password = gtk_entry_get_text(GTK_ENTRY(pass));
 
-	if (*username == '\0') {
-		gaim_notify_error(NULL, NULL, _("Please enter your login."), NULL);
-		return;
-	}
+	item = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(name))));
+	account = g_object_get_data(G_OBJECT(item), "account");
 
-	/* if there is more than one user of the same name, then fuck 
-	 * them, they just have to use the account editor to sign in 
-	 * the second one */
-
-	account = gaim_accounts_find(username, -1);
 	if (!account) {
-		account = gaim_account_new(username, "prpl-oscar");
-		gaim_account_set_remember_password(account, TRUE);
-		gaim_accounts_add(account);
+		gaim_notify_error(NULL, NULL, _("Please create an account."), NULL);
+		return;
 	}
 
 	gaim_account_set_password(account, (*password != '\0') ? password : NULL);
@@ -173,7 +175,7 @@ static int dologin_named(char *name)
 	if (name !=NULL) {	/* list of names given */
 		names = g_strsplit(name, ",", 32);
 		for (n = names; *n != NULL; n++) {
-			account = gaim_accounts_find(*n, -1);
+			account = gaim_accounts_find(*n, NULL);
 			if (account) {	/* found a user */
 				retval = 0;
 				gaim_account_connect(account);
@@ -190,21 +192,8 @@ static int dologin_named(char *name)
 }
 
 
-static void doenter(GtkWidget *widget, GtkWidget *w)
+static void combo_changed(GtkWidget *menu, GaimAccount *account, gpointer data)
 {
-	gtk_entry_set_text(GTK_ENTRY(pass), "");
-	gtk_editable_select_region(GTK_EDITABLE(GTK_COMBO(name)->entry), 0, 0);
-	gtk_widget_grab_focus(pass);
-}
-
-
-static void combo_changed(GtkWidget *w, GtkWidget *combo)
-{
-	const char *txt = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));
-	GaimAccount *account;
-
-	account = gaim_accounts_find(txt, -1);
-
 	if (account && gaim_account_get_remember_password(account)) {
 		gtk_entry_set_text(GTK_ENTRY(pass), account->password);
 	} else {
@@ -213,24 +202,6 @@ static void combo_changed(GtkWidget *w, GtkWidget *combo)
 }
 
 
-static GList *combo_user_names()
-{
-	GList *accts = gaim_accounts_get_all();
-	GList *tmp = NULL;
-	GaimAccount *account;
-
-	if (!accts)
-		return g_list_append(NULL, _("<New User>"));
-
-	while (accts) {
-		account = (GaimAccount *)accts->data;
-		tmp = g_list_append(tmp, account->username);
-		accts = accts->next;
-	}
-
-	return tmp;
-}
-
 static void login_window_closed(GtkWidget *w, GdkEvent *ev, gpointer d)
 {
 	if(docklet_count) {
@@ -250,7 +221,6 @@ void show_login()
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *vbox2;
-	GList *tmp;
 
 	/* Do we already have a main window opened? If so, bring it back, baby... ribs... yeah */
 	if (mainwindow) {
@@ -278,29 +248,28 @@ void show_login()
 	vbox2 = gtk_vbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 5);
 
-	label = gtk_label_new(_("Screen Name:"));
+	/* why isn't there a gtk_label_new_with_markup? */
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("<b>_Account:</b>"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
-	name = gtk_combo_new();
-	tmp = combo_user_names();
-	gtk_combo_set_popdown_strings(GTK_COMBO(name), tmp);
-	g_list_free(tmp);
-	g_signal_connect(G_OBJECT(GTK_COMBO(name)->entry), "activate",
-					 G_CALLBACK(doenter), mainwindow);
-	g_signal_connect(G_OBJECT(GTK_COMBO(name)->entry), "changed",
-					 G_CALLBACK(combo_changed), name);
+	name = gaim_gtk_account_option_menu_new(NULL, TRUE, G_CALLBACK(combo_changed), NULL, NULL);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), name);
+
 	gtk_box_pack_start(GTK_BOX(vbox2), name, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, TRUE, 0);
 
 	vbox2 = gtk_vbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 5);
 
-	label = gtk_label_new(_("Password:"));
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), _("<b>_Password:</b>"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
 	pass = gtk_entry_new();
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), pass);
 	gtk_entry_set_visibility(GTK_ENTRY(pass), FALSE);
 	g_signal_connect(G_OBJECT(pass), "activate",
 					 G_CALLBACK(dologin), mainwindow);
@@ -312,23 +281,19 @@ void show_login()
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
 
 	/* And now for the buttons */
-	button = gaim_pixbuf_button_from_stock(_("Accounts"), GAIM_STOCK_ACCOUNTS, GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("_Accounts"), GAIM_STOCK_ACCOUNTS, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(gaim_gtk_accounts_window_show), mainwindow);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
-#ifdef NO_MULTI
-	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
-#endif
-
-	button = gaim_pixbuf_button_from_stock(_("Preferences"), GTK_STOCK_PREFERENCES, GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("_Preferences"), GTK_STOCK_PREFERENCES, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(gaim_gtk_prefs_show), mainwindow);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
-	button = gaim_pixbuf_button_from_stock(_("Sign on"), GAIM_STOCK_SIGN_ON, GAIM_BUTTON_VERTICAL);
+	button = gaim_pixbuf_button_from_stock(_("_Sign on"), GAIM_STOCK_SIGN_ON, GAIM_BUTTON_VERTICAL);
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button), "clicked",
 					 G_CALLBACK(dologin), mainwindow);
@@ -340,13 +305,14 @@ void show_login()
 		GaimAccount *account = gaim_accounts_get_all()->data;
 
 		if (gaim_account_get_remember_password(account)) {
-			combo_changed(NULL, name);
+			combo_changed(NULL, account, NULL);
 			gtk_widget_grab_focus(button);
 		} else {
 			gtk_widget_grab_focus(pass);
 		}
 	} else {
-		gtk_widget_grab_focus(name);
+		gaim_gtk_accounts_window_show();
+		gtk_widget_grab_focus(button);
 	}
 
 	/* And raise the curtain! */
@@ -354,6 +320,24 @@ void show_login()
 
 }
 
+static void
+clean_pid(void)
+{
+#ifndef _WIN32
+	int status;
+	pid_t pid;
+
+	do {
+		pid = waitpid(-1, &status, WNOHANG);
+	} while (pid != 0 && pid != (pid_t)-1);
+	if(pid == (pid_t)-1 && errno != ECHILD) {
+		char errmsg[BUFSIZ];
+		snprintf(errmsg, BUFSIZ, "Warning: waitpid() returned %d", pid);
+		perror(errmsg);
+	}
+#endif
+}
+
 #if HAVE_SIGNAL_H
 void sighandler(int sig)
 {
@@ -379,7 +363,12 @@ void sighandler(int sig)
 			"assistance, please IM either RobFlynn or SeanEgn and\n"
 			"they can help you.\n");
 #else
-		fprintf(stderr, "Oh no!  Segmentation fault!\n");
+		fprintf(stderr, "Hi, user.  We need to talk.\n"
+			"I think something's gone wrong here.  It's probably my fault.\n"
+			"No, really, it's not you... it's me... no no no, I think we get along well\n"
+			"it's just that.... well, I want to see other people.  I... what?!?  NO!  I haven't\n"
+			"been cheating on you!!  How many times do you want me to tell you?!  And for the\n"
+			"last time, it's just a rash!\n");
 		/*g_on_error_query (g_get_prgname());*/
 #endif
 		abort();
@@ -445,10 +434,10 @@ static void set_first_user(char *name)
 {
 	GaimAccount *account;
 
-	account = gaim_accounts_find(name, -1);
+	account = gaim_accounts_find(name, NULL);
 
 	if (account == NULL) { /* new user */
-		account = gaim_account_new(name, "prpl-oscar");
+		account = gaim_account_new(name, GAIM_PROTO_DEFAULT);
 		gaim_accounts_add(account);
 	}
 
@@ -456,33 +445,10 @@ static void set_first_user(char *name)
 	gaim_accounts_reorder(account, 0);
 }
 
-#ifdef _WIN32
-/* WIN32 print and log handlers */
-
-static void gaim_dummy_print( const gchar* string ) {
-	return;
-}
-
-static void gaim_dummy_log_handler (const gchar    *domain,
-				    GLogLevelFlags  flags,
-				    const gchar    *msg,
-				    gpointer        user_data) {
-	return;
-}
-
-static void gaim_log_handler (const gchar    *domain,
-			      GLogLevelFlags  flags,
-			      const gchar    *msg,
-			      gpointer        user_data) {
-	gaim_debug(GAIM_DEBUG_MISC, "log", "%s - %s\n", domain, msg);
-	g_log_default_handler(domain, flags, msg, user_data);
-}
-#endif /* _WIN32 */
-
 static void
 debug_init(void)
 {
-	gaim_set_debug_ui_ops(gaim_get_gtk_debug_ui_ops());
+	gaim_debug_set_ui_ops(gaim_gtk_debug_get_ui_ops());
 	gaim_gtk_debug_init();
 }
 
@@ -490,21 +456,24 @@ static void
 gaim_gtk_ui_init(void)
 {
 	/* Set the UI operation structures. */
-	gaim_set_win_ui_ops(gaim_get_gtk_window_ui_ops());
-	gaim_set_xfer_ui_ops(gaim_get_gtk_xfer_ui_ops());
-	gaim_set_blist_ui_ops(gaim_get_gtk_blist_ui_ops());
-	gaim_set_notify_ui_ops(gaim_get_gtk_notify_ui_ops());
-	gaim_set_privacy_ui_ops(gaim_gtk_privacy_get_ui_ops());
-	gaim_set_request_ui_ops(gaim_get_gtk_request_ui_ops());
-	gaim_set_sound_ui_ops(gaim_get_gtk_sound_ui_ops());
-	gaim_set_connection_ui_ops(gaim_get_gtk_connection_ui_ops());
+	gaim_accounts_set_ui_ops(gaim_gtk_accounts_get_ui_ops());
+	gaim_conversations_set_win_ui_ops(gaim_gtk_conversations_get_win_ui_ops());
+	gaim_xfers_set_ui_ops(gaim_gtk_xfers_get_ui_ops());
+	gaim_blist_set_ui_ops(gaim_gtk_blist_get_ui_ops());
+	gaim_notify_set_ui_ops(gaim_gtk_notify_get_ui_ops());
+	gaim_privacy_set_ui_ops(gaim_gtk_privacy_get_ui_ops());
+	gaim_request_set_ui_ops(gaim_gtk_request_get_ui_ops());
+	gaim_sound_set_ui_ops(gaim_gtk_sound_get_ui_ops());
+	gaim_connections_set_ui_ops(gaim_gtk_connections_get_ui_ops());
 
+	gaim_gtk_stock_init();
 	gaim_gtk_prefs_init();
 	gaim_gtk_blist_init();
-	gaim_gtk_conversation_init();
+	gaim_gtk_conversations_init();
 	gaim_gtk_pounces_init();
 	gaim_gtk_privacy_init();
 	gaim_gtk_xfers_init();
+	gaim_gtk_roomlist_init();
 }
 
 static void
@@ -513,13 +482,16 @@ gaim_gtk_quit(void)
 	/* XXX? */
 
 	/* captain's log, stardate... */
-	system_log(log_quit, NULL, NULL, OPT_LOG_BUDDY_SIGNON | OPT_LOG_MY_SIGNON);
+	/* LOG system_log(log_quit, NULL, NULL, OPT_LOG_BUDDY_SIGNON | OPT_LOG_MY_SIGNON); */
 
 #ifdef USE_SM
 	/* unplug */
 	session_end();
 #endif
 
+	/* Save the plugins we have loaded for next time. */
+	gaim_gtk_plugins_save();
+
 	/* and end it all... */
 	gtk_main_quit();
 }
@@ -533,21 +505,103 @@ static GaimCoreUiOps core_ops =
 };
 
 static GaimCoreUiOps *
-gaim_get_gtk_core_ui_ops(void)
+gaim_gtk_core_get_ui_ops(void)
 {
 	return &core_ops;
 }
 
+static void
+show_usage(int mode, const char *name)
+{
+	char *text=NULL;
+
+	switch (mode) {
+	case 0:		/* full help text */
+		text=g_strdup_printf(_("Gaim %s\n"
+		       "Usage: %s [OPTION]...\n\n"
+		       "  -a, --acct          display account editor window\n"
+		       "  -w, --away[=MESG]   make away on signon (optional argument MESG specifies\n"
+		       "                      name of away message to use)\n"
+		       "  -l, --login[=NAME]  automatically login (optional argument NAME specifies\n"
+		       "                      account(s) to use, seperated by commas)\n"
+		       "  -n, --loginwin      don't automatically login; show login window\n"
+		       "  -u, --user=NAME     use account NAME\n"
+		       "  -c, --config=DIR    use DIR for config files\n"
+		       "  -d, --debug         print debugging messages to stdout\n"
+		       "  -v, --version       display the current version and exit\n"
+		       "  -h, --help          display this help and exit\n"), VERSION, name);
+		break;
+	case 1:		/* short message */
+		text=g_strdup_printf(_("Gaim %s. Try `%s -h' for more information.\n"), VERSION, name);
+		break;
+	}
+
+	if(text) {
+		char *text_conv;
+		GError *error=NULL;
+
+		/* tries to convert 'text' to users locale */
+		text_conv=g_locale_from_utf8(text,-1,NULL,NULL,&error);
+		if(text_conv) {
+			puts(text_conv);
+			g_free(text_conv);
+		}
+		/* use 'text' as a fallback */
+		else {
+			g_warning("%s\n", error->message);
+			g_error_free(error);
+			puts(text);
+		}
+		g_free(text);
+	}
+}
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+static void
+sn_error_trap_push(SnDisplay *display, Display *xdisplay)
+{
+	gdk_error_trap_push();
+}
+
+static void
+sn_error_trap_pop(SnDisplay *display, Display *xdisplay)
+{
+	gdk_error_trap_pop();
+}
+
+static void
+startup_notification_complete(void)
+{
+	Display *xdisplay;
+
+	xdisplay = GDK_DISPLAY();
+	sn_display = sn_display_new(xdisplay,
+								sn_error_trap_push,
+								sn_error_trap_pop);
+	sn_context =
+		sn_launchee_context_new_from_environment(sn_display,
+												 DefaultScreen(xdisplay));
+
+	if (sn_context != NULL)
+	{
+		sn_launchee_context_complete(sn_context);
+		sn_launchee_context_unref(sn_context);
+
+		sn_display_unref(sn_display);
+	}
+}
+#endif /* HAVE_STARTUP_NOTIFICATION */
+
 /* FUCKING GET ME A TOWEL! */
 #ifdef _WIN32
-int gaim_main(int argc, char *argv[])
+int gaim_main(HINSTANCE hint, int argc, char *argv[])
 #else
 int main(int argc, char *argv[])
 #endif
 {
 	int opt_acct = 0, opt_help = 0, opt_version = 0, opt_login = 0, opt_nologin = 0, dologin_ret = -1;
 	char *opt_user_arg = NULL, *opt_login_arg = NULL;
-	char *opt_session_arg = NULL;
+	char *opt_session_arg = NULL, *opt_config_dir_arg = NULL;
 	char *plugin_search_paths[3];
 #if HAVE_SIGNAL_H
 	int sig_indx;	/* for setting up signal catching */
@@ -556,6 +610,7 @@ int main(int argc, char *argv[])
 #endif
 	int opt, opt_user = 0;
 	int i;
+	gboolean gui_check;
 
 	struct option long_options[] = {
 		{"acct", no_argument, NULL, 'a'},
@@ -564,7 +619,7 @@ int main(int argc, char *argv[])
 		/*{"login", optional_argument, NULL, 'l'}, */
 		{"loginwin", no_argument, NULL, 'n'},
 		{"user", required_argument, NULL, 'u'},
-		{"file", required_argument, NULL, 'f'},
+		{"config", required_argument, NULL, 'c'},
 		{"debug", no_argument, NULL, 'd'},
 		{"version", no_argument, NULL, 'v'},
 		{"session", required_argument, NULL, 's'},
@@ -693,16 +748,15 @@ int main(int argc, char *argv[])
 	   }
 	 */
 
-	gtk_set_locale();
-	gtk_init(&argc, &argv);
+	gui_check = gtk_init_check(&argc, &argv);
 
 	/* scan command-line options */
 	opterr = 1;
 	while ((opt = getopt_long(argc, argv,
 #ifndef _WIN32
-				  "adhu:f:vns:", 
+				  "adhu:c:vns:", 
 #else
-				  "adghu:f:vn", 
+				  "adhu:c:vn", 
 #endif
 				  long_options, NULL)) != -1) {
 		switch (opt) {
@@ -716,8 +770,9 @@ int main(int argc, char *argv[])
 		case 'd':	/* debug */
 			opt_debug = 1;
 			break;
-		case 'f':
-			opt_rcfile_arg = g_strdup(optarg);
+		case 'c':	/* use specified config dir */
+			set_gaim_user_dir(optarg);
+			opt_config_dir_arg = g_strdup(optarg);
 			break;
 		case 's':	/* use existing session ID */
 			opt_session_arg = g_strdup(optarg);
@@ -731,11 +786,6 @@ int main(int argc, char *argv[])
 		case 'n':       /* don't autologin */
 			opt_nologin = 1;
 			break;
-#ifdef _WIN32
-		case 'g':       /* debug GTK and GLIB */
-			opt_gdebug = 1;
-			break;
-#endif
 		case '?':
 		default:
 			show_usage(1, argv[0]);
@@ -744,45 +794,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-#ifdef _WIN32
-	/* We don't want a console window.. */
-	/*
-	 *  Any calls to the glib logging functions, result in a call to AllocConsole().
-	 *  ME and 98 will in such cases produce a console window (2000 not), despite
-	 *  being built as a windows app rather than a console app.  So we should either
-	 *  ignore messages by setting dummy log handlers, or redirect messages.
-	 *  This requires setting handlers for all domains (any lib which uses g_logging).
-	 */
-
-	g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);	
-	g_log_set_handler ("Gdk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-	g_log_set_handler ("Gtk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-	g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-	g_log_set_handler ("GModule", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-	g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-	g_log_set_handler ("GThread", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
-			   (opt_gdebug ? gaim_log_handler : gaim_dummy_log_handler),
-			   NULL);
-
-	/* g_print also makes a call to AllocConsole(), therefore a handler needs to be
-	   set here aswell */
-	if(!opt_debug)
-		g_set_print_handler( gaim_dummy_print );
-
-#endif
-
 	/* show help message */
 	if (opt_help) {
 		show_usage(0, argv[0]);
@@ -794,9 +805,20 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	gaim_gtk_stock_init();
+	if (!gui_check) {
+		char *display = gdk_get_display();
 
-	gaim_set_core_ui_ops(gaim_get_gtk_core_ui_ops());
+		g_warning("cannot open display: %s", display ? display : "unset");
+		g_free(display);
+
+		return 1;
+	}
+
+#ifdef _WIN32
+        wgaim_init(hint);
+#endif
+	gaim_core_set_ui_ops(gaim_gtk_core_get_ui_ops());
+	gaim_eventloop_set_ui_ops(gaim_gtk_eventloop_get_ui_ops());
 
 	if (!gaim_core_init(GAIM_GTK_UI)) {
 		fprintf(stderr,
@@ -807,7 +829,7 @@ int main(int argc, char *argv[])
 
 	plugin_search_paths[0] = LIBDIR;
 	plugin_search_paths[1] = gaim_user_dir();
-	plugin_search_paths[2] = g_strdup_printf("%s/plugins", gaim_user_dir());
+	plugin_search_paths[2] = g_build_filename(gaim_user_dir(), "plugins", NULL);
 
 	gaim_plugins_set_search_paths(sizeof(plugin_search_paths) /
 								  sizeof(*plugin_search_paths),
@@ -817,9 +839,6 @@ int main(int argc, char *argv[])
 
 	gaim_plugins_probe(NULL);
 
-	gaim_prefs_load();
-
-
 	/* we only read ~/.gaimrc (load_prefs()) if there is no accounts.xml
 	 * since prefs.xml existed alongside ~/.gaim in 0.64 */
 	if (!gaim_accounts_load()) {
@@ -827,30 +846,35 @@ int main(int argc, char *argv[])
 		gaim_prefs_sync();
 	}
 
+	gaim_set_blist(gaim_blist_new());
+	gaim_blist_load();
+
+	gaim_prefs_load();
+
+	gaim_prefs_update_old();
+	gaim_gtk_prefs_update_old();
+
 	/* load plugins we had when we quit */
 	gaim_plugins_load_saved("/gaim/gtk/plugins/loaded");
 
 	gaim_pounces_load();
 	gaim_status_load();
 
-#ifdef _WIN32
-	/* Various win32 initializations */
-	wgaim_init();
-#endif
-
-	gaim_set_blist(gaim_blist_new());
-	gaim_blist_load();
-
 	load_pounces();
 	ui_main();
 
 #ifdef USE_SM
-	session_init(argv[0], opt_session_arg);
+	session_init(argv[0], opt_session_arg, opt_config_dir_arg);
 #endif
 	if (opt_session_arg != NULL) {
 		g_free(opt_session_arg);
 		opt_session_arg = NULL;
-	};
+	}
+	
+	if (opt_config_dir_arg != NULL) {
+		g_free(opt_config_dir_arg);
+		opt_config_dir_arg = NULL;
+	}
 
 	/* set the default username */
 	if (opt_user_arg != NULL) {
@@ -879,6 +903,10 @@ int main(int argc, char *argv[])
 	} else if ((dologin_ret == -1) && !gaim_connections_get_all())
 		show_login();
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+	startup_notification_complete();
+#endif
+
 	gtk_main();
 	gaim_sound_shutdown();
 #ifdef _WIN32
