@@ -1,7 +1,9 @@
 /*
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,23 +19,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
-/*
- * ----------------
- * The Plug-in plugin
- *
- * Plugin support is currently being maintained by Mike Saraf
- * msaraf@dwc.edu
- *
- * Well, I didn't see any work done on it for a while, so I'm going to try
- * my hand at it. - Eric warmenhoven@yahoo.com
- *
- * Mike is my roomate.  I can assure you that he's lazy :-P
- * -- Rob rob@marko.net
- *
- * Yeah, well now I'm re-writing a good portion of it! The perl stuff was
- * a hack. Tsk tsk! -- Christian <chipx86@gnupdate.org>
- */
 #include "internal.h"
 
 #include "accountopt.h"
@@ -54,10 +39,29 @@
 #endif
 #endif
 
-static GList *loaded_plugins = NULL;
-static GList *plugins = NULL;
-static GList *plugin_loaders = NULL;
+typedef struct
+{
+	GHashTable *commands;
+	size_t command_count;
+
+} GaimPluginIpcInfo;
+
+typedef struct
+{
+	GaimCallback func;
+	GaimSignalMarshalFunc marshal;
+
+	int num_params;
+	GaimValue **params;
+	GaimValue *ret_value;
+
+} GaimPluginIpcCommand;
+
+static GList *loaded_plugins   = NULL;
+static GList *plugins          = NULL;
+static GList *plugin_loaders   = NULL;
 static GList *protocol_plugins = NULL;
+static GList *load_queue       = NULL;
 
 static size_t search_path_count = 0;
 static char **search_paths = NULL;
@@ -69,6 +73,15 @@ static void *load_cb_data = NULL;
 static void (*unload_cb)(GaimPlugin *, void *) = NULL;
 static void *unload_cb_data = NULL;
 
+
+void *
+gaim_plugins_get_handle(void)
+{
+	static int handle;
+	return &handle;
+}
+
+
 #ifdef GAIM_PLUGINS
 static int
 is_so_file(const char *filename, const char *ext)
@@ -131,10 +144,17 @@ static gint
 compare_prpl(GaimPlugin *a, GaimPlugin *b)
 {
 	/* neg if a before b, 0 if equal, pos if a after b */
-	return ((GAIM_IS_PROTOCOL_PLUGIN(a)
-			 ? GAIM_PLUGIN_PROTOCOL_INFO(a)->protocol : -1) -
-			((GAIM_IS_PROTOCOL_PLUGIN(b)
-			 ? GAIM_PLUGIN_PROTOCOL_INFO(b)->protocol : -1)));
+	if(GAIM_IS_PROTOCOL_PLUGIN(a)) {
+		if(GAIM_IS_PROTOCOL_PLUGIN(b))
+			return strcmp(a->info->name, b->info->name);
+		else
+			return -1;
+	} else {
+		if(GAIM_IS_PROTOCOL_PLUGIN(b))
+			return 1;
+		else
+			return 0;
+	}
 }
 
 GaimPlugin *
@@ -158,6 +178,7 @@ gaim_plugin_probe(const char *filename)
 	GaimPlugin *loader;
 	gboolean (*gaim_init_plugin)(GaimPlugin *);
 
+	gaim_debug_misc("plugins", "probing %s\n", filename);
 	g_return_val_if_fail(filename != NULL, NULL);
 
 	if (!g_file_test(filename, G_FILE_TEST_EXISTS))
@@ -218,6 +239,14 @@ gaim_plugin_probe(const char *filename)
 		return NULL;
 	}
 
+	if (plugin->info->api_version != GAIM_PLUGIN_API_VERSION)
+	{
+		gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable: API version mismatch %d (need %d)\n",
+				   plugin->path, plugin->info->api_version, GAIM_PLUGIN_API_VERSION);
+		gaim_plugin_destroy(plugin);
+		return NULL;
+	}
+
 	return plugin;
 #else
 	return NULL;
@@ -301,10 +330,12 @@ gaim_plugin_load(GaimPlugin *plugin)
 	if (dep_list != NULL)
 		g_list_free(dep_list);
 
-	if (plugin->native_plugin) {
-		if (plugin->info != NULL) {
-			if (plugin->info->load != NULL)
-				plugin->info->load(plugin);
+	if (plugin->native_plugin)
+	{
+		if (plugin->info != NULL && plugin->info->load != NULL)
+		{
+			if (!plugin->info->load(plugin))
+				return FALSE;
 		}
 	}
 	else {
@@ -319,7 +350,10 @@ gaim_plugin_load(GaimPlugin *plugin)
 		loader_info = GAIM_PLUGIN_LOADER_INFO(loader);
 
 		if (loader_info->load != NULL)
-			loader_info->load(plugin);
+		{
+			if (!loader_info->load(plugin))
+				return FALSE;
+		}
 	}
 
 	loaded_plugins = g_list_append(loaded_plugins, plugin);
@@ -330,6 +364,8 @@ gaim_plugin_load(GaimPlugin *plugin)
 	if (load_cb != NULL)
 		load_cb(plugin, load_cb_data);
 
+	gaim_signal_emit(gaim_plugins_get_handle(), "plugin-load", plugin);
+
 	return TRUE;
 
 #else
@@ -395,11 +431,15 @@ gaim_plugin_unload(GaimPlugin *plugin)
 	}
 
 	gaim_signals_disconnect_by_handle(plugin);
+	gaim_plugin_ipc_unregister_all(plugin);
 
 	/* TODO */
 	if (unload_cb != NULL)
 		unload_cb(plugin, unload_cb_data);
 
+	/* I suppose this is the right place to call this... */
+	gaim_signal_emit(gaim_plugins_get_handle(), "plugin-unload", plugin);
+
 	return TRUE;
 #else
 	return TRUE;
@@ -436,30 +476,45 @@ gaim_plugin_destroy(GaimPlugin *plugin)
 
 	plugins = g_list_remove(plugins, plugin);
 
+	if (load_queue != NULL)
+		load_queue = g_list_remove(load_queue, plugin);
+
 	if (plugin->info != NULL && plugin->info->dependencies != NULL)
 		g_list_free(plugin->info->dependencies);
 
-	if (plugin->native_plugin) {
-
-		if (plugin->info != NULL && plugin->info->type == GAIM_PLUGIN_LOADER) {
+	if (plugin->native_plugin)
+	{
+		if (plugin->info != NULL && plugin->info->type == GAIM_PLUGIN_LOADER)
+		{
+			GaimPluginLoaderInfo *loader_info;
 			GList *exts, *l, *next_l;
 			GaimPlugin *p2;
 
-			for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
-				 exts != NULL;
-				 exts = exts->next) {
+			loader_info = GAIM_PLUGIN_LOADER_INFO(plugin);
 
-				for (l = gaim_plugins_get_all(); l != NULL; l = next_l) {
-					next_l = l->next;
+			if (loader_info != NULL && plugin->info->api_version >= 3 &&
+				loader_info->exts != NULL)
+			{
+				for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
+					 exts != NULL;
+					 exts = exts->next) {
 
-					p2 = l->data;
+					for (l = gaim_plugins_get_all(); l != NULL; l = next_l)
+					{
+						next_l = l->next;
 
-					if (p2->path != NULL && is_so_file(p2->path, exts->data))
-						gaim_plugin_destroy(p2);
+						p2 = l->data;
+
+						if (p2->path != NULL &&
+							is_so_file(p2->path, exts->data))
+						{
+							gaim_plugin_destroy(p2);
+						}
+					}
 				}
-			}
 
-			g_list_free(GAIM_PLUGIN_LOADER_INFO(plugin)->exts);
+				g_list_free(loader_info->exts);
+			}
 
 			plugin_loaders = g_list_remove(plugin_loaders, plugin);
 		}
@@ -470,13 +525,15 @@ gaim_plugin_destroy(GaimPlugin *plugin)
 		if (plugin->handle != NULL)
 			g_module_close(plugin->handle);
 	}
-	else {
+	else
+	{
 		GaimPlugin *loader;
 		GaimPluginLoaderInfo *loader_info;
 
 		loader = find_loader_for_plugin(plugin);
 
-		if (loader != NULL) {
+		if (loader != NULL)
+		{
 			loader_info = GAIM_PLUGIN_LOADER_INFO(loader);
 
 			if (loader_info->destroy != NULL)
@@ -499,6 +556,201 @@ gaim_plugin_is_loaded(const GaimPlugin *plugin)
 	return plugin->loaded;
 }
 
+/**************************************************************************
+ * Plugin IPC
+ **************************************************************************/
+static void
+destroy_ipc_info(void *data)
+{
+	GaimPluginIpcCommand *ipc_command = (GaimPluginIpcCommand *)data;
+	int i;
+
+	for (i = 0; i < ipc_command->num_params; i++)
+		gaim_value_destroy(ipc_command->params[i]);
+
+	if (ipc_command->ret_value != NULL)
+		gaim_value_destroy(ipc_command->ret_value);
+
+	g_free(ipc_command);
+}
+
+gboolean
+gaim_plugin_ipc_register(GaimPlugin *plugin, const char *command,
+						 GaimCallback func, GaimSignalMarshalFunc marshal,
+						 GaimValue *ret_value, int num_params, ...)
+{
+	GaimPluginIpcInfo *ipc_info;
+	GaimPluginIpcCommand *ipc_command;
+
+	g_return_val_if_fail(plugin  != NULL, FALSE);
+	g_return_val_if_fail(command != NULL, FALSE);
+	g_return_val_if_fail(func    != NULL, FALSE);
+	g_return_val_if_fail(marshal != NULL, FALSE);
+
+	if (plugin->ipc_data == NULL)
+	{
+		ipc_info = plugin->ipc_data = g_new0(GaimPluginIpcInfo, 1);
+		ipc_info->commands = g_hash_table_new_full(g_str_hash, g_str_equal,
+												   g_free, destroy_ipc_info);
+	}
+	else
+		ipc_info = (GaimPluginIpcInfo *)plugin->ipc_data;
+
+	ipc_command = g_new0(GaimPluginIpcCommand, 1);
+	ipc_command->func       = func;
+	ipc_command->marshal    = marshal;
+	ipc_command->num_params = num_params;
+	ipc_command->ret_value  = ret_value;
+
+	if (num_params > 0)
+	{
+		va_list args;
+		int i;
+
+		ipc_command->params = g_new0(GaimValue *, num_params);
+
+		va_start(args, num_params);
+
+		for (i = 0; i < num_params; i++)
+			ipc_command->params[i] = va_arg(args, GaimValue *);
+
+		va_end(args);
+	}
+
+	g_hash_table_replace(ipc_info->commands, g_strdup(command), ipc_command);
+
+	ipc_info->command_count++;
+
+	return TRUE;
+}
+
+void
+gaim_plugin_ipc_unregister(GaimPlugin *plugin, const char *command)
+{
+	GaimPluginIpcInfo *ipc_info;
+
+	g_return_if_fail(plugin  != NULL);
+	g_return_if_fail(command != NULL);
+
+	ipc_info = (GaimPluginIpcInfo *)plugin->ipc_data;
+
+	if (ipc_info == NULL ||
+		g_hash_table_lookup(ipc_info->commands, command) == NULL)
+	{
+		gaim_debug_error("plugins",
+						 "IPC command '%s' was not registered for plugin %s\n",
+						 command, plugin->info->name);
+		return;
+	}
+
+	g_hash_table_remove(ipc_info->commands, command);
+
+	ipc_info->command_count--;
+
+	if (ipc_info->command_count == 0)
+	{
+		g_hash_table_destroy(ipc_info->commands);
+		g_free(ipc_info);
+
+		plugin->ipc_data = NULL;
+	}
+}
+
+void
+gaim_plugin_ipc_unregister_all(GaimPlugin *plugin)
+{
+	GaimPluginIpcInfo *ipc_info;
+
+	g_return_if_fail(plugin != NULL);
+
+	if (plugin->ipc_data == NULL)
+		return; /* Silently ignore it. */
+
+	ipc_info = (GaimPluginIpcInfo *)plugin->ipc_data;
+
+	g_hash_table_destroy(ipc_info->commands);
+	g_free(ipc_info);
+
+	plugin->ipc_data = NULL;
+}
+
+gboolean
+gaim_plugin_ipc_get_params(GaimPlugin *plugin, const char *command,
+						   GaimValue **ret_value, int *num_params,
+						   GaimValue ***params)
+{
+	GaimPluginIpcInfo *ipc_info;
+	GaimPluginIpcCommand *ipc_command;
+
+	g_return_val_if_fail(plugin  != NULL, FALSE);
+	g_return_val_if_fail(command != NULL, FALSE);
+
+	ipc_info = (GaimPluginIpcInfo *)plugin->ipc_data;
+
+	if (ipc_info == NULL ||
+		(ipc_command = g_hash_table_lookup(ipc_info->commands,
+										   command)) == NULL)
+	{
+		gaim_debug_error("plugins",
+						 "IPC command '%s' was not registered for plugin %s\n",
+						 command, plugin->info->name);
+
+		return FALSE;
+	}
+
+	if (num_params != NULL)
+		*num_params = ipc_command->num_params;
+
+	if (params != NULL)
+		*params = ipc_command->params;
+
+	if (ret_value != NULL)
+		*ret_value = ipc_command->ret_value;
+
+	return TRUE;
+}
+
+void *
+gaim_plugin_ipc_call(GaimPlugin *plugin, const char *command,
+					 gboolean *ok, ...)
+{
+	GaimPluginIpcInfo *ipc_info;
+	GaimPluginIpcCommand *ipc_command;
+	va_list args;
+	void *ret_value;
+
+	if (ok != NULL)
+		*ok = FALSE;
+
+	g_return_val_if_fail(plugin  != NULL, NULL);
+	g_return_val_if_fail(command != NULL, NULL);
+
+	ipc_info = (GaimPluginIpcInfo *)plugin->ipc_data;
+
+	if (ipc_info == NULL ||
+		(ipc_command = g_hash_table_lookup(ipc_info->commands,
+										   command)) == NULL)
+	{
+		gaim_debug_error("plugins",
+						 "IPC command '%s' was not registered for plugin %s\n",
+						 command, plugin->info->name);
+
+		return NULL;
+	}
+
+	va_start(args, ok);
+	ipc_command->marshal(ipc_command->func, args, NULL, &ret_value);
+	va_end(args);
+
+	if (ok != NULL)
+		*ok = TRUE;
+
+	return ret_value;
+}
+
+/**************************************************************************
+ * Plugins subsystem
+ **************************************************************************/
 void
 gaim_plugins_set_search_paths(size_t count, char **paths)
 {
@@ -558,8 +810,28 @@ gaim_plugins_load_saved(const char *key)
 
 	files = gaim_prefs_get_string_list(key);
 
-	for (f = files; f; f = f->next) {
-		gaim_plugin_load(gaim_plugin_probe(f->data));
+	for (f = files; f; f = f->next)
+	{
+		char *filename = g_path_get_basename(f->data);
+		GaimPlugin *plugin = NULL;
+
+		if (filename != NULL)
+		{
+			if ((plugin = gaim_plugins_find_with_basename(filename)) != NULL)
+			{
+				gaim_debug_info("plugins", "Loading saved plugin %s\n",
+								filename);
+				gaim_plugin_load(plugin);
+			}
+			else
+			{
+				gaim_debug_error("plugins", "Unable to find saved plugin %s\n",
+								 filename);
+			}
+
+			g_free(filename);
+		}
+
 		g_free(f->data);
 	}
 
@@ -578,9 +850,22 @@ gaim_plugins_probe(const char *ext)
 	GaimPlugin *plugin;
 	size_t i;
 
+	void *handle;
+
 	if (!g_module_supported())
 		return;
 
+	handle = gaim_plugins_get_handle();
+
+	gaim_debug_info("plugins", "registering plugin-load signal\n");
+	gaim_signal_register(handle, "plugin-load", gaim_marshal_VOID__POINTER, NULL,
+			1, gaim_value_new(GAIM_TYPE_SUBTYPE, GAIM_SUBTYPE_PLUGIN));
+
+	gaim_debug_info("plugins", "registering plugin-unload signal\n");
+	gaim_signal_register(handle, "plugin-unload", gaim_marshal_VOID__POINTER, NULL,
+			1, gaim_value_new(GAIM_TYPE_SUBTYPE, GAIM_SUBTYPE_PLUGIN));
+
+
 	for (i = 0; i < search_path_count; i++) {
 		if (search_paths[i] == NULL)
 			continue;
@@ -601,6 +886,66 @@ gaim_plugins_probe(const char *ext)
 		}
 	}
 
+	/* See if we have any plugins waiting to load. */
+	while (load_queue != NULL)
+	{
+		plugin = (GaimPlugin *)load_queue->data;
+
+		load_queue = g_list_remove(load_queue, plugin);
+
+		if (plugin == NULL || plugin->info == NULL)
+			continue;
+
+		if (plugin->info->type == GAIM_PLUGIN_LOADER)
+		{
+			GList *exts;
+
+			/* We'll just load this right now. */
+			if (!gaim_plugin_load(plugin))
+			{
+				gaim_plugin_destroy(plugin);
+
+				continue;
+			}
+
+			plugin_loaders = g_list_append(plugin_loaders, plugin);
+
+			for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
+				 exts != NULL;
+				 exts = exts->next)
+			{
+				gaim_plugins_probe(exts->data);
+			}
+		}
+		else if (plugin->info->type == GAIM_PLUGIN_PROTOCOL)
+		{
+			/* We'll just load this right now. */
+			if (!gaim_plugin_load(plugin))
+			{
+				gaim_plugin_destroy(plugin);
+
+				continue;
+			}
+
+			if (gaim_find_prpl(plugin->info->id))
+			{
+				/* Nothing to see here--move along, move along */
+				gaim_plugin_destroy(plugin);
+
+				continue;
+			}
+
+			protocol_plugins = g_list_insert_sorted(protocol_plugins, plugin,
+													(GCompareFunc)compare_prpl);
+		}
+	}
+
+	if (load_queue != NULL)
+	{
+		g_list_free(load_queue);
+		load_queue = NULL;
+	}
+
 	if (probe_cb != NULL)
 		probe_cb(probe_cb_data);
 
@@ -615,48 +960,37 @@ gaim_plugin_register(GaimPlugin *plugin)
 	if (g_list_find(plugins, plugin))
 		return TRUE;
 
-	/* Special exception for loader plugins. We want them loaded NOW! */
-	if (plugin->info->type == GAIM_PLUGIN_LOADER) {
-		GList *exts;
+	if (plugin->info->type == GAIM_PLUGIN_LOADER)
+	{
+		GaimPluginLoaderInfo *loader_info;
 
-		/* We'll just load this right now. */
-		if (!gaim_plugin_load(plugin)) {
-
-			gaim_plugin_destroy(plugin);
+		loader_info = GAIM_PLUGIN_LOADER_INFO(plugin);
 
+		if (loader_info == NULL ||
+			loader_info->api_version != GAIM_LOADER_API_VERSION)
+		{
+			gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable: API version mismatch %d (need %d)\n",
+							   plugin->path, loader_info->api_version, GAIM_LOADER_API_VERSION);
 			return FALSE;
 		}
 
-		plugin_loaders = g_list_append(plugin_loaders, plugin);
-
-		for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
-			 exts != NULL;
-			 exts = exts->next) {
-
-			gaim_plugins_probe(exts->data);
-		}
+		load_queue = g_list_append(load_queue, plugin);
 	}
-	else if (plugin->info->type == GAIM_PLUGIN_PROTOCOL) {
+	else if (plugin->info->type == GAIM_PLUGIN_PROTOCOL)
+	{
+		GaimPluginProtocolInfo *prpl_info;
 
-		/* We'll just load this right now. */
-		if (!gaim_plugin_load(plugin)) {
-
-			gaim_plugin_destroy(plugin);
-
-			return FALSE;
-		}
-	
-		if (GAIM_PLUGIN_PROTOCOL_INFO(plugin)->protocol == GAIM_PROTO_ICQ ||
-			gaim_find_prpl(GAIM_PLUGIN_PROTOCOL_INFO(plugin)->protocol)) {
-
-			/* Nothing to see here--move along, move along */
-			gaim_plugin_destroy(plugin);
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
 
+		if (prpl_info == NULL ||
+			prpl_info->api_version != GAIM_PRPL_API_VERSION)
+		{
+			gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable: API version mismatch %d (need %d)\n",
+							   plugin->path, prpl_info->api_version, GAIM_PRPL_API_VERSION);
 			return FALSE;
 		}
 
-		protocol_plugins = g_list_insert_sorted(protocol_plugins, plugin,
-												(GCompareFunc)compare_prpl);
+		load_queue = g_list_append(load_queue, plugin);
 	}
 
 	plugins = g_list_append(plugins, plugin);
@@ -756,6 +1090,33 @@ gaim_plugins_find_with_filename(const char *filename)
 	return NULL;
 }
 
+GaimPlugin *
+gaim_plugins_find_with_basename(const char *basename)
+{
+	GaimPlugin *plugin;
+	GList *l;
+
+	g_return_val_if_fail(basename != NULL, NULL);
+
+	for (l = plugins; l != NULL; l = l->next)
+	{
+		char *tmp;
+
+		plugin = (GaimPlugin *)l->data;
+
+		if (plugin->path != NULL) {
+			tmp = g_path_get_basename(plugin->path);
+			if (!strcmp(tmp, basename)) {
+				g_free(tmp);
+				return plugin;
+			}
+			g_free(tmp);
+		}
+	}
+
+	return NULL;
+}
+
 GaimPlugin *
 gaim_plugins_find_with_id(const char *id)
 {
@@ -793,3 +1154,15 @@ gaim_plugins_get_all(void)
 	return plugins;
 }
 
+
+GaimPluginAction *
+gaim_plugin_action_new(char* label, void (*callback)(GaimPluginAction *))
+{
+	GaimPluginAction *act = g_new0(GaimPluginAction, 1);
+
+	act->label = label;
+	act->callback = callback;
+
+	return act;
+}
+
