@@ -1,6 +1,6 @@
 /*
  * gaim - Gadu-Gadu Protocol Plugin
- * $Id: gg.c,v 1.95 2004/04/22 01:53:12 chipx86 Exp $
+ * $Id: gg.c,v 1.104 2004/05/23 17:27:42 thekingant Exp $
  *
  * Copyright (C) 2001 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
  *
@@ -26,8 +26,8 @@
 
 #include "account.h"
 #include "accountopt.h"
+#include "blist.h"
 #include "debug.h"
-#include "multi.h"
 #include "notify.h"
 #include "proxy.h"
 #include "prpl.h"
@@ -244,27 +244,154 @@ static GList *agg_away_states(GaimConnection *gc)
 }
 
 /* Enhance these functions, more options and such stuff */
-static GList *agg_buddy_menu(GaimConnection *gc, const char *who)
+static GList *agg_buddy_menu(GaimBuddy *buddy)
 {
 	GList *m = NULL;
-	struct proto_buddy_menu *pbm;
-	GaimBuddy *b = gaim_find_buddy(gc->account, who);
+	GaimBlistNodeAction *act;
+
 	static char buf[AGG_BUF_LEN];
+	g_snprintf(buf, sizeof(buf), _("Status: %s"), get_away_text(buddy->uc));
 
-	if (!b) {
-		return m;
-	}
-
-	pbm = g_new0(struct proto_buddy_menu, 1);
-	g_snprintf(buf, sizeof(buf), _("Status: %s"), get_away_text(b->uc));
-	pbm->label = buf;
-	pbm->callback = NULL;
-	pbm->gc = gc;
-	m = g_list_append(m, pbm);
+	/* um... this seems silly. since in this pass, I'm only converting
+	   over the menu building, I'm not going to mess with it though */
+	/* XXX: shouldn't this be in the tooltip instead? */
+	act = gaim_blist_node_action_new(buf, NULL, NULL);
+	m = g_list_append(m, act);
 
 	return m;
 }
 
+
+static GList *
+agg_blist_node_menu(GaimBlistNode *node)
+{
+	if(GAIM_BLIST_NODE_IS_BUDDY(node)) {
+		return agg_buddy_menu((GaimBuddy *) node);
+	} else {
+		return NULL;
+	}
+}
+
+
+static void agg_load_buddy_list(GaimConnection *gc, char *buddylist)
+{
+	struct agg_data *gd = (struct agg_data *)gc->proto_data;
+	gchar *ptr = buddylist;
+	gchar **users_tbl;
+	int i;
+	uin_t *userlist = NULL;
+	int userlist_size = 0;
+
+	users_tbl = g_strsplit(ptr, "\r\n", AGG_PUBDIR_MAX_ENTRIES);
+
+	/* Parse array of Buddies List */
+	for (i = 0; users_tbl[i] != NULL; i++) {
+		gchar **data_tbl;
+		gchar *name, *show;
+
+		if (strlen(users_tbl[i])==0) {
+			gaim_debug(GAIM_DEBUG_MISC, "gg",
+					   "import_buddies_server_results: users_tbl[i] is empty\n");
+			continue;
+		}
+
+		data_tbl = g_strsplit(users_tbl[i], ";", 8);
+
+		show = charset_convert(data_tbl[3], "CP1250", "UTF-8");
+		name = data_tbl[6];
+
+		if (invalid_uin(name)) {
+			continue;
+		}
+
+		gaim_debug(GAIM_DEBUG_MISC, "gg",
+				   "import_buddies_server_results: uin: %s\n", name);
+		if (!gaim_find_buddy(gc->account, name)) {
+			GaimBuddy *b;
+			GaimGroup *g;
+			/* Default group if none specified on server */
+			gchar *group = g_strdup("Gadu-Gadu");
+			if (strlen(data_tbl[5])) {
+				gchar **group_tbl = g_strsplit(data_tbl[5], ",", 2);
+				if (strlen(group_tbl[0])) {
+					g_free(group);
+					group = g_strdup(group_tbl[0]);
+				}
+				g_strfreev(group_tbl);
+			}
+			/* Add Buddy to our userlist */
+			if (!(g = gaim_find_group(group))) {
+				g = gaim_group_new(group);
+				gaim_blist_add_group(g, NULL);
+			}
+			b = gaim_buddy_new(gc->account, name, strlen(show) ? show : NULL);
+			gaim_blist_add_buddy(b,NULL,g,NULL);
+			gaim_blist_save();
+
+			userlist_size++;
+			userlist = g_renew(uin_t, userlist, userlist_size);
+			userlist[userlist_size - 1] =
+			    (uin_t) strtol((char *)name, (char **)NULL, 10);
+
+			g_free(group);
+		}
+		g_free(show);
+		g_strfreev(data_tbl);
+	}
+	g_strfreev(users_tbl);
+
+	if (userlist) {
+	    gg_notify(gd->sess, userlist, userlist_size);
+	    g_free(userlist);
+	}
+}
+
+static void agg_save_buddy_list (GaimConnection *gc, char *existlist)
+{
+    GaimBlistNode *gnode, *cnode, *bnode;
+    char *buddylist = g_strdup(existlist ? existlist : "");
+    char *ptr;
+    struct agg_data *gd = (struct agg_data *)gc->proto_data;
+
+    for(gnode = gaim_get_blist()->root; gnode; gnode = gnode->next) {
+		GaimGroup *g = (GaimGroup *)gnode;
+		if(!GAIM_BLIST_NODE_IS_GROUP(gnode))
+			continue;
+		for(cnode = gnode->child; cnode; cnode = cnode->next) {
+			if(!GAIM_BLIST_NODE_IS_CONTACT(cnode))
+				continue;
+			for(bnode = cnode->child; bnode; bnode = bnode->next) {
+				GaimBuddy *b = (GaimBuddy *)bnode;
+
+				if(!GAIM_BLIST_NODE_IS_BUDDY(bnode))
+					continue;
+
+				if(b->account == gc->account) {
+					gchar *newdata;
+					/* GG Number */
+					gchar *name = b->name;
+					/* GG Pseudo */
+					gchar *show = b->alias ? b->alias : b->name;
+					/* Group Name */
+					gchar *gname = g->name;
+
+					newdata = g_strdup_printf("%s;%s;%s;%s;%s;%s;%s;%s%s\r\n",
+							show, show, show, show, "", gname, name, "", "");
+
+					ptr = buddylist;
+					buddylist = g_strconcat(ptr, newdata, NULL);
+
+					g_free(newdata);
+					g_free(ptr);
+				}
+			}
+		}
+	}
+
+	/* save the list to the gadu gadu server */
+	gg_userlist_request(gd->sess, GG_USERLIST_PUT, buddylist);
+}
+
 static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 {
 	GaimConnection *gc = data;
@@ -347,6 +474,34 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 			}
 		}
 		break;
+	case GG_EVENT_NOTIFY60:
+		{
+			gchar user[20];
+			struct gg_notify_reply60 *n = (void *)e->event.notify60;
+			guint status;
+
+			while (n->uin) {
+				switch (n->status) {
+				case GG_STATUS_NOT_AVAIL:
+					status = UC_UNAVAILABLE;
+					break;
+				case GG_STATUS_AVAIL:
+				case GG_STATUS_BUSY:
+				case GG_STATUS_INVISIBLE:
+					status = UC_NORMAL | (n->status << 5);
+					break;
+				default:
+					status = UC_NORMAL;
+					break;
+				}
+
+				g_snprintf(user, sizeof(user), "%lu", (long unsigned int)n->uin);
+				serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
+						status);
+				n++;
+			}
+		}
+		break;
 	case GG_EVENT_STATUS:
 		{
 			gchar user[20];
@@ -371,11 +526,55 @@ static void main_callback(gpointer data, gint source, GaimInputCondition cond)
 					status);
 		}
 		break;
+	case GG_EVENT_STATUS60:
+		{
+			gchar user[20];
+			guint status;
+
+			switch (e->event.status60.status) {
+			case GG_STATUS_NOT_AVAIL:
+				status = UC_UNAVAILABLE;
+				break;
+			case GG_STATUS_AVAIL:
+			case GG_STATUS_BUSY:
+			case GG_STATUS_INVISIBLE:
+				status = UC_NORMAL | (e->event.status60.status << 5);
+				break;
+			default:
+				status = UC_NORMAL;
+				break;
+			}
+
+			g_snprintf(user, sizeof(user), "%lu", e->event.status60.uin);
+			serv_got_update(gc, user, (status == UC_UNAVAILABLE) ? 0 : 1, 0, 0, 0,
+					status);
+		}
+		break;
 	case GG_EVENT_ACK:
 		gaim_debug(GAIM_DEBUG_MISC, "gg",
 				   "main_callback: message %d to %lu sent with status %d\n",
 			     e->event.ack.seq, e->event.ack.recipient, e->event.ack.status);
 		break;
+	case GG_EVENT_USERLIST:
+	{
+	    gaim_debug(GAIM_DEBUG_MISC, "gg", "main_callback: received userlist reply\n");
+	    switch (e->event.userlist.type) {
+		case GG_USERLIST_GET_REPLY:
+		{
+
+			if (e->event.userlist.reply) {
+			    agg_load_buddy_list(gc, e->event.userlist.reply);
+			}
+			break;
+		}
+
+		case GG_USERLIST_PUT_REPLY:
+		{
+		    /* ignored */
+		}
+	    }
+	}
+
 	default:
 		gaim_debug(GAIM_DEBUG_ERROR, "gg",
 				   "main_callback: unsupported event %d\n", e->type);
@@ -415,16 +614,16 @@ void login_callback(gpointer data, gint source, GaimInputCondition cond)
 	gaim_debug(GAIM_DEBUG_INFO, "gg", "Checking State.\n");
 	switch (gd->sess->state) {
 	case GG_STATE_READING_DATA:
-		gaim_connection_update_progress(gc, _("Reading data"), 2, GG_CONNECT_STEPS);
+		gaim_connection_update_progress(gc, _("Reading data"), 1, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_CONNECTING_GG:
-		gaim_connection_update_progress(gc, _("Balancer handshake"), 3, GG_CONNECT_STEPS);
+		gaim_connection_update_progress(gc, _("Balancer handshake"), 2, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_READING_KEY:
-		gaim_connection_update_progress(gc, _("Reading server key"), 4, GG_CONNECT_STEPS);
+		gaim_connection_update_progress(gc, _("Reading server key"), 3, GG_CONNECT_STEPS);
 		break;
 	case GG_STATE_READING_REPLY:
-		gaim_connection_update_progress(gc, _("Exchanging key hash"), 5, GG_CONNECT_STEPS);
+		gaim_connection_update_progress(gc, _("Exchanging key hash"), 4, GG_CONNECT_STEPS);
 		break;
 	default:
 		gaim_debug(GAIM_DEBUG_INFO, "gg", "No State found\n");
@@ -517,7 +716,7 @@ static void agg_login(GaimAccount *account)
 
 	gd->sess = g_new0(struct gg_session, 1);
 
-	gaim_connection_update_progress(gc, _("Looking up GG server"), 1, GG_CONNECT_STEPS);
+	gaim_connection_update_progress(gc, _("Looking up GG server"), 0, GG_CONNECT_STEPS);
 
 	if (invalid_uin(account->username)) {
 		gaim_connection_error(gc, _("Invalid Gadu-Gadu UIN specified"));
@@ -588,6 +787,7 @@ static void agg_add_buddy(GaimConnection *gc, const char *who, GaimGroup *group)
 	if (invalid_uin(who))
 		return;
 	gg_add_notify(gd->sess, strtol(who, (char **)NULL, 10));
+	agg_save_buddy_list(gc, NULL);
 }
 
 static void agg_rem_buddy(GaimConnection *gc, const char *who, const char *group)
@@ -596,6 +796,7 @@ static void agg_rem_buddy(GaimConnection *gc, const char *who, const char *group
 	if (invalid_uin(who))
 		return;
 	gg_remove_notify(gd->sess, strtol(who, (char **)NULL, 10));
+	agg_save_buddy_list(gc, NULL);
 }
 
 static void agg_add_buddies(GaimConnection *gc, GList *whos)
@@ -618,6 +819,8 @@ static void agg_add_buddies(GaimConnection *gc, GList *whos)
 		gg_notify(gd->sess, userlist, userlist_size);
 		g_free(userlist);
 	}
+
+	agg_save_buddy_list(gc, NULL);
 }
 
 static void search_results(GaimConnection *gc, gchar *webdata)
@@ -729,11 +932,14 @@ static void search_results(GaimConnection *gc, gchar *webdata)
 }
 
 static void
-change_pass(GaimConnection *gc)
+change_pass(GaimPluginAction *action)
 {
-	gaim_account_request_change_password(gaim_connection_get_account(gc));
+	GaimConnection *gc = (GaimConnection *) action->context;
+	GaimAccount *account = gaim_connection_get_account(gc);
+	gaim_account_request_change_password(account);
 }
 
+#if 0
 static void import_buddies_server_results(GaimConnection *gc, gchar *webdata)
 {
 	gchar *ptr;
@@ -839,6 +1045,7 @@ static void delete_buddies_server_results(GaimConnection *gc, gchar *webdata)
 					  _("Couldn't delete Buddy List from Gadu-Gadu server"),
 					  NULL);
 }
+#endif
 
 static void password_change_server_results(GaimConnection *gc, gchar *webdata)
 {
@@ -903,6 +1110,7 @@ static void http_results(gpointer data, gint source, GaimInputCondition cond)
 	case AGG_HTTP_SEARCH:
 		search_results(gc, webdata);
 		break;
+#if 0
 	case AGG_HTTP_USERLIST_IMPORT:
 		import_buddies_server_results(gc, webdata);
 		break;
@@ -912,6 +1120,7 @@ static void http_results(gpointer data, gint source, GaimInputCondition cond)
 	case AGG_HTTP_USERLIST_DELETE:
 	        delete_buddies_server_results(gc, webdata);
 		break;
+#endif
 	case AGG_HTTP_PASSWORD_CHANGE:
 		password_change_server_results(gc, webdata);
 		break;
@@ -980,6 +1189,7 @@ static void http_req_callback(gpointer data, gint source, GaimInputCondition con
 	hdata->inpa = gaim_input_add(source, GAIM_INPUT_READ, http_results, hdata);
 }
 
+#if 0
 static void import_buddies_server(GaimConnection *gc)
 {
 	struct agg_http *hi = g_new0(struct agg_http, 1);
@@ -1102,7 +1312,9 @@ static void delete_buddies_server(GaimConnection *gc)
 		return;
 	}
 }
+#endif
 
+#if 0
 static void agg_dir_search(GaimConnection *gc, const char *first, const char *middle,
 			   const char *last, const char *maiden, const char *city, const char *state,
 			   const char *country, const char *email)
@@ -1152,6 +1364,7 @@ static void agg_dir_search(GaimConnection *gc, const char *first, const char *mi
 		return;
 	}
 }
+#endif
 
 static void agg_change_passwd(GaimConnection *gc, const char *old, const char *new)
 {
@@ -1176,7 +1389,7 @@ static void agg_change_passwd(GaimConnection *gc, const char *old, const char *n
 	g_free(eold);
 
 	if (gaim_proxy_connect(gc->account, GG_REGISTER_HOST, GG_REGISTER_PORT, http_req_callback, hpass) < 0) {
-	       	gaim_notify_error(gc, NULL,
+		gaim_notify_error(gc, NULL,
 							  _("Unable to change Gadu-Gadu password"),
 							  _("Gaim was unable to change your password "
 								"due to an error connecting to the "
@@ -1185,49 +1398,36 @@ static void agg_change_passwd(GaimConnection *gc, const char *old, const char *n
 		g_free(hpass->request);
 		g_free(hpass);
 		return;
-	}                                        
+	}
 }
 
-static GList *agg_actions(GaimConnection *gc)
+static GList *agg_actions(GaimPlugin *plugin, gpointer context)
 {
 	GList *m = NULL;
-	struct proto_actions_menu *pam;
+	GaimPluginAction *act = NULL;
 
 #if 0
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Directory Search");
-	pam->callback = show_find_info;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
-
+	act = gaim_plugin_action_new(_("Directory Search"), show_find_info);
+	m = g_list_append(m, act);
 	m = g_list_append(m, NULL);
 #endif
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Change Password");
-	pam->callback = change_pass;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+	act = gaim_plugin_action_new(_("Change Password"), change_pass);
+	m = g_list_append(m, act);
 
-	m = g_list_append(m, NULL);
+#if 0
+	act = gaim_plugin_action_new(_("Import Buddy List from Server"),
+			import_buddies_server);
+	m = g_list_append(m, act);
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Import Buddy List from Server");
-	pam->callback = import_buddies_server;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+	act = gaim_plugin_action_new(_("Export Buddy List to Server"),
+			export_buddies_server);
+	m = g_list_append(m, act);
 
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Export Buddy List to Server");
-	pam->callback = export_buddies_server;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
-
-	pam = g_new0(struct proto_actions_menu, 1);
-	pam->label = _("Delete Buddy List from Server");
-	pam->callback = delete_buddies_server;
-	pam->gc = gc;
-	m = g_list_append(m, pam);
+	act = gaim_plugin_action_new(_("Delete Buddy List from Server"),
+			delete_buddies_server);
+	m = g_list_append(m, act);
+#endif
 
 	return m;
 }
@@ -1244,7 +1444,7 @@ static void agg_get_info(GaimConnection *gc, const char *who)
 	if (invalid_uin(who)) {
 		gchar *new_who = charset_convert(who, "UTF-8", "CP1250");
 		gchar *enew_who = gg_urlencode(new_who);
-		
+
 		g_free(new_who);
 
 		srch->request = g_strdup_printf("Mode=0&FirstName=%s&LastName=%s&Gender=%d"
@@ -1301,6 +1501,30 @@ static void agg_permit_deny_dummy(GaimConnection *gc, const char *who)
 	/* It's implemented on client side because GG server doesn't support this */
 }
 
+static void agg_group_buddy (GaimConnection *gc, const char *who,
+			const char *old_group, const char *new_group)
+{
+    GaimBuddy *buddy = gaim_find_buddy(gaim_connection_get_account(gc), who);
+    gchar *newdata;
+    /* GG Number */
+    gchar *name = buddy->name;
+    /* GG Pseudo */
+    gchar *show = buddy->alias ? buddy->alias : buddy->name;
+    /* Group Name */
+    const gchar *gname = new_group;
+
+    newdata = g_strdup_printf("%s;%s;%s;%s;%s;%s;%s;%s%s\r\n",
+		    show, show, show, show, "", gname, name, "", "");
+    agg_save_buddy_list(gc, newdata);
+    g_free(newdata);
+}
+
+static void agg_rename_group (GaimConnection *gc, const char *old_group,
+		     const char *new_group, GList *members)
+{
+    agg_save_buddy_list(gc, NULL);
+}
+
 static GaimPlugin *my_protocol = NULL;
 
 static GaimPluginProtocolInfo prpl_info =
@@ -1314,8 +1538,7 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,
 	NULL,
 	agg_away_states,
-	agg_actions,
-	agg_buddy_menu,
+	agg_blist_node_menu,
 	NULL,
 	agg_login,
 	agg_close,
@@ -1325,9 +1548,6 @@ static GaimPluginProtocolInfo prpl_info =
 	agg_get_info,
 	agg_set_away,
 	NULL,
-	NULL,
-	agg_dir_search,
-	NULL,
 	agg_change_passwd,
 	agg_add_buddy,
 	agg_add_buddies,
@@ -1350,12 +1570,19 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,
 	NULL,
 	NULL,
+	agg_group_buddy,
+	agg_rename_group,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
 	NULL,
-	NULL
 };
 
 static GaimPluginInfo info =
@@ -1367,7 +1594,7 @@ static GaimPluginInfo info =
 	NULL,                                             /**< dependencies   */
 	GAIM_PRIORITY_DEFAULT,                            /**< priority       */
 
-	"prpl-gg",		                                  /**< id             */
+	"prpl-gg",		                          /**< id             */
 	"Gadu-Gadu",                                      /**< name           */
 	VERSION,                                          /**< version        */
 	                                                  /**  summary        */
@@ -1382,7 +1609,9 @@ static GaimPluginInfo info =
 	NULL,                                             /**< destroy        */
 
 	NULL,                                             /**< ui_info        */
-	&prpl_info                                        /**< extra_info     */
+	&prpl_info,                                       /**< extra_info     */
+	NULL,
+	agg_actions
 };
 
 static void
@@ -1391,9 +1620,9 @@ init_plugin(GaimPlugin *plugin)
 	GaimAccountOption *option;
 
 	option = gaim_account_option_string_new(_("Nick"), "nick",
-											"Gadu-Gadu User");
+			"Gadu-Gadu User");
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options,
-											   option);
+			option);
 
 	my_protocol = plugin;
 }
