@@ -22,9 +22,6 @@
 
 #include "internal.h"
 
-/* XXX g_show_info_text */
-#include "gaim.h"
-
 #include "conversation.h"
 #include "blist.h"
 #include "notify.h"
@@ -59,11 +56,11 @@ static char *irc_mask_userhost(const char *mask)
 
 static void irc_chat_remove_buddy(GaimConversation *convo, char *data[2])
 {
-	GList *users = gaim_chat_get_users(GAIM_CHAT(convo));
+	GList *users = gaim_conv_chat_get_users(GAIM_CONV_CHAT(convo));
 	char *message = g_strdup_printf("quit: %s", data[1]);
 
 	if (g_list_find_custom(users, data[0], (GCompareFunc)(strcmp)))
-		gaim_chat_remove_user(GAIM_CHAT(convo), data[0], message);
+		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), data[0], message);
 
 	g_free(message);
 }
@@ -88,7 +85,7 @@ void irc_msg_away(struct irc_conn *irc, const char *name, const char *from, char
 
 	gc = gaim_account_get_connection(irc->account);
 	if (gc)
-		serv_got_im(gc, args[1], args[2], GAIM_IM_AUTO_RESP, time(NULL), -1);
+		serv_got_im(gc, args[1], args[2], GAIM_CONV_IM_AUTO_RESP, time(NULL));
 }
 
 void irc_msg_badmode(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -127,7 +124,7 @@ void irc_msg_chanmode(struct irc_conn *irc, const char *name, const char *from,
 		return;
 
 	buf = g_strdup_printf("mode for %s: %s %s", args[1], args[2], args[3] ? args[3] : "");
-	gaim_chat_write(GAIM_CHAT(convo), "", buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+	gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 	g_free(buf);
 
 	return;
@@ -170,6 +167,7 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 {
 	GaimConnection *gc;
 	GString *info;
+	char buffer[256];
 	char *str;
 
 	if (!irc->whois.nick) {
@@ -205,7 +203,7 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 		g_free(irc->whois.channels);
 	}
 	if (irc->whois.idle) {
-		gchar *timex = sec_to_text(irc->whois.idle);
+		gchar *timex = gaim_str_seconds_to_string(irc->whois.idle);
 		g_string_append_printf(info, _("<b>Idle for:</b> %s<br>"), timex);
 		g_free(timex);
 		g_string_append_printf(info, "<b>%s:</b> %s", _("Online since"), ctime(&irc->whois.signon));
@@ -216,41 +214,78 @@ void irc_msg_endwhois(struct irc_conn *irc, const char *name, const char *from,
 
 	gc = gaim_account_get_connection(irc->account);
 	str = g_string_free(info, FALSE);
-	g_show_info_text(gc, irc->whois.nick, 2, str, NULL);
+
+	g_snprintf(buffer, sizeof(buffer),
+			   _("Buddy Information for %s"), irc->whois.nick);
+	gaim_notify_formatted(gc, NULL, buffer, NULL, str, NULL, NULL);
+
 	g_free(str);
 	memset(&irc->whois, 0, sizeof(irc->whois));
 }
 
+void irc_msg_list(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	if (!irc->roomlist)
+		return;
+
+	if (!strcmp(name, "321")) {
+		gaim_roomlist_set_in_progress(irc->roomlist, TRUE);
+		return;
+	}
+
+	if (!strcmp(name, "323")) {
+		gaim_roomlist_set_in_progress(irc->roomlist, FALSE);
+		gaim_roomlist_unref(irc->roomlist);
+		irc->roomlist = NULL;
+	}
+
+	if (!strcmp(name, "322")) {
+		GaimRoomlistRoom *room;
+
+		if (!args[0] || !args[1] || !args[2] || !args[3])
+			return;
+
+		room = gaim_roomlist_room_new(GAIM_ROOMLIST_ROOMTYPE_ROOM, args[1], NULL);
+		gaim_roomlist_room_add_field(irc->roomlist, room, args[1]);
+		gaim_roomlist_room_add_field(irc->roomlist, room, GINT_TO_POINTER(strtol(args[2], NULL, 10)));
+		gaim_roomlist_room_add_field(irc->roomlist, room, args[3]);
+		gaim_roomlist_room_add(irc->roomlist, room);
+	}
+}
+
 void irc_msg_topic(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
-	char *chan, *topic, *msg, *nick;
+	char *chan, *topic, *msg, *nick, *tmp;
 	GaimConversation *convo;
 
 	if (!strcmp(name, "topic")) {
 		chan = args[0];
-		topic = args[1];
+		topic = irc_mirc2txt (args[1]);
 	} else {
 		chan = args[1];
-		topic = args[2];
+		topic = irc_mirc2txt (args[2]);
 	}
 
 	convo = gaim_find_conversation_with_account(chan, irc->account);
 	if (!convo) {
 		gaim_debug(GAIM_DEBUG_ERROR, "irc", "Got a topic for %s, which doesn't exist\n", chan);
 	}
-	gaim_chat_set_topic(GAIM_CHAT(convo), NULL, topic);
+	gaim_conv_chat_set_topic(GAIM_CONV_CHAT(convo), NULL, topic);
 	/* If this is an interactive update, print it out */
+	tmp = gaim_escape_html(topic);
 	if (!strcmp(name, "topic")) {
 		nick = irc_mask_nick(from);
-		msg = g_strdup_printf(_("%s has changed the topic to: %s"), nick, topic);
+		msg = g_strdup_printf(_("%s has changed the topic to: %s"), nick, tmp);
 		g_free(nick);
-		gaim_chat_write(GAIM_CHAT(convo), from, msg, GAIM_MESSAGE_SYSTEM, time(NULL));
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), from, msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
 	} else {
-		msg = g_strdup_printf(_("The topic for %s is: %s"), chan, topic);
-		gaim_chat_write(GAIM_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM, time(NULL));
+		msg = g_strdup_printf(_("The topic for %s is: %s"), chan, tmp);
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
 	}
+	g_free(tmp);
+	g_free(topic);
 }
 
 void irc_msg_unknown(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -287,9 +322,9 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 		if (irc->nameconv) {
 			msg = g_strdup_printf("Users on %s: %s", args[1], names);
 			if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT)
-				gaim_chat_write(GAIM_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+				gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 			else
-				gaim_im_write(GAIM_IM(convo), "", msg, -1, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+				gaim_conv_im_write(GAIM_CONV_IM(convo), "", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 			g_free(msg);
 			g_free(irc->nameconv);
 			irc->nameconv = NULL;
@@ -312,7 +347,7 @@ void irc_msg_names(struct irc_conn *irc, const char *name, const char *from, cha
 			if (users != NULL) {
 				GList *l;
 
-				gaim_chat_add_users(GAIM_CHAT(convo), users);
+				gaim_conv_chat_add_users(GAIM_CONV_CHAT(convo), users);
 
 				for (l = users; l != NULL; l = l->next)
 					g_free(l->data);
@@ -353,9 +388,21 @@ void irc_msg_endmotd(struct irc_conn *irc, const char *name, const char *from, c
 		return;
 
 	gaim_connection_set_state(gc, GAIM_CONNECTED);
+	serv_finish_login (gc);
 
 	irc_blist_timeout(irc);
-	irc->timer = g_timeout_add(45000, (GSourceFunc)irc_blist_timeout, (gpointer)irc);
+	if (!irc->timer)
+		irc->timer = gaim_timeout_add(45000, (GSourceFunc)irc_blist_timeout, (gpointer)irc);
+}
+
+void irc_msg_nochan(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConnection *gc = gaim_account_get_connection(irc->account);
+
+	if (gc == NULL || args == NULL || args[1] == NULL)
+		return;
+
+	gaim_notify_error(gc, NULL, _("No such channel"), args[1]);
 }
 
 void irc_msg_nonick(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -366,10 +413,10 @@ void irc_msg_nonick(struct irc_conn *irc, const char *name, const char *from, ch
 	convo = gaim_find_conversation_with_account(args[1], irc->account);
 	if (convo) {
 		if (gaim_conversation_get_type(convo) == GAIM_CONV_CHAT) /* does this happen? */
-			gaim_chat_write(GAIM_CHAT(convo), args[1], _("no such channel"),
+			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[1], _("no such channel"),
 					GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 		else
-			gaim_im_write(GAIM_IM(convo), args[1], _("User is not logged in"), -1,
+			gaim_conv_im_write(GAIM_CONV_IM(convo), args[1], _("User is not logged in"),
 				      GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 	} else {
 		if ((gc = gaim_account_get_connection(irc->account)) == NULL)
@@ -390,7 +437,7 @@ void irc_msg_nosend(struct irc_conn *irc, const char *name, const char *from, ch
 
 	convo = gaim_find_conversation_with_account(args[1], irc->account);
 	if (convo) {
-		gaim_chat_write(GAIM_CHAT(convo), args[1], args[2], GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[1], args[2], GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 	} else {
 		if ((gc = gaim_account_get_connection(irc->account)) == NULL)
 			return;
@@ -406,7 +453,7 @@ void irc_msg_notinchan(struct irc_conn *irc, const char *name, const char *from,
 	if (convo) {
 		/*g_slist_remove(irc->gc->buddy_chats, convo);
 		  gaim_conversation_set_account(convo, NULL);*/
-		gaim_chat_write(GAIM_CHAT(convo), args[1], args[2], GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[1], args[2], GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 	}
 }
 
@@ -421,7 +468,7 @@ void irc_msg_notop(struct irc_conn *irc, const char *name, const char *from, cha
 	if (!convo)
 		return;
 
-	gaim_chat_write(GAIM_CHAT(convo), "", args[2], GAIM_MESSAGE_SYSTEM, time(NULL));
+	gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "", args[2], GAIM_MESSAGE_SYSTEM, time(NULL));
 }
 
 void irc_msg_invite(struct irc_conn *irc, const char *name, const char *from, char **args)
@@ -481,7 +528,7 @@ void irc_msg_ison(struct irc_conn *irc, const char *name, const char *from, char
 static void irc_buddy_status(char *name, struct irc_buddy *ib, struct irc_conn *irc)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
-	struct buddy *buddy = gaim_find_buddy(irc->account, name);
+	GaimBuddy *buddy = gaim_find_buddy(irc->account, name);
 
 	if (!gc || !buddy)
 		return;
@@ -524,7 +571,7 @@ void irc_msg_join(struct irc_conn *irc, const char *name, const char *from, char
 	}
 
 	userhost = irc_mask_userhost(from);
-	gaim_chat_add_user(GAIM_CHAT(convo), nick, userhost);
+	gaim_conv_chat_add_user(GAIM_CONV_CHAT(convo), nick, userhost);
 	g_free(userhost);
 	g_free(nick);
 }
@@ -548,15 +595,12 @@ void irc_msg_kick(struct irc_conn *irc, const char *name, const char *from, char
 
 	if (!gaim_utf8_strcasecmp(gaim_connection_get_display_name(gc), args[1])) {
 		buf = g_strdup_printf(_("You have been kicked by %s: (%s)"), nick, args[2]);
-		gaim_chat_write(GAIM_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM, time(NULL));
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(buf);
-		/*g_slist_remove(irc->gc->buddy_chats, convo);
-		  gaim_conversation_set_account(convo, NULL);*/
-		/*g_list_free(gaim_chat_get_users(GAIM_CHAT(convo)));
-		  gaim_chat_set_users(GAIM_CHAT(convo), NULL);*/
+		serv_got_chat_left(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)));
 	} else {
 		buf = g_strdup_printf(_("Kicked by %s (%s)"), nick, args[2]);
-		gaim_chat_remove_user(GAIM_CHAT(convo), args[1], buf);
+		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), args[1], buf);
 		g_free(buf);
 	}
 
@@ -577,7 +621,7 @@ void irc_msg_mode(struct irc_conn *irc, const char *name, const char *from, char
 			return;
 		}
 		buf = g_strdup_printf(_("mode (%s %s) by %s"), args[1], args[2] ? args[2] : "", nick);
-		gaim_chat_write(GAIM_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], buf, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 		g_free(buf);
 	} else {					/* User		*/
 	}
@@ -601,15 +645,15 @@ void irc_msg_nick(struct irc_conn *irc, const char *name, const char *from, char
 	}
 
 	while (chats) {
-		GaimChat *chat = GAIM_CHAT(chats->data);
-		GList *users = gaim_chat_get_users(chat);
+		GaimConvChat *chat = GAIM_CONV_CHAT(chats->data);
+		GList *users = gaim_conv_chat_get_users(chat);
 
 		while (users) {
 			char *user = users->data;
 
 			if (!strcmp(nick, user)) {
-				gaim_chat_rename_user(chat, user, args[0]);
-				users = gaim_chat_get_users(chat);
+				gaim_conv_chat_rename_user(chat, user, args[0]);
+				users = gaim_conv_chat_get_users(chat);
 				break;
 			}
 			users = users->next;
@@ -647,13 +691,26 @@ void irc_msg_notice(struct irc_conn *irc, const char *name, const char *from, ch
 	irc_msg_privmsg(irc, name, from, newargs);
 }
 
+void irc_msg_nochangenick(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	char *msg;
+
+	if (!args || !args[2] || !gc)
+		return;
+
+	msg = g_strdup_printf(_("Could not change nick"));
+	gaim_notify_error(gc, _("Cannot change nick"), msg, args[2]);
+	g_free(msg);
+}
+
 void irc_msg_part(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
 	GaimConversation *convo;
 	char *nick, *msg;
 
-	if (!args || !args[0] || !args[1] || !gc)
+	if (!args || !args[0] || !gc)
 		return;
 
 	convo = gaim_find_conversation_with_account(args[0], irc->account);
@@ -664,11 +721,13 @@ void irc_msg_part(struct irc_conn *irc, const char *name, const char *from, char
 
 	nick = irc_mask_nick(from);
 	if (!gaim_utf8_strcasecmp(nick, gaim_connection_get_display_name(gc))) {
-		msg = g_strdup_printf(_("You have parted the channel%s%s"), *args[1] ? ": " : "", args[1]);
-		gaim_chat_write(GAIM_CHAT(convo), args[0], msg, GAIM_MESSAGE_SYSTEM, time(NULL));
+		msg = g_strdup_printf(_("You have parted the channel%s%s"),
+                                      (args[1] && *args[1]) ? ": " : "", args[1]);
+		gaim_conv_chat_write(GAIM_CONV_CHAT(convo), args[0], msg, GAIM_MESSAGE_SYSTEM, time(NULL));
 		g_free(msg);
+		serv_got_chat_left(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)));
 	} else {
-		gaim_chat_remove_user(GAIM_CHAT(convo), nick, args[1]);
+		gaim_conv_chat_remove_user(GAIM_CONV_CHAT(convo), nick, args[1]);
 	}
 	g_free(nick);
 }
@@ -711,9 +770,9 @@ void irc_msg_pong(struct irc_conn *irc, const char *name, const char *from, char
 	g_strfreev(parts);
 	if (convo) {
 		if (gaim_conversation_get_type (convo) == GAIM_CONV_CHAT)
-			gaim_chat_write(GAIM_CHAT(convo), "PONG", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+			gaim_conv_chat_write(GAIM_CONV_CHAT(convo), "PONG", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 		else
-			gaim_im_write(GAIM_IM(convo), "PONG", msg, -1, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
+			gaim_conv_im_write(GAIM_CONV_IM(convo), "PONG", msg, GAIM_MESSAGE_SYSTEM|GAIM_MESSAGE_NO_LOG, time(NULL));
 	} else {
 		gc = gaim_account_get_connection(irc->account);
 		if (!gc) {
@@ -743,8 +802,13 @@ void irc_msg_privmsg(struct irc_conn *irc, const char *name, const char *from, c
 		g_free(nick);
 		return;
 	}
-	msg = irc_mirc2html(tmp);
+
+	msg = gaim_escape_html(tmp);
 	g_free(tmp);
+
+	tmp = irc_mirc2html(msg);
+	g_free(msg);
+	msg = tmp;
 	if (notice) {
 		tmp = g_strdup_printf("(notice) %s", msg);
 		g_free(msg);
@@ -752,13 +816,13 @@ void irc_msg_privmsg(struct irc_conn *irc, const char *name, const char *from, c
 	}
 
 	if (!gaim_utf8_strcasecmp(args[0], gaim_connection_get_display_name(gc))) {
-		serv_got_im(gc, nick, msg, 0, time(NULL), -1);
+		serv_got_im(gc, nick, msg, 0, time(NULL));
 	} else if (notice) {
-		serv_got_im(gc, nick, msg, 0, time(NULL), -1);
+		serv_got_im(gc, nick, msg, 0, time(NULL));
 	} else {
 		convo = gaim_find_conversation_with_account(args[0], irc->account);
 		if (convo)
-			serv_got_chat_in(gc, gaim_chat_get_id(GAIM_CHAT(convo)), nick, 0, msg, time(NULL));
+			serv_got_chat_in(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(convo)), nick, 0, msg, time(NULL));
 		else
 			gaim_debug(GAIM_DEBUG_ERROR, "irc", "Got a PRIVMSG on %s, which does not exist\n", args[0]);
 	}
@@ -766,6 +830,19 @@ void irc_msg_privmsg(struct irc_conn *irc, const char *name, const char *from, c
 	g_free(nick);
 }
 
+void irc_msg_regonly(struct irc_conn *irc, const char *name, const char *from, char **args)
+{
+	GaimConnection *gc = gaim_account_get_connection(irc->account);
+	char *msg;
+
+	if (!args || !args[1] || !args[2] || !gc)
+		return;
+
+	msg = g_strdup_printf(_("Cannot join %s:"), args[1]);
+	gaim_notify_error(gc, _("Cannot join channel"), msg, args[2]);
+	g_free(msg);
+}
+
 void irc_msg_quit(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
@@ -786,7 +863,7 @@ void irc_msg_quit(struct irc_conn *irc, const char *name, const char *from, char
 void irc_msg_wallops(struct irc_conn *irc, const char *name, const char *from, char **args)
 {
 	GaimConnection *gc = gaim_account_get_connection(irc->account);
-	char *nick, *msg;
+	char *nick, *msg, *wallop;
 
 	if (!args || !args[0] || !gc)
 		return;
@@ -794,8 +871,10 @@ void irc_msg_wallops(struct irc_conn *irc, const char *name, const char *from, c
 	nick = irc_mask_nick(from);
 	msg = g_strdup_printf (_("Wallops from %s"), nick);
 	g_free(nick);
-	gaim_notify_info(gc, NULL, msg, args[0]);
+	wallop = g_markup_escape_text(args[0], strlen(args[0]));
+	gaim_notify_info(gc, NULL, msg, wallop);
 	g_free(msg);
+	g_free(wallop);
 }
 
 void irc_msg_ignore(struct irc_conn *irc, const char *name, const char *from, char **args)
