@@ -3,8 +3,8 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Copyright (C) 2003-2004 Christian Hammond <chipx86@gnupdate.org>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,6 +22,18 @@
 #ifndef _MSN_ERROR_H_
 #define _MSN_ERROR_H_
 
+typedef enum
+{
+	MSN_ERROR_NONE,
+	MSN_ERROR_MISC,
+	MSN_ERROR_CONNECT,
+	MSN_ERROR_WRITE,
+	MSN_ERROR_READ,
+	MSN_ERROR_SIGNOTHER,
+	MSN_ERROR_SERVDOWN
+
+} MsnErrorType;
+
 #include "session.h"
 
 /**
