@@ -1,9 +1,9 @@
 /*
  * Family 0x0013 - Server-Side/Stored Information.
  *
- * Relatively new facility that allows storing of certain types of information,
- * such as a users buddy list, permit/deny list, and permit/deny preferences, 
- * to be stored on the server, so that they can be accessed from any client.
+ * Relatively new facility that allows certain types of information, such as 
+ * a user's buddy list, permit/deny list, and permit/deny preferences, to be 
+ * stored on the server, so that they can be accessed from any client.
  *
  * We keep 2 copies of SSI data:
  * 1) An exact copy of what is stored on the AIM servers.
@@ -11,7 +11,7 @@
  *    between this and the exact copy to keep them in sync.
  *
  * All the "aim_ssi_itemlist_bleh" functions near the top just modify the list 
- * that is given to them (eg. they don't send SNACs).
+ * that is given to them (i.e. they don't send SNACs).
  *
  * The SNAC sending and receiving functions are lower down in the file, and 
  * they're simpler.  They are in the order of the subtypes they deal with, 
@@ -21,8 +21,6 @@
  * This is entirely too complicated.
  * You don't know the half of it.
  *
- * XXX - Preserve unknown data in TLV lists
- *
  */
 
 #define FAIM_INTERNAL
@@ -48,10 +46,6 @@ static struct aim_ssi_item *aim_ssi_itemlist_rebuildgroup(struct aim_ssi_item *l
 	if (!(group = aim_ssi_itemlist_finditem(list, name, NULL, AIM_SSI_TYPE_GROUP)))
 		return NULL;
 
-	/* Free the old data */
-	aim_freetlvchain(&group->data);
-	group->data = NULL;
-
 	/* Find the length for the new additional data */
 	newlen = 0;
 	if (group->gid == 0x0000) {
@@ -80,7 +74,7 @@ static struct aim_ssi_item *aim_ssi_itemlist_rebuildgroup(struct aim_ssi_item *l
 				if ((cur->gid == group->gid) && (cur->type == AIM_SSI_TYPE_BUDDY))
 						newlen += aimutil_put16(newdata+newlen, cur->bid);
 		}
-		aim_addtlvtochain_raw(&group->data, 0x00c8, newlen, newdata);
+		aim_tlvlist_replace_raw(&group->data, 0x00c8, newlen, newdata);
 
 		free(newdata);
 	}
@@ -185,12 +179,12 @@ static int aim_ssi_itemlist_del(struct aim_ssi_item **list, struct aim_ssi_item
 		struct aim_ssi_item *cur;
 		for (cur=*list; (cur->next && (cur->next!=del)); cur=cur->next);
 		if (cur->next)
-			cur->next = cur->next->next;
+			cur->next = del->next;
 	}
 
-	/* Free the deleted item */
+	/* Free the removed item */
 	free(del->name);
-	aim_freetlvchain(&del->data);
+	aim_tlvlist_free(&del->data);
 	free(del);
 
 	return 0;
@@ -359,12 +353,9 @@ faim_export int aim_ssi_getpermdeny(struct aim_ssi_item *list)
 {
 	struct aim_ssi_item *cur = aim_ssi_itemlist_finditem(list, NULL, NULL, AIM_SSI_TYPE_PDINFO);
 	if (cur) {
-		aim_tlvlist_t *tlvlist = cur->data;
-		if (tlvlist) {
-			aim_tlv_t *tlv = aim_gettlv(tlvlist, 0x00ca, 1);
-			if (tlv && tlv->value)
-				return aimutil_get8(tlv->value);
-		}
+		aim_tlv_t *tlv = aim_tlv_gettlv(cur->data, 0x00ca, 1);
+		if (tlv && tlv->value)
+			return aimutil_get8(tlv->value);
 	}
 	return 0;
 }
@@ -381,12 +372,9 @@ faim_export fu32_t aim_ssi_getpresence(struct aim_ssi_item *list)
 {
 	struct aim_ssi_item *cur = aim_ssi_itemlist_finditem(list, NULL, NULL, AIM_SSI_TYPE_PRESENCEPREFS);
 	if (cur) {
-		aim_tlvlist_t *tlvlist = cur->data;
-		if (tlvlist) {
-			aim_tlv_t *tlv = aim_gettlv(tlvlist, 0x00c9, 1);
-			if (tlv && tlv->length)
-				return aimutil_get32(tlv->value);
-		}
+		aim_tlv_t *tlv = aim_tlv_gettlv(cur->data, 0x00c9, 1);
+		if (tlv && tlv->length)
+			return aimutil_get32(tlv->value);
 	}
 	return 0xFFFFFFFF;
 }
@@ -397,7 +385,7 @@ faim_export fu32_t aim_ssi_getpresence(struct aim_ssi_item *list)
  * @param list A pointer to the current list of items.
  * @param gn The group of the buddy.
  * @param sn The name of the buddy.
- * @return A pointer to a NULL terminated string that is the buddies 
+ * @return A pointer to a NULL terminated string that is the buddy's 
  *         alias, or NULL if the buddy has no alias.  You should free
  *         this returned value!
  */
@@ -405,15 +393,37 @@ faim_export char *aim_ssi_getalias(struct aim_ssi_item *list, const char *gn, co
 {
 	struct aim_ssi_item *cur = aim_ssi_itemlist_finditem(list, gn, sn, AIM_SSI_TYPE_BUDDY);
 	if (cur) {
-		aim_tlvlist_t *tlvlist = cur->data;
-		if (tlvlist) {
-			aim_tlv_t *tlv = aim_gettlv(tlvlist, 0x0131, 1);
-			if (tlv && tlv->length) {
-				char *alias = (char *)malloc((tlv->length+1)*sizeof(char));
-				strncpy(alias, tlv->value, tlv->length);
-				alias[tlv->length] = 0;
-				return alias;
-			}
+		aim_tlv_t *tlv = aim_tlv_gettlv(cur->data, 0x0131, 1);
+		if (tlv && tlv->length) {
+			char *alias = (char *)malloc((tlv->length+1)*sizeof(char));
+			strncpy(alias, tlv->value, tlv->length);
+			alias[tlv->length] = 0;
+			return alias;
+		}
+	}
+	return NULL;
+}
+
+/**
+ * Locally find the comment of the given buddy.
+ *
+ * @param list A pointer to the current list of items.
+ * @param gn The group of the buddy.
+ * @param sn The name of the buddy.
+ * @return A pointer to a NULL terminated string that is the buddy's 
+ *         comment, or NULL if the buddy has no comment.  You should free
+ *         this returned value!
+ */
+faim_export char *aim_ssi_getcomment(struct aim_ssi_item *list, const char *gn, const char *sn)
+{
+	struct aim_ssi_item *cur = aim_ssi_itemlist_finditem(list, gn, sn, AIM_SSI_TYPE_BUDDY);
+	if (cur) {
+		aim_tlv_t *tlv = aim_tlv_gettlv(cur->data, 0x013c, 1);
+		if (tlv && tlv->length) {
+			char *alias = (char *)malloc((tlv->length+1)*sizeof(char));
+			strncpy(alias, tlv->value, tlv->length);
+			alias[tlv->length] = 0;
+			return alias;
 		}
 	}
 	return NULL;
@@ -433,10 +443,8 @@ faim_export int aim_ssi_waitingforauth(struct aim_ssi_item *list, const char *gn
 {
 	struct aim_ssi_item *cur = aim_ssi_itemlist_finditem(list, gn, sn, AIM_SSI_TYPE_BUDDY);
 	if (cur) {
-		aim_tlvlist_t *tlvlist = cur->data;
-		if (tlvlist)
-			if (aim_gettlv(tlvlist, 0x0066, 1))
-				return 1;
+		if (aim_tlv_gettlv(cur->data, 0x0066, 1))
+			return 1;
 	}
 	return 0;
 }
@@ -561,7 +569,7 @@ static int aim_ssi_freelist(aim_session_t *sess)
 		del = cur;
 		cur = cur->next;
 		free(del->name);
-		aim_freetlvchain(&del->data);
+		aim_tlvlist_free(&del->data);
 		free(del);
 	}
 
@@ -570,7 +578,7 @@ static int aim_ssi_freelist(aim_session_t *sess)
 		del = cur;
 		cur = cur->next;
 		free(del->name);
-		aim_freetlvchain(&del->data);
+		aim_tlvlist_free(&del->data);
 		free(del);
 	}
 
@@ -609,7 +617,7 @@ faim_export int aim_ssi_deletelist(aim_session_t *sess)
 		del = cur;
 		cur = cur->next;
 		free(del->name);
-		aim_freetlvchain(&del->data);
+		aim_tlvlist_free(&del->data);
 		free(del);
 	}
 	sess->ssi.local = NULL;
@@ -663,7 +671,7 @@ faim_export int aim_ssi_cleanlist(aim_session_t *sess)
 	while (cur) {
 		next = cur->next;
 		if (cur->type == AIM_SSI_TYPE_GROUP) {
-			aim_tlv_t *tlv = aim_gettlv(cur->data, 0x00c8, 1);
+			aim_tlv_t *tlv = aim_tlv_gettlv(cur->data, 0x00c8, 1);
 			if (!tlv || !tlv->length)
 				aim_ssi_itemlist_del(&sess->ssi.local, cur);
 		}
@@ -712,17 +720,17 @@ faim_export int aim_ssi_addbuddy(aim_session_t *sess, const char *name, const ch
 
 	/* Create a TLV list for the new buddy */
 	if (needauth)
-		aim_addtlvtochain_noval(&data, 0x0066);
+		aim_tlvlist_add_noval(&data, 0x0066);
 	if (alias)
-		aim_addtlvtochain_raw(&data, 0x0131, strlen(alias), alias);
+		aim_tlvlist_add_raw(&data, 0x0131, strlen(alias), alias);
 	if (smsnum)
-		aim_addtlvtochain_raw(&data, 0x013a, strlen(smsnum), smsnum);
+		aim_tlvlist_add_raw(&data, 0x013a, strlen(smsnum), smsnum);
 	if (comment)
-		aim_addtlvtochain_raw(&data, 0x013c, strlen(comment), comment);
+		aim_tlvlist_add_raw(&data, 0x013c, strlen(comment), comment);
 
 	/* Add that bad boy */
 	aim_ssi_itemlist_add(&sess->ssi.local, name, parent->gid, 0xFFFF, AIM_SSI_TYPE_BUDDY, data);
-	aim_freetlvchain(&data);
+	aim_tlvlist_free(&data);
 
 	/* Modify the parent group */
 	aim_ssi_itemlist_rebuildgroup(sess->ssi.local, group);
@@ -898,13 +906,13 @@ faim_export int aim_ssi_movebuddy(aim_session_t *sess, const char *oldgn, const
  * @param sess The oscar session.
  * @param gn The group that the buddy is currently in.
  * @param sn The screen name of the buddy.
- * @param alias The new alias for the buddy.
+ * @param alias The new alias for the buddy, or NULL if you want to remove 
+ *        a buddy's comment.
  * @return Return 0 if no errors, otherwise return the error number.
  */
 faim_export int aim_ssi_aliasbuddy(aim_session_t *sess, const char *gn, const char *sn, const char *alias)
 {
 	struct aim_ssi_item *tmp;
-	aim_tlvlist_t *data = NULL;
 
 	if (!sess || !gn || !sn)
 		return -EINVAL;
@@ -912,15 +920,43 @@ faim_export int aim_ssi_aliasbuddy(aim_session_t *sess, const char *gn, const ch
 	if (!(tmp = aim_ssi_itemlist_finditem(sess->ssi.local, gn, sn, AIM_SSI_TYPE_BUDDY)))
 		return -EINVAL;
 
-	if (alias && !strlen(alias))
-		alias = NULL;
+	/* Either add or remove the 0x0131 TLV from the TLV chain */
+	if ((alias != NULL) && (strlen(alias) > 0))
+		aim_tlvlist_replace_raw(&tmp->data, 0x0131, strlen(alias), alias);
+	else
+		aim_tlvlist_remove(&tmp->data, 0x0131);
 
-	/* Need to add the x0131 TLV to the TLV chain */
-	if (alias)
-		aim_addtlvtochain_raw(&data, 0x0131, strlen(alias), alias);
+	/* Sync our local list with the server list */
+	aim_ssi_sync(sess);
 
-	aim_freetlvchain(&tmp->data);
-	tmp->data = data;
+	return 0;
+}
+
+/**
+ * Change the comment stored on the server for a given buddy.
+ *
+ * @param sess The oscar session.
+ * @param gn The group that the buddy is currently in.
+ * @param sn The screen name of the buddy.
+ * @param alias The new comment for the buddy, or NULL if you want to remove 
+ *        a buddy's comment.
+ * @return Return 0 if no errors, otherwise return the error number.
+ */
+faim_export int aim_ssi_editcomment(aim_session_t *sess, const char *gn, const char *sn, const char *comment)
+{
+	struct aim_ssi_item *tmp;
+
+	if (!sess || !gn || !sn)
+		return -EINVAL;
+
+	if (!(tmp = aim_ssi_itemlist_finditem(sess->ssi.local, gn, sn, AIM_SSI_TYPE_BUDDY)))
+		return -EINVAL;
+
+	/* Either add or remove the 0x0131 TLV from the TLV chain */
+	if ((comment != NULL) && (strlen(comment) > 0))
+		aim_tlvlist_replace_raw(&tmp->data, 0x013c, strlen(comment), comment);
+	else
+		aim_tlvlist_remove(&tmp->data, 0x013c);
 
 	/* Sync our local list with the server list */
 	aim_ssi_sync(sess);
@@ -973,24 +1009,19 @@ faim_export int aim_ssi_rename_group(aim_session_t *sess, const char *oldgn, con
 faim_export int aim_ssi_setpermdeny(aim_session_t *sess, fu8_t permdeny, fu32_t vismask)
 {
 	struct aim_ssi_item *tmp;
-	aim_tlvlist_t *data = NULL;
 
 	if (!sess)
 		return -EINVAL;
 
-	/* Need to add the x00ca TLV to the TLV chain */
-	aim_addtlvtochain8(&data, 0x00ca, permdeny);
+	/* Find the PDINFO item, or add it if it does not exist */
+	if (!(tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, NULL, AIM_SSI_TYPE_PDINFO)))
+		tmp = aim_ssi_itemlist_add(&sess->ssi.local, NULL, 0x0000, 0xFFFF, AIM_SSI_TYPE_PDINFO, NULL);
 
-	/* Need to add the x00cb TLV to the TLV chain */
-	aim_addtlvtochain32(&data, 0x00cb, vismask);
+	/* Need to add the 0x00ca TLV to the TLV chain */
+	aim_tlvlist_replace_8(&tmp->data, 0x00ca, permdeny);
 
-	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, NULL, AIM_SSI_TYPE_PDINFO))) {
-		aim_freetlvchain(&tmp->data);
-		tmp->data = data;
-	} else {
-		tmp = aim_ssi_itemlist_add(&sess->ssi.local, NULL, 0x0000, 0xFFFF, AIM_SSI_TYPE_PDINFO, data);
-		aim_freetlvchain(&data);
-	}
+	/* Need to add the 0x00cb TLV to the TLV chain */
+	aim_tlvlist_replace_32(&tmp->data, 0x00cb, vismask);
 
 	/* Sync our local list with the server list */
 	aim_ssi_sync(sess);
@@ -1009,70 +1040,78 @@ faim_export int aim_ssi_setpermdeny(aim_session_t *sess, fu8_t permdeny, fu32_t
 faim_export int aim_ssi_seticon(aim_session_t *sess, fu8_t *iconsum, fu16_t iconsumlen)
 {
 	struct aim_ssi_item *tmp;
-	aim_tlvlist_t *data = NULL;
 	fu8_t *csumdata;
 
 	if (!sess || !iconsum || !iconsumlen)
 		return -EINVAL;
 
+	/* Find the ICONINFO item, or add it if it does not exist */
+	if (!(tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, "1", AIM_SSI_TYPE_ICONINFO))) {
+		tmp = aim_ssi_itemlist_add(&sess->ssi.local, "1", 0x0000, 0x51F4, AIM_SSI_TYPE_ICONINFO, NULL);
+	}
+
+	/* Need to add the 0x00d5 TLV to the TLV chain */
 	if (!(csumdata = (fu8_t *)malloc((iconsumlen+2)*sizeof(fu8_t))))
 		return -ENOMEM;
 	csumdata[0] = 0x00;
 	csumdata[1] = 0x10;
 	memcpy(&csumdata[2], iconsum, iconsumlen);
+	aim_tlvlist_replace_raw(&tmp->data, 0x00d5, (iconsumlen+2) * sizeof(fu8_t), csumdata);
+	free(csumdata);
 
-	/* Need to add the x00d5 TLV to the TLV chain */
-	aim_addtlvtochain_raw(&data, 0x00d5, (iconsumlen+2) * sizeof(fu8_t), csumdata);
-
-	/* This TLV is added to cache the icon. */
-	aim_addtlvtochain_noval(&data, 0x0131);
-
-	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, "1", AIM_SSI_TYPE_ICONINFO))) {
-		/* If the new tlvchain and oldtlvchain are the same, then do nothing */
-		if (!aim_tlvlist_cmp(tmp->data, data)) {
-			/* The new tlvlist is the identical to the old one */
-			aim_freetlvchain(&data);
-			free(csumdata);
-			return 0;
-		}
-		aim_freetlvchain(&tmp->data);
-		tmp->data = data;
-	} else {
-		tmp = aim_ssi_itemlist_add(&sess->ssi.local, "1", 0x0000, 0x51F4, AIM_SSI_TYPE_ICONINFO, data);
-		aim_freetlvchain(&data);
-	}
+	/* Need to add the 0x0131 TLV to the TLV chain, used to cache the icon */
+	aim_tlvlist_replace_noval(&tmp->data, 0x0131);
 
 	/* Sync our local list with the server list */
 	aim_ssi_sync(sess);
-	free(csumdata);
 	return 0;
 }
 
 /**
- * Stores your setting for whether you should show up as idle or not.
+ * Remove a reference to a server stored buddy icon.  This will make your 
+ * icon stop showing up to other people.
+ *
+ * @param sess The oscar session.
+ * @return Return 0 if no errors, otherwise return the error number.
+ */
+faim_export int aim_ssi_delicon(aim_session_t *sess)
+{
+	struct aim_ssi_item *tmp;
+
+	if (!sess)
+		return -EINVAL;
+
+	/* Find the ICONINFO item and delete it if it exists*/
+	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, "1", AIM_SSI_TYPE_ICONINFO)))
+		aim_ssi_itemlist_del(&sess->ssi.local, tmp);
+
+	/* Sync our local list with the server list */
+	aim_ssi_sync(sess);
+	return 0;
+}
+
+/**
+ * Stores your setting for various SSI settings.  Whether you 
+ * should show up as idle or not, etc.
  *
  * @param sess The oscar session.
  * @param presence I think it's a bitmask, but I only know what one of the bits is:
+ *        0x00000002 - Hide wireless?
  *        0x00000400 - Allow others to see your idle time
  * @return Return 0 if no errors, otherwise return the error number.
  */
 faim_export int aim_ssi_setpresence(aim_session_t *sess, fu32_t presence) {
 	struct aim_ssi_item *tmp;
-	aim_tlvlist_t *data = NULL;
 
 	if (!sess)
 		return -EINVAL;
 
-	/* Need to add the x00c9 TLV to the TLV chain */
-	aim_addtlvtochain32(&data, 0x00c9, presence);
+	/* Find the PRESENCEPREFS item, or add it if it does not exist */
+	if (!(tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, NULL, AIM_SSI_TYPE_PRESENCEPREFS)))
+		tmp = aim_ssi_itemlist_add(&sess->ssi.local, NULL, 0x0000, 0xFFFF, AIM_SSI_TYPE_PRESENCEPREFS, NULL);
 
-	if ((tmp = aim_ssi_itemlist_finditem(sess->ssi.local, NULL, NULL, AIM_SSI_TYPE_PRESENCEPREFS))) {
-		aim_freetlvchain(&tmp->data);
-		tmp->data = data;
-	} else {
-		tmp = aim_ssi_itemlist_add(&sess->ssi.local, NULL, 0x0000, 0xFFFF, AIM_SSI_TYPE_PRESENCEPREFS, data);
-		aim_freetlvchain(&data);
-	}
+	/* Need to add the x00c9 TLV to the TLV chain */
+	aim_tlvlist_replace_32(&tmp->data, 0x00c9, presence);
 
 	/* Sync our local list with the server list */
 	aim_ssi_sync(sess);
@@ -1090,7 +1129,7 @@ faim_export int aim_ssi_reqrights(aim_session_t *sess)
 	if (!sess || !(conn = aim_conn_findbygroup(sess, AIM_CB_FAM_SSI)))
 		return -EINVAL;
 
-	return aim_genericreq_n(sess, conn, AIM_CB_FAM_SSI, AIM_CB_SSI_REQRIGHTS);
+	return aim_genericreq_n_snacid(sess, conn, AIM_CB_FAM_SSI, AIM_CB_SSI_REQRIGHTS);
 }
 
 /*
@@ -1106,18 +1145,18 @@ static int parserights(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx,
 	fu16_t *maxitems;
 
 	/* This SNAC is made up of a bunch of TLVs */
-	tlvlist = aim_readtlvchain(bs);
+	tlvlist = aim_tlvlist_read(bs);
 
 	/* TLV 0x0004 contains the maximum number of each item */
-	if (!(tlv = aim_gettlv(tlvlist, 0x0004, 1))) {
-		aim_freetlvchain(&tlvlist);
+	if (!(tlv = aim_tlv_gettlv(tlvlist, 0x0004, 1))) {
+		aim_tlvlist_free(&tlvlist);
 		return 0;
 	}
 
 	aim_bstream_init(&bstream, tlv->value, tlv->length);
 
 	if (!(maxitems = (fu16_t *)malloc((tlv->length/2)*sizeof(fu16_t)))) {
-		aim_freetlvchain(&tlvlist);
+		aim_tlvlist_free(&tlvlist);
 		return 0;
 	}
 
@@ -1127,7 +1166,7 @@ static int parserights(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx,
 	if ((userfunc = aim_callhandler(sess, rx->conn, snac->family, snac->subtype)))
 		ret = userfunc(sess, rx, tlv->length/2, maxitems);
 
-	aim_freetlvchain(&tlvlist);
+	aim_tlvlist_free(&tlvlist);
 	free(maxitems);
 
 	return ret;
@@ -1211,10 +1250,10 @@ static int parsedata(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, ai
 		gid = aimbs_get16(bs);
 		bid = aimbs_get16(bs);
 		type = aimbs_get16(bs);
-		data = aim_readtlvchain_len(bs, aimbs_get16(bs));
+		data = aim_tlvlist_readlen(bs, aimbs_get16(bs));
 		aim_ssi_itemlist_add(&sess->ssi.official, name, gid, bid, type, data);
 		free(name);
-		aim_freetlvchain(&data);
+		aim_tlvlist_free(&data);
 	}
 
 	/* Read in the timestamp */
@@ -1280,7 +1319,7 @@ faim_export int aim_ssi_addmoddel(aim_session_t *sess)
 		if (cur->item->name)
 			snaclen += strlen(cur->item->name);
 		if (cur->item->data)
-			snaclen += aim_sizetlvchain(&cur->item->data);
+			snaclen += aim_tlvlist_size(&cur->item->data);
 	}
 
 	if (!(fr = aim_tx_new(sess, conn, AIM_FRAMETYPE_FLAP, 0x02, snaclen)))
@@ -1296,9 +1335,9 @@ faim_export int aim_ssi_addmoddel(aim_session_t *sess)
 		aimbs_put16(&fr->data, cur->item->gid);
 		aimbs_put16(&fr->data, cur->item->bid);
 		aimbs_put16(&fr->data, cur->item->type);
-		aimbs_put16(&fr->data, cur->item->data ? aim_sizetlvchain(&cur->item->data) : 0);
+		aimbs_put16(&fr->data, cur->item->data ? aim_tlvlist_size(&cur->item->data) : 0);
 		if (cur->item->data)
-			aim_writetlvchain(&fr->data, &cur->item->data);
+			aim_tlvlist_write(&fr->data, &cur->item->data);
 	}
 
 	aim_tx_enqueue(sess, fr);
@@ -1309,7 +1348,8 @@ faim_export int aim_ssi_addmoddel(aim_session_t *sess)
 /*
  * Subtype 0x0008 - Incoming SSI add.
  *
- * XXX - It would probably be good for the client to actually do something when it gets this.
+ * Sent by the server, for example, when someone is added to 
+ * your "Recent Buddies" group.
  */
 static int parseadd(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim_modsnac_t *snac, aim_bstream_t *bs)
 {
@@ -1328,17 +1368,16 @@ static int parseadd(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 		bid = aimbs_get16(bs);
 		type = aimbs_get16(bs);
 		if ((len = aimbs_get16(bs)))
-			data = aim_readtlvchain_len(bs, len);
+			data = aim_tlvlist_readlen(bs, len);
 		else
 			data = NULL;
 
 		aim_ssi_itemlist_add(&sess->ssi.local, name, gid, bid, type, data);
 		aim_ssi_itemlist_add(&sess->ssi.official, name, gid, bid, type, data);
-		free(name);
-		aim_freetlvchain(&data);
+		aim_tlvlist_free(&data);
 
 		if ((userfunc = aim_callhandler(sess, rx->conn, snac->family, snac->subtype)))
-			ret = userfunc(sess, rx);
+			ret = userfunc(sess, rx, type, name);
 
 		free(name);
 	}
@@ -1369,7 +1408,7 @@ static int parsemod(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 		bid = aimbs_get16(bs);
 		type = aimbs_get16(bs);
 		if ((len = aimbs_get16(bs)))
-			data = aim_readtlvchain_len(bs, len);
+			data = aim_tlvlist_readlen(bs, len);
 		else
 			data = NULL;
 
@@ -1382,7 +1421,7 @@ static int parsemod(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 				strcpy(item->name, name);
 			} else
 				item->name = NULL;
-			aim_freetlvchain(&item->data);
+			aim_tlvlist_free(&item->data);
 			item->data = aim_tlvlist_copy(data);
 		}
 
@@ -1394,7 +1433,7 @@ static int parsemod(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 				strcpy(item->name, name);
 			} else
 				item->name = NULL;
-			aim_freetlvchain(&item->data);
+			aim_tlvlist_free(&item->data);
 			item->data = aim_tlvlist_copy(data);
 		}
 
@@ -1402,7 +1441,7 @@ static int parsemod(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 			ret = userfunc(sess, rx);
 
 		free(name);
-		aim_freetlvchain(&data);
+		aim_tlvlist_free(&data);
 	}
 
 	return ret;
@@ -1490,7 +1529,7 @@ static int parseack(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 							strcpy(cur->item->name, cur1->name);
 						} else
 							cur->item->name = NULL;
-						aim_freetlvchain(&cur->item->data);
+						aim_tlvlist_free(&cur->item->data);
 						cur->item->data = aim_tlvlist_copy(cur1->data);
 					}
 				} else
@@ -1524,7 +1563,7 @@ static int parseack(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx, aim
 							strcpy(cur1->name, cur->item->name);
 						} else
 							cur1->name = NULL;
-						aim_freetlvchain(&cur1->data);
+						aim_tlvlist_free(&cur1->data);
 						cur1->data = aim_tlvlist_copy(cur->item->data);
 					}
 				} else
@@ -1908,8 +1947,6 @@ static int snachandler(aim_session_t *sess, aim_module_t *mod, aim_frame_t *rx,
 static void ssi_shutdown(aim_session_t *sess, aim_module_t *mod)
 {
 	aim_ssi_freelist(sess);
-
-	return;
 }
 
 faim_internal int ssi_modfirst(aim_session_t *sess, aim_module_t *mod)
