@@ -4,8 +4,9 @@
  *
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,8 @@
 
 /* this is a little piece of code to handle proxy connection */
 /* it is intended to : 1st handle http proxy, using the CONNECT command
- , 2nd provide an easy way to add socks support */
+ , 2nd provide an easy way to add socks support
+ , 3rd draw women to it like flies to honey */
 
 #include "internal.h"
 #include "debug.h"
@@ -34,9 +36,6 @@
 #include "proxy.h"
 #include "util.h"
 
-#define GAIM_READ_COND  (G_IO_IN | G_IO_HUP | G_IO_ERR)
-#define GAIM_WRITE_COND (G_IO_OUT | G_IO_HUP | G_IO_ERR | G_IO_NVAL)
-
 static GaimProxyInfo *global_proxy_info = NULL;
 
 static int opt_debug = 0;
@@ -51,12 +50,17 @@ struct PHB {
 	GaimAccount *account;
 };
 
-typedef struct _GaimIOClosure {
-	GaimInputFunction function;
-	guint result;
-	gpointer data;
-
-} GaimIOClosure;
+const char* socks5errors[] = {
+	"succeeded\n",
+	"general SOCKS server failure\n",
+	"connection not allowed by ruleset\n",
+	"Network unreachable\n",
+	"Host unreachable\n",
+	"Connection refused\n",
+	"TTL expired\n",
+	"Command not supported\n",
+	"Address type not supported\n"
+};
 
 /**************************************************************************
  * Proxy structure API
@@ -180,69 +184,6 @@ gaim_global_proxy_get_info(void)
 /**************************************************************************
  * Proxy API
  **************************************************************************/
-static void gaim_io_destroy(gpointer data)
-{
-	g_free(data);
-}
-
-static gboolean gaim_io_invoke(GIOChannel *source, GIOCondition condition, gpointer data)
-{
-	GaimIOClosure *closure = data;
-	GaimInputCondition gaim_cond = 0;
-
-	if (condition & GAIM_READ_COND)
-		gaim_cond |= GAIM_INPUT_READ;
-	if (condition & GAIM_WRITE_COND)
-		gaim_cond |= GAIM_INPUT_WRITE;
-
-	/*
-	gaim_debug(GAIM_DEBUG_MISC, "proxy",
-			   "CLOSURE: callback for %d, fd is %d\n",
-			   closure->result, g_io_channel_unix_get_fd(source));
-	*/
-
-	closure->function(closure->data, g_io_channel_unix_get_fd(source), gaim_cond);
-
-	return TRUE;
-}
-
-gint gaim_input_add(gint source, GaimInputCondition condition, GaimInputFunction function, gpointer data)
-{
-	GaimIOClosure *closure = g_new0(GaimIOClosure, 1);
-	GIOChannel *channel;
-	GIOCondition cond = 0;
-
-	closure->function = function;
-	closure->data = data;
-
-	if (condition & GAIM_INPUT_READ)
-		cond |= GAIM_READ_COND;
-	if (condition & GAIM_INPUT_WRITE)
-		cond |= GAIM_WRITE_COND;
-
-	channel = g_io_channel_unix_new(source);
-	closure->result = g_io_add_watch_full(channel, G_PRIORITY_DEFAULT, cond,
-					      gaim_io_invoke, closure, gaim_io_destroy);
-
-	/*
-	gaim_debug(GAIM_DEBUG_MISC, "proxy",
-			   "CLOSURE: adding input watcher %d for fd %d\n",
-			   closure->result, source);
-	*/
-
-	g_io_channel_unref(channel);
-	return closure->result;
-}
-
-void gaim_input_remove(gint tag)
-{
-	/* gaim_debug(GAIM_DEBUG_MISC, "proxy",
-	              "CLOSURE: removing input watcher %d\n", tag); */
-	if (tag > 0)
-		g_source_remove(tag);
-}
-
-
 typedef void (*dns_callback_t)(GSList *hosts, gpointer data,
 		const char *error_message);
 
@@ -374,7 +315,7 @@ static void host_resolved(gpointer data, gint source, GaimInputCondition cond)
 	int rc, err;
 	GSList *hosts = NULL;
 	struct sockaddr *addr = NULL;
-	socklen_t addrlen;
+	size_t addrlen;
 
 	gaim_debug(GAIM_DEBUG_INFO, "dns", "Host '%s' resolved\n", req->host);
 	gaim_input_remove(req->inpa);
@@ -382,13 +323,13 @@ static void host_resolved(gpointer data, gint source, GaimInputCondition cond)
 	rc=read(req->fd_out, &err, sizeof(err));
 	if((rc==4) && (err!=0)) {
 		char message[1024];
+#if HAVE_GETADDRINFO
 		g_snprintf(message, sizeof(message), "DNS error: %s (pid=%d)",
-#ifdef HAVE_GETADDRINFO
-			gai_strerror(err),
+				   gai_strerror(err), req->dns_pid);
 #else
-			hstrerror(err),
+		g_snprintf(message, sizeof(message), "DNS error: %d (pid=%d)",
+				   err, req->dns_pid);
 #endif
-			req->dns_pid);
 		gaim_debug(GAIM_DEBUG_ERROR, "dns", "%s\n", message);
 		req->callback(NULL, req->data, message);
 		release_dns_child(req);
@@ -455,7 +396,7 @@ static void trap_gdb_bug()
 static void cope_with_gdb_brokenness()
 {
 	static gboolean already_done = FALSE;
-	char s[300], e[300];
+	char s[256], e[512];
 	int n;
 	pid_t ppid;
 
@@ -464,10 +405,13 @@ static void cope_with_gdb_brokenness()
 		return;
 	already_done = TRUE;
 	ppid = getppid();
-	snprintf(s, 300, "/proc/%d/exe", ppid);
+	snprintf(s, sizeof(s), "/proc/%d/exe", ppid);
 	n = readlink(s, e, sizeof(e));
-	e[MAX(n,sizeof(e)-1)] = '\0';
-	
+	if(n < 0)
+		return;
+
+	e[MIN(n,sizeof(e)-1)] = '\0';
+
 	if(strstr(e,"gdb")) {
 		gaim_debug(GAIM_DEBUG_INFO, "dns",
 				   "Debugger detected, performing useless query...\n");
@@ -534,12 +478,12 @@ int gaim_gethostbyname_async(const char *hostname, int port, dns_callback_t call
 			const int zero = 0;
 			int rc;
 
-#ifdef HAVE_GETADDRINFO
+#if HAVE_GETADDRINFO
 			struct addrinfo hints, *res, *tmp;
 			char servname[20];
 #else
 			struct sockaddr_in sin;
-			const socklen_t addrlen = sizeof(sin);
+			const size_t addrlen = sizeof(sin);
 #endif
 #ifdef HAVE_SIGNAL_H
 			signal(SIGHUP, SIG_DFL);
@@ -584,7 +528,7 @@ int gaim_gethostbyname_async(const char *hostname, int port, dns_callback_t call
 					write(child_out[1], &Y, 1);
 				}
 
-#ifdef HAVE_GETADDRINFO
+#if HAVE_GETADDRINFO
 				g_snprintf(servname, sizeof(servname), "%d", dns_params.port);
 				memset(&hints,0,sizeof(hints));
 				
@@ -608,7 +552,8 @@ int gaim_gethostbyname_async(const char *hostname, int port, dns_callback_t call
 				write(child_out[1], &zero, sizeof(zero));
 				tmp = res;
 				while(res) {
-					write(child_out[1], &(res->ai_addrlen), sizeof(res->ai_addrlen));
+					size_t ai_addrlen = res->ai_addrlen;
+					write(child_out[1], &ai_addrlen, sizeof(ai_addrlen));
 					write(child_out[1], res->ai_addr, res->ai_addrlen);
 					res = res->ai_next;
 				}
@@ -621,7 +566,7 @@ int gaim_gethostbyname_async(const char *hostname, int port, dns_callback_t call
 						write(child_out[1], &h_errno, sizeof(int));
 						close(child_out[1]);
 						if(opt_debug)
-							fprintf(stderr,"DNS Error: %s\n",hstrerror(h_errno));
+							fprintf(stderr,"DNS Error: %d\n", h_errno);
 						_exit(0);
 					}
 					memset(&sin, 0, sizeof(struct sockaddr_in));
@@ -697,8 +642,8 @@ gaim_gethostbyname_async(const char *hostname, int port,
 		struct hostent *hp;
 		if(!(hp = gethostbyname(hostname))) {
 			gaim_debug(GAIM_DEBUG_ERROR, "dns",
-					   "gaim_gethostbyname(\"%s\", %d) failed: %s\n",
-					   hostname, port, hstrerror(h_errno));
+					   "gaim_gethostbyname(\"%s\", %d) failed: %d\n",
+					   hostname, port, h_errno);
 			return -1;
 		}
 		memset(&sin, 0, sizeof(struct sockaddr_in));
@@ -713,7 +658,7 @@ gaim_gethostbyname_async(const char *hostname, int port,
 	req->addrlen = sizeof(sin);
 	req->data = data;
 	req->callback = callback;
-	g_timeout_add(10, host_resolved, req);
+	gaim_timeout_add(10, host_resolved, req);
 	return 0;
 }
 
@@ -724,16 +669,28 @@ no_one_calls(gpointer data, gint source, GaimInputCondition cond)
 {
 	struct PHB *phb = data;
 	unsigned int len;
-	int error=0;
+	int error=0, ret;
 
 	gaim_debug(GAIM_DEBUG_INFO, "proxy", "Connected.\n");
 
 	len = sizeof(error);
 
-	if (getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
-/*	if (ret < 0 || error != 0) { */
-		/* The fourth parameter above isn't really "error", is it? */
-		/* if(ret==0) errno = error; */
+	/*
+	 * getsockopt after a non-blocking connect returns -1 if something is
+	 * really messed up (bad descriptor, usually). Otherwise, it returns 0 and
+	 * error holds what connect would have returned if it blocked until now.
+	 * Thus, error == 0 is success, error == EINPROGRESS means "try again",
+	 * and anything else is a real error.
+	 *
+	 * (error == EINPROGRESS can happen after a select because the kernel can
+	 * be overly optimistic sometimes. select is just a hint that you might be
+	 * able to do something.)
+	 */
+	ret = getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &len);
+	if (ret == 0 && error == EINPROGRESS)
+		return; // we'll be called again later
+	if (ret < 0 || error != 0) {
+		if(ret!=0) error = errno;
 		close(source);
 		gaim_input_remove(phb->inpa);
 
@@ -747,7 +704,7 @@ no_one_calls(gpointer data, gint source, GaimInputCondition cond)
 		g_free(phb);
 
 		gaim_debug(GAIM_DEBUG_ERROR, "proxy",
-			   "getsockopt SO_ERROR check: %s\n", strerror(errno));
+			   "getsockopt SO_ERROR check: %s\n", strerror(error));
 		return;
 	}
 
@@ -804,7 +761,7 @@ proxy_connect_none(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		}
 		else {
 			gaim_debug(GAIM_DEBUG_ERROR, "proxy",
-					   "Connect failed (errno %d)\n", errno);
+					   "Connect failed: %s\n", strerror(errno));
 			close(fd);
 			return -1;
 		}
@@ -821,7 +778,7 @@ proxy_connect_none(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		}
 		fcntl(fd, F_SETFL, 0);
 		phb->port = fd;	/* bleh */
-		g_timeout_add(50, clean_connect, phb);	/* we do this because we never
+		gaim_timeout_add(50, clean_connect, phb);	/* we do this because we never
 							   want to call our callback
 							   before we return. */
 	}
@@ -832,110 +789,18 @@ proxy_connect_none(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 #define HTTP_GOODSTRING "HTTP/1.0 200"
 #define HTTP_GOODSTRING2 "HTTP/1.1 200"
 
-#if 0
-/* QQQ */
 static void
-http_uri_get_host(const char *uri)
+http_complete(struct PHB *phb, gint source)
 {
-	GaimProxyInfo *info;
-
-	char *c, *d;
-	char buffer[2048];
-
-	char host[128];
-	char user[128];
-	char pass[128];
-	int  port = 0;
-	int  len  = 0;
-
-	host[0] = '\0';
-	user[0] = '\0';
-	pass[0] = '\0';
-
-	gaim_debug(GAIM_DEBUG_MISC, "gaimrc",
-			   "gaimrc_parse_proxy_uri(%s)\n", proxy);
-
-	if ((c = strchr(proxy, ':')) == NULL)
-	{
-		gaim_debug(GAIM_DEBUG_ERROR, "gaimrc",
-				   "No URI detected.\n");
-		/* No URI detected. */
-		return FALSE;
-	}
-
-	len = c - proxy;
-
-	if (strncmp(proxy, "http://", len + 3))
-		return FALSE;
-
-	gaim_debug(GAIM_DEBUG_MISC, "gaimrc", "Found HTTP proxy.\n");
-	/* Get past "://" */
-	c += 3;
-
-	gaim_debug(GAIM_DEBUG_MISC, "gaimrc", "Looking at %s\n", c);
-
-	for (;;)
-	{
-		*buffer = '\0';
-		d = buffer;
-
-		while (*c != '\0' && *c != '@' && *c != ':' && *c != '/')
-			*d++ = *c++;
-
-		*d = '\0';
-
-		if (*c == ':')
-		{
-			/*
-			 * If there is a '@' in there somewhere, we are in the auth part.
-			 * If not, host.
-			 */
-			if (strchr(c, '@') != NULL)
-				strcpy(user, buffer);
-			else
-				strcpy(host, buffer);
-		}
-		else if (*c == '@')
-		{
-			if (user[0] == '\0')
-				strcpy(user, buffer);
-			else
-				strcpy(pass, buffer);
-		}
-		else if (*c == '/' || *c == '\0')
-		{
-			if (host[0] == '\0')
-				strcpy(host, buffer);
-			else
-				port = atoi(buffer);
-
-			/* Done. */
-			break;
-		}
-
-		c++;
-	}
-
-	/* NOTE: HTTP_PROXY takes precendence. */
-	info = gaim_global_proxy_get_info();
-
-	if (*host) gaim_proxy_info_set_host(info, host);
-	if (*user) gaim_proxy_info_set_username(info, user);
-	if (*pass) gaim_proxy_info_set_password(info, pass);
-
-	gaim_proxy_info_set_port(info, port);
-
-	gaim_debug(GAIM_DEBUG_MISC, "gaimrc",
-			   "Host: '%s', User: '%s', Password: '%s', Port: %d\n",
-			   gaim_proxy_info_get_host(info),
-			   gaim_proxy_info_get_username(info),
-			   gaim_proxy_info_get_password(info),
-			   gaim_proxy_info_get_port(info));
-
-	return TRUE;
+	gaim_debug(GAIM_DEBUG_INFO, "http proxy", "proxy connection established\n");
+	if(!phb->account || phb->account->gc)
+		phb->func(phb->data, source, GAIM_INPUT_READ);
+	g_free(phb->host);
+	g_free(phb);
 }
-#endif
 
+
+/* read the response to the CONNECT request, if we are requesting a non-port-80 tunnel */
 static void
 http_canread(gpointer data, gint source, GaimInputCondition cond)
 {
@@ -947,7 +812,7 @@ http_canread(gpointer data, gint source, GaimInputCondition cond)
 
 	gaim_input_remove(phb->inpa);
 
-	while ((nlc != 2) && (read(source, &inputline[pos++], 1) == 1)) {
+	while ((pos < sizeof(inputline)-1) && (nlc != 2) && (read(source, &inputline[pos++], 1) == 1)) {
 		if (inputline[pos - 1] == '\n')
 			nlc++;
 		else if (inputline[pos - 1] != '\r')
@@ -980,15 +845,24 @@ http_canread(gpointer data, gint source, GaimInputCondition cond)
 	}
 	else if(status!=200) {
 		gaim_debug(GAIM_DEBUG_ERROR, "proxy",
-				   "Proxy server replied: (%s)\n", p);
-		close(source);
-		source=-1;
+				   "Proxy server replied with:\n%s\n", p);
+ 		close(source);
+		source = -1;
+		
+		if ( status == 403 /* Forbidden */ ) {
+			gchar *msg = g_strdup_printf(_("Access denied: proxy server forbids port %d tunnelling."), phb->port);
+			gaim_connection_error(phb->account->gc, msg);
+			g_free(msg);
+		} else {
+			char *msg = g_strdup_printf(_("Proxy connection error %d"), status);
+			gaim_connection_error(phb->account->gc, msg);
+			g_free(msg);
+		}
+		
+	} else {
+		http_complete(phb, source);
 	}
-
-	if(!phb->account || phb->account->gc)
-		phb->func(phb->data, source, GAIM_INPUT_READ);
-	g_free(phb->host);
-	g_free(phb);
+	
 	return;
 }
 
@@ -1021,22 +895,24 @@ http_canwrite(gpointer data, gint source, GaimInputCondition cond)
 		g_free(phb);
 		return;
 	}
+
+	gaim_debug(GAIM_DEBUG_INFO, "proxy", "using CONNECT tunnelling for %s:%d\n", phb->host, phb->port);
 	request_len = g_snprintf(request, sizeof(request),
-							 "CONNECT %s:%d HTTP/1.1\r\nHost: %s:%d\r\n",
-							 phb->host, phb->port, phb->host, phb->port);
+				 "CONNECT %s:%d HTTP/1.1\r\nHost: %s:%d\r\n",
+				 phb->host, phb->port, phb->host, phb->port);
 
 	if (gaim_proxy_info_get_username(phb->gpi) != NULL) {
 		char *t1, *t2;
 		t1 = g_strdup_printf("%s:%s",
-							 gaim_proxy_info_get_username(phb->gpi),
-							 gaim_proxy_info_get_password(phb->gpi) ? 
-							 gaim_proxy_info_get_password(phb->gpi) : "");
-		t2 = tobase64(t1, strlen(t1));
+				     gaim_proxy_info_get_username(phb->gpi),
+				     gaim_proxy_info_get_password(phb->gpi) ? 
+				     gaim_proxy_info_get_password(phb->gpi) : "");
+		t2 = gaim_base64_encode(t1, strlen(t1));
 		g_free(t1);
 		g_return_if_fail(request_len < sizeof(request));
 		request_len += g_snprintf(request + request_len,
-								  sizeof(request) - request_len,
-								  "Proxy-Authorization: Basic %s\r\n", t2);
+					  sizeof(request) - request_len,
+					  "Proxy-Authorization: Basic %s\r\n", t2);
 		g_free(t2);
 	}
 
@@ -1058,6 +934,7 @@ http_canwrite(gpointer data, gint source, GaimInputCondition cond)
 		return;
 	}
 
+	/* register the response handler for the CONNECT request */
 	phb->inpa = gaim_input_add(source, GAIM_INPUT_READ, http_canread, phb);
 }
 
@@ -1082,8 +959,14 @@ proxy_connect_http(struct PHB *phb, struct sockaddr *addr, socklen_t addrlen)
 		if ((errno == EINPROGRESS) || (errno == EINTR)) {
 			gaim_debug(GAIM_DEBUG_WARNING, "http proxy",
 					   "Connect would have blocked.\n");
-			phb->inpa = gaim_input_add(fd, GAIM_INPUT_WRITE,
-									   http_canwrite, phb);
+			
+			if (phb->port != 80) {
+				/* we need to do CONNECT first */
+				phb->inpa = gaim_input_add(fd, GAIM_INPUT_WRITE,
+							   http_canwrite, phb);
+			} else {
+				http_complete(phb, fd);
+			}
 		} else {
 			close(fd);
 			return -1;
@@ -1271,7 +1154,7 @@ s5_canread_again(gpointer data, gint source, GaimInputCondition cond)
 	gaim_input_remove(phb->inpa);
 	gaim_debug(GAIM_DEBUG_INFO, "socks5 proxy", "Able to read again.\n");
 
-	if (read(source, buf, 10) < 10) {
+	if (read(source, buf, 4) < 4) {
 		gaim_debug(GAIM_DEBUG_WARNING, "socks5 proxy", "or not...\n");
 		close(source);
 
@@ -1286,7 +1169,10 @@ s5_canread_again(gpointer data, gint source, GaimInputCondition cond)
 		return;
 	}
 	if ((buf[0] != 0x05) || (buf[1] != 0x00)) {
-		gaim_debug(GAIM_DEBUG_ERROR, "socks5 proxy", "Bad data.\n");
+		if ((buf[0] == 0x05) && (buf[1] < 0x09))
+			gaim_debug(GAIM_DEBUG_ERROR, "socks5 proxy", socks5errors[buf[1]]);
+		else
+			gaim_debug(GAIM_DEBUG_ERROR, "socks5 proxy", "Bad data.\n");
 		close(source);
 
 		if (phb->account == NULL ||
@@ -1300,6 +1186,25 @@ s5_canread_again(gpointer data, gint source, GaimInputCondition cond)
 		return;
 	}
 
+	/* Skip past BND.ADDR */
+	switch(buf[3]) {
+		case 0x01: /* the address is a version-4 IP address, with a length of 4 octets */
+			read(source, buf, 4);
+			break;
+		case 0x03: /* the address field contains a fully-qualified domain name.  The first
+					  octet of the address field contains the number of octets of name that
+					  follow, there is no terminating NUL octet. */
+			read(source, buf, 1);
+			read(source, buf, buf[0]);
+			break;
+		case 0x04: /* the address is a version-6 IP address, with a length of 16 octets */
+			read(source, buf, 16);
+			break;
+	}
+
+	/* Skip past BND.PORT */
+	read(source, buf, 2);
+
 	if (phb->account == NULL ||
 		gaim_account_get_connection(phb->account) != NULL) {
 
@@ -1651,15 +1556,26 @@ gaim_proxy_connect(GaimAccount *account, const char *host, int port,
 		if ((tmp = g_getenv("HTTP_PROXY")) != NULL ||
 			(tmp = g_getenv("http_proxy")) != NULL ||
 			(tmp= g_getenv("HTTPPROXY")) != NULL) {
-			connecthost = tmp;
-			gaim_proxy_info_set_host(phb->gpi, connecthost);
-		}
+			char *proxyhost,*proxypath;
+			int proxyport;
 
-		if ((tmp = g_getenv("HTTP_PROXY_PORT")) != NULL ||
-			(tmp = g_getenv("http_proxy_port")) != NULL ||
-			(tmp = g_getenv("HTTPPROXYPORT")) != NULL) {
-			connectport = atoi(tmp);
-			gaim_proxy_info_set_port(phb->gpi, connectport);
+			/* http_proxy-format: 
+			 * export http_proxy="http://your.proxy.server:port/" 
+			 */
+			if(gaim_url_parse(tmp, &proxyhost, &proxyport, &proxypath)) {
+				gaim_proxy_info_set_host(phb->gpi, proxyhost);
+				g_free(proxyhost);
+				g_free(proxypath);
+
+				/* only for backward compatibility */
+				if (proxyport == 80 && 
+				    ((tmp = g_getenv("HTTP_PROXY_PORT")) != NULL ||
+				     (tmp = g_getenv("http_proxy_port")) != NULL ||
+				     (tmp = g_getenv("HTTPPROXYPORT")) != NULL))
+				    proxyport = atoi(tmp);
+				
+				gaim_proxy_info_set_port(phb->gpi, proxyport);
+			}
 		}
 
 		if ((tmp = g_getenv("HTTP_PROXY_USER")) != NULL ||
@@ -1706,6 +1622,23 @@ gaim_proxy_connect(GaimAccount *account, const char *host, int port,
 									connection_host_resolved, phb);
 }
 
+int
+gaim_proxy_connect_socks5(GaimProxyInfo *gpi, const char *host, int port,
+		GaimInputFunction func, gpointer data)
+{
+	struct PHB *phb;
+
+	phb = g_new0(struct PHB, 1);
+	phb->gpi = gpi;
+	phb->func = func;
+	phb->data = data;
+	phb->host = g_strdup(host);
+	phb->port = port;
+
+	return gaim_gethostbyname_async(gaim_proxy_info_get_host(gpi), gaim_proxy_info_get_port(gpi),
+									connection_host_resolved, phb);
+}
+
 
 static void
 proxy_pref_cb(const char *name, GaimPrefType type, gpointer value,
