@@ -4,7 +4,9 @@
  *
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,15 +65,17 @@ typedef enum
 	GAIM_PROTO_TREPIA,      /**< Trepia IM protocol        */
 	GAIM_PROTO_BATTLENET,   /**< Battle.NET protocol       */
 	GAIM_PROTO_SSCP,	/**< SSCP (ConfMgr) protocol   */
+	GAIM_PROTO_BLOGGER,	/**< Blogger xml-rpc protocol  */
+	/* If you add any more prpl numbers,  then a curse will
+	 * be placed upon your keyboard the likes of which the
+	 * world has never known. prpl numbers are no longer
+	 * necessary.  --Nathan
+	 * */
 	GAIM_PROTO_UNTAKEN      /**< Untaken protocol number   */
 
 } GaimProtocol;
 
 
-#include "blist.h"
-#include "proxy.h"
-#include "plugin.h"
-
 /** Default protocol plugin description */
 #define GAIM_PRPL_DESC(x) \
 		"Allows gaim to use the " (x) " protocol.\n\n"      \
@@ -82,7 +86,7 @@ typedef enum
 		"in the \"Tools\" menu in the buddy list window."
 
 /** Default protocol */
-#define GAIM_PROTO_DEFAULT GAIM_PROTO_OSCAR
+#define GAIM_PROTO_DEFAULT "prpl-oscar"
 
 /*@}*/
 
@@ -91,8 +95,14 @@ typedef enum
  */
 typedef enum
 {
-	GAIM_IM_AUTO_RESP = 0x01,    /**< Auto response.    */
-} GaimImFlags;
+	GAIM_CONV_IM_AUTO_RESP = 0x0001,    /**< Auto response.    */
+	GAIM_CONV_IM_IMAGES    = 0x0002     /**< Contains images.  */
+
+} GaimConvImFlags;
+
+#include "blist.h"
+#include "proxy.h"
+#include "plugin.h"
 
 /**
  * Protocol options
@@ -101,24 +111,6 @@ typedef enum
  */
 typedef enum
 {
-	/**
-	 * TOC and Oscar send HTML-encoded messages;
-	 * most other protocols don't.
-	 */
-#if 0
-	#define OPT_PROTO_HTML            0x00000001 this should be per-connection */
-#endif
-
-	/**
-	 * Synchronize the time between the local computer and the server.
-	 *
-	 * TOC and Oscar have signon time, and the server's time needs
-	 * to be adjusted to match your computer's time.
-	 *
-	 * We wouldn't need this if everyone used NTP.
-	 */
-	OPT_PROTO_CORRECT_TIME = 0x00000002,
-
 	/**
 	 * Use a unique name, not an alias, for chat rooms.
 	 *
@@ -184,6 +176,10 @@ typedef enum
 /** Custom away message. */
 #define GAIM_AWAY_CUSTOM _("Custom")
 
+/** Some structs defined in roomlist.h */
+struct _GaimRoomlist;
+struct _GaimRoomlistRoom;
+
 /**
  * A protocol plugin information structure.
  *
@@ -192,7 +188,8 @@ typedef enum
  */
 struct _GaimPluginProtocolInfo
 {
-	GaimProtocol protocol;        /**< The protocol type.         */
+	unsigned int api_version;     /**< API version number.             */
+
 	GaimProtocolOptions options;  /**< Protocol options.          */
 
 	GList *user_splits;      /* A GList of GaimAccountUserSplit */
@@ -202,65 +199,45 @@ struct _GaimPluginProtocolInfo
 	 * Returns the base icon name for the given buddy and account.
 	 * If buddy is NULL, it will return the name to use for the account's icon
 	 */
-	const char *(*list_icon)(GaimAccount *account, struct buddy *buddy);
+	const char *(*list_icon)(GaimAccount *account, GaimBuddy *buddy);
 
 	/**
 	 * Fills the four char**'s with string identifiers for "emblems"
 	 * that the UI will interpret and display as relevant
 	 */
-	void (*list_emblems)(struct buddy *buddy, char **se, char **sw,
+	void (*list_emblems)(GaimBuddy *buddy, char **se, char **sw,
 						  char **nw, char **ne);
 
 	/**
 	 * Gets a short string representing this buddy's status.  This will
 	 * be shown on the buddy list.
 	 */
-	char *(*status_text)(struct buddy *buddy);
+	char *(*status_text)(GaimBuddy *buddy);
 
 	/**
 	 * Gets a string to put in the buddy list tooltip.
 	 */
-	char *(*tooltip_text)(struct buddy *buddy);
+	char *(*tooltip_text)(GaimBuddy *buddy);
 
 	GList *(*away_states)(GaimConnection *gc);
-	GList *(*actions)(GaimConnection *gc);
 
-	GList *(*buddy_menu)(GaimConnection *, const char *);
+	GList *(*blist_node_menu)(GaimBlistNode *node);
 	GList *(*chat_info)(GaimConnection *);
 
 	/* All the server-related functions */
-
-	/*
-	 * A lot of these (like get_dir) are protocol-dependent and should
-	 * be removed. ones like set_dir (which is also protocol-dependent)
-	 * can stay though because there's a dialog (i.e. the prpl says you
-	 * can set your dir info, the ui shows a dialog and needs to call
-	 * set_dir in order to set it)
-	 */
 	void (*login)(GaimAccount *);
 	void (*close)(GaimConnection *);
 	int  (*send_im)(GaimConnection *, const char *who,
-					const char *message, int len,
-					GaimImFlags flags);
+					const char *message,
+					GaimConvImFlags flags);
 	void (*set_info)(GaimConnection *, const char *info);
 	int  (*send_typing)(GaimConnection *, const char *name, int typing);
 	void (*get_info)(GaimConnection *, const char *who);
 	void (*set_away)(GaimConnection *, const char *state, const char *message);
-	void (*get_away)(GaimConnection *, const char *who);
-	void (*set_dir)(GaimConnection *, const char *first,
-					const char *middle, const char *last,
-					const char *maiden, const char *city,
-					const char *state, const char *country, int web);
-	void (*get_dir)(GaimConnection *, const char *who);
-	void (*dir_search)(GaimConnection *, const char *first,
-					   const char *middle, const char *last,
-					   const char *maiden, const char *city,
-					   const char *state, const char *country,
-					   const char *email);
 	void (*set_idle)(GaimConnection *, int idletime);
 	void (*change_passwd)(GaimConnection *, const char *old_pass,
 						  const char *new_pass);
-	void (*add_buddy)(GaimConnection *, const char *name);
+	void (*add_buddy)(GaimConnection *, const char *name, GaimGroup *group);
 	void (*add_buddies)(GaimConnection *, GList *buddies);
 	void (*remove_buddy)(GaimConnection *, const char *name,
 						const char *group);
@@ -273,6 +250,7 @@ struct _GaimPluginProtocolInfo
 	void (*set_permit_deny)(GaimConnection *);
 	void (*warn)(GaimConnection *, const char *who, int anonymous);
 	void (*join_chat)(GaimConnection *, GHashTable *components);
+	void (*reject_chat)(GaimConnection *, GHashTable *components);
 	void (*chat_invite)(GaimConnection *, int id,
 						const char *who, const char *message);
 	void (*chat_leave)(GaimConnection *, int id);
@@ -300,14 +278,26 @@ struct _GaimPluginProtocolInfo
 	void (*rename_group)(GaimConnection *, const char *old_group,
 						 const char *new_group, GList *members);
 
-	void (*buddy_free)(struct buddy *);
+	void (*buddy_free)(GaimBuddy *);
 
-	/* this is really bad. */
 	void (*convo_closed)(GaimConnection *, const char *who);
 
-	char *(*normalize)(const char *);
+	const char *(*normalize)(const GaimAccount *, const char *);
 
 	void (*set_buddy_icon)(GaimConnection *, const char *filename);
+
+	void (*remove_group)(GaimConnection *gc, const char *group);
+
+	char *(*get_cb_real_name)(GaimConnection *gc, int id, const char *who);
+
+	void (*set_chat_topic)(GaimConnection *gc, int id, const char *topic);
+
+	GaimChat *(*find_blist_chat)(GaimAccount *account, const char *name);
+
+	/* room listing prpl callbacks */
+	struct _GaimRoomlist *(*roomlist_get_list)(GaimConnection *gc);
+	void (*roomlist_cancel)(struct _GaimRoomlist *list);
+	void (*roomlist_expand_category)(struct _GaimRoomlist *list, struct _GaimRoomlistRoom *category);
 };
 
 #define GAIM_IS_PROTOCOL_PLUGIN(plugin) \
@@ -316,6 +306,10 @@ struct _GaimPluginProtocolInfo
 #define GAIM_PLUGIN_PROTOCOL_INFO(plugin) \
 	((GaimPluginProtocolInfo *)(plugin)->info->extra_info)
 
+/* It's not like we're going to run out of integers for this version
+   number, but we only want to really change it once per release. */
+/* GAIM_PRPL_API_VERSION last changed for version: 0.78 */
+#define GAIM_PRPL_API_VERSION 4
 
 #ifdef __cplusplus
 extern "C" {
@@ -333,7 +327,7 @@ const char *gaim_prpl_num_to_id(GaimProtocol protocol);
 /**
  * Returns the plugin number for a protocol ID.
  *
- * @param protocol_id The protocol ID.
+ * @param id The protocol ID.
  *
  * @return The protocol ID for valid protocol plugin IDs.
  */
@@ -342,43 +336,9 @@ GaimProtocol gaim_prpl_id_to_num(const char *id);
 /**
  * Finds a protocol plugin structure of the specified type.
  *
- * @param type The protocol plugin;
+ * @param id The protocol plugin;
  */
-GaimPlugin *gaim_find_prpl(GaimProtocol type);
-
-/**
- * Shows a message saying that somebody added you as a buddy, and asks
- * if you would like to do the same.
- *
- * @param gc    The gaim connection.
- * @param id    The ID of the user.
- * @param who   The username.
- * @param alias The user's alias.
- * @param msg   The message to go along with the request.
- */
-void show_got_added(GaimConnection *gc, const char *id,
-					const char *who, const char *alias, const char *msg);
-
-/**
- * Retrieves and sets the new buddy icon for a user.
- *
- * @param gc   The gaim connection.
- * @param who  The user.
- * @param data The icon data.
- * @param len  The length of @a data.
- */
-void set_icon_data(GaimConnection *gc, const char *who, void *data, int len);
-
-/**
- * Retrieves the buddy icon data for a user.
- *
- * @param gc  The gaim connection.
- * @param who The user.
- * @param len The returned length of the data.
- *
- * @return The buddy icon data.
- */
-void *get_icon_data(GaimConnection *gc, const char *who, int *len);
+GaimPlugin *gaim_find_prpl(const char *id);
 
 #ifdef __cplusplus
 }
