@@ -4,8 +4,10 @@
  *
  * gaim
  *
- * Copyright (C) 2003 Christian Hammond <chipx86@gnupdate.org>
- * 
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -27,6 +29,8 @@
 #include <glib-object.h>
 #include <glib.h>
 
+#include "account.h"
+
 /**
  * Request types.
  */
@@ -48,25 +52,63 @@ typedef enum
 	GAIM_REQUEST_FIELD_STRING,
 	GAIM_REQUEST_FIELD_INTEGER,
 	GAIM_REQUEST_FIELD_BOOLEAN,
-	GAIM_REQUEST_FIELD_CHOICE
+	GAIM_REQUEST_FIELD_CHOICE,
+	GAIM_REQUEST_FIELD_LIST,
+	GAIM_REQUEST_FIELD_LABEL,
+	GAIM_REQUEST_FIELD_ACCOUNT
 
 } GaimRequestFieldType;
 
+/**
+ * Multiple fields request data.
+ */
+typedef struct
+{
+	GList *groups;
+
+	GHashTable *fields;
+
+	GList *required_fields;
+
+	void *ui_data;
+
+} GaimRequestFields;
+
+/**
+ * A group of fields with a title.
+ */
+typedef struct
+{
+	GaimRequestFields *fields_list;
+
+	char *title;
+
+	GList *fields;
+
+} GaimRequestFieldGroup;
+
 /**
  * A request field.
  */
 typedef struct
 {
 	GaimRequestFieldType type;
+	GaimRequestFieldGroup *group;
 
 	char *id;
 	char *label;
+	char *type_hint;
+
+	gboolean visible;
+	gboolean required;
 
 	union
 	{
 		struct
 		{
 			gboolean multiline;
+			gboolean masked;
+			gboolean editable;
 			char *default_value;
 			char *value;
 
@@ -95,36 +137,33 @@ typedef struct
 
 		} choice;
 
+		struct
+		{
+			GList *items;
+			GHashTable *item_data;
+			GList *selected;
+			GHashTable *selected_table;
+
+			gboolean multiple_selection;
+
+		} list;
+
+		struct
+		{
+			GaimAccount *default_account;
+			GaimAccount *account;
+			gboolean show_all;
+
+			GaimFilterAccountFunc filter_func;
+
+		} account;
+
 	} u;
 
 	void *ui_data;
 
 } GaimRequestField;
 
-/**
- * Multiple fields request data.
- */
-typedef struct
-{
-	GList *groups;
-
-	GHashTable *fields;
-
-} GaimRequestFields;
-
-/**
- * A group of fields with a title.
- */
-typedef struct
-{
-	GaimRequestFields *fields_list;
-
-	char *title;
-
-	GList *fields;
-
-} GaimRequestFieldGroup;
-
 /**
  * Request UI operations.
  */
@@ -132,7 +171,7 @@ typedef struct
 {
 	void *(*request_input)(const char *title, const char *primary,
 						   const char *secondary, const char *default_value,
-						   gboolean multiline, gboolean masked,
+						   gboolean multiline, gboolean masked, gchar *hint,
 						   const char *ok_text, GCallback ok_cb,
 						   const char *cancel_text, GCallback cancel_cb,
 						   void *user_data);
@@ -152,8 +191,10 @@ typedef struct
 							const char *cancel_text, GCallback cancel_cb,
 							void *user_data);
 
+	void *(*request_file)(const char *title, const char *filename,
+			      GCallback ok_cb, GCallback cancel_cb,
+			      void *user_data);
 	void (*close_request)(GaimRequestType type, void *ui_handle);
-
 } GaimRequestUiOps;
 
 typedef void (*GaimRequestInputCb)(void *, const char *);
@@ -201,6 +242,47 @@ void gaim_request_fields_add_group(GaimRequestFields *fields,
  */
 GList *gaim_request_fields_get_groups(const GaimRequestFields *fields);
 
+/**
+ * Returns whether or not the field with the specified ID exists.
+ *
+ * @param fields The fields list.
+ * @param id     The ID of the field.
+ *
+ * @return TRUE if the field exists, or FALSE.
+ */
+gboolean gaim_request_fields_exists(const GaimRequestFields *fields,
+									const char *id);
+
+/**
+ * Returns a list of all required fields.
+ *
+ * @param fields The fields list.
+ *
+ * @return The list of required fields.
+ */
+const GList *gaim_request_fields_get_required(const GaimRequestFields *fields);
+
+/**
+ * Returns whether or not a field with the specified ID is required.
+ *
+ * @param fields The fields list.
+ * @param id     The field ID.
+ *
+ * @return TRUE if the specified field is required, or FALSE.
+ */
+gboolean gaim_request_fields_is_field_required(const GaimRequestFields *fields,
+											   const char *id);
+
+/**
+ * Returns whether or not all required fields have values.
+ *
+ * @param fields The fields list.
+ *
+ * @return TRUE if all required fields have values, or FALSE.
+ */
+gboolean gaim_request_fields_all_required_filled(
+	const GaimRequestFields *fields);
+
 /**
  * Return the field with the specified ID.
  *
@@ -256,6 +338,17 @@ gboolean gaim_request_fields_get_bool(const GaimRequestFields *fields,
 int gaim_request_fields_get_choice(const GaimRequestFields *fields,
 								   const char *id);
 
+/**
+ * Returns the account of a field with the specified ID.
+ *
+ * @param fields The fields list.
+ * @param id     The ID of the field.
+ *
+ * @return The account value, if found, or NULL otherwise.
+ */
+GaimAccount *gaim_request_fields_get_account(const GaimRequestFields *fields,
+											 const char *id);
+
 /*@}*/
 
 /**************************************************************************/
@@ -342,6 +435,35 @@ void gaim_request_field_destroy(GaimRequestField *field);
  */
 void gaim_request_field_set_label(GaimRequestField *field, const char *label);
 
+/**
+ * Sets whether or not a field is visible.
+ *
+ * @param field   The field.
+ * @param visible TRUE if visible, or FALSE if not.
+ */
+void gaim_request_field_set_visible(GaimRequestField *field, gboolean visible);
+
+/**
+ * Sets the type hint for the field.
+ *
+ * This is optionally used by the UIs to provide such features as
+ * auto-completion for type hints like "screenname."
+ *
+ * @param field     The field.
+ * @param type_hint The type hint.
+ */
+void gaim_request_field_set_type_hint(GaimRequestField *field,
+									  const char *type_hint);
+
+/**
+ * Sets whether or not a field is required.
+ *
+ * @param field    The field.
+ * @param required TRUE if required, or FALSE.
+ */
+void gaim_request_field_set_required(GaimRequestField *field,
+									 gboolean required);
+
 /**
  * Returns the type of a field.
  *
@@ -369,6 +491,33 @@ const char *gaim_request_field_get_id(const GaimRequestField *field);
  */
 const char *gaim_request_field_get_label(const GaimRequestField *field);
 
+/**
+ * Returns whether or not a field is visible.
+ *
+ * @param field The field.
+ *
+ * @return TRUE if the field is visible. FALSE otherwise.
+ */
+gboolean gaim_request_field_is_visible(const GaimRequestField *field);
+
+/**
+ * Returns the field's type hint.
+ *
+ * @param field The field.
+ *
+ * @return The field's type hint.
+ */
+const char *gaim_request_field_get_type_hint(const GaimRequestField *field);
+
+/**
+ * Returns whether or not a field is required.
+ *
+ * @param field The field.
+ *
+ * @return TRUE if the field is required, or FALSE.
+ */
+gboolean gaim_request_field_is_required(const GaimRequestField *field);
+
 /*@}*/
 
 /**************************************************************************/
@@ -409,11 +558,30 @@ void gaim_request_field_string_set_default_value(GaimRequestField *field,
 void gaim_request_field_string_set_value(GaimRequestField *field,
 										 const char *value);
 
+/**
+ * Sets whether or not a string field is masked
+ * (commonly used for password fields).
+ *
+ * @param field  The field.
+ * @param masked The masked value.
+ */
+void gaim_request_field_string_set_masked(GaimRequestField *field,
+										  gboolean masked);
+
+/**
+ * Sets whether or not a string field is editable.
+ *
+ * @param field    The field.
+ * @param editable The editable value.
+ */
+void gaim_request_field_string_set_editable(GaimRequestField *field,
+											gboolean editable);
+
 /**
  * Returns the default value in a string field.
  *
  * @param field The field.
- * 
+ *
  * @return The default value.
  */
 const char *gaim_request_field_string_get_default_value(
@@ -437,6 +605,24 @@ const char *gaim_request_field_string_get_value(const GaimRequestField *field);
  */
 gboolean gaim_request_field_string_is_multiline(const GaimRequestField *field);
 
+/**
+ * Returns whether or not a string field is masked.
+ *
+ * @param field The field.
+ *
+ * @return @c TRUE if the field is masked, or @c FALSE otherwise.
+ */
+gboolean gaim_request_field_string_is_masked(const GaimRequestField *field);
+
+/**
+ * Returns whether or not a string field is editable.
+ *
+ * @param field The field.
+ *
+ * @return @c TRUE if the field is editable, or @c FALSE otherwise.
+ */
+gboolean gaim_request_field_string_is_editable(const GaimRequestField *field);
+
 /*@}*/
 
 /**************************************************************************/
@@ -478,7 +664,7 @@ void gaim_request_field_int_set_value(GaimRequestField *field, int value);
  * Returns the default value in an integer field.
  *
  * @param field The field.
- * 
+ *
  * @return The default value.
  */
 int gaim_request_field_int_get_default_value(const GaimRequestField *field);
@@ -517,8 +703,8 @@ GaimRequestField *gaim_request_field_bool_new(const char *id,
 /**
  * Sets the default value in an boolean field.
  *
- * @param field The field.
- * @param value The default value.
+ * @param field         The field.
+ * @param default_value The default value.
  */
 void gaim_request_field_bool_set_default_value(GaimRequestField *field,
 											   gboolean default_value);
@@ -526,8 +712,8 @@ void gaim_request_field_bool_set_default_value(GaimRequestField *field,
 /**
  * Sets the value in an boolean field.
  *
- * @param field         The field.
- * @param default_value The default value.
+ * @param field The field.
+ * @param value The value.
  */
 void gaim_request_field_bool_set_value(GaimRequestField *field,
 									   gboolean value);
@@ -536,7 +722,7 @@ void gaim_request_field_bool_set_value(GaimRequestField *field,
  * Returns the default value in an boolean field.
  *
  * @param field The field.
- * 
+ *
  * @return The default value.
  */
 gboolean gaim_request_field_bool_get_default_value(
@@ -603,7 +789,7 @@ void gaim_request_field_choice_set_value(GaimRequestField *field, int value);
  * Returns the default value in an choice field.
  *
  * @param field The field.
- * 
+ *
  * @return The default value.
  */
 int gaim_request_field_choice_get_default_value(const GaimRequestField *field);
@@ -628,6 +814,249 @@ GList *gaim_request_field_choice_get_labels(const GaimRequestField *field);
 
 /*@}*/
 
+/**************************************************************************/
+/** @name List Field API                                                  */
+/**************************************************************************/
+/*@{*/
+
+/**
+ * Creates a multiple list item field.
+ *
+ * @param id   The field ID.
+ * @param text The optional label of the field.
+ *
+ * @return The new field.
+ */
+GaimRequestField *gaim_request_field_list_new(const char *id, const char *text);
+
+/**
+ * Sets whether or not a list field allows multiple selection.
+ *
+ * @param field        The list field.
+ * @param multi_select TRUE if multiple selection is enabled,
+ *                     or FALSE otherwise.
+ */
+void gaim_request_field_list_set_multi_select(GaimRequestField *field,
+											  gboolean multi_select);
+
+/**
+ * Returns whether or not a list field allows multiple selection.
+ *
+ * @param field The list field.
+ *
+ * @return TRUE if multiple selection is enabled, or FALSE otherwise.
+ */
+gboolean gaim_request_field_list_get_multi_select(
+	const GaimRequestField *field);
+
+/**
+ * Returns the data for a particular item.
+ *
+ * @param field The list field.
+ * @param text  The item text.
+ *
+ * @return The data associated with the item.
+ */
+void *gaim_request_field_list_get_data(const GaimRequestField *field,
+									   const char *text);
+
+/**
+ * Adds an item to a list field.
+ *
+ * @param field The list field.
+ * @param item  The list item.
+ * @param data  The associated data.
+ */
+void gaim_request_field_list_add(GaimRequestField *field,
+								 const char *item, void *data);
+
+/**
+ * Adds a selected item to the list field.
+ *
+ * @param field The field.
+ * @param item  The item to add.
+ */
+void gaim_request_field_list_add_selected(GaimRequestField *field,
+										  const char *item);
+
+/**
+ * Clears the list of selected items in a list field.
+ *
+ * @param field The field.
+ */
+void gaim_request_field_list_clear_selected(GaimRequestField *field);
+
+/**
+ * Sets a list of selected items in a list field.
+ *
+ * @param field The field.
+ * @param items The list of selected items.
+ */
+void gaim_request_field_list_set_selected(GaimRequestField *field,
+										  GList *items);
+
+/**
+ * Returns whether or not a particular item is selected in a list field.
+ *
+ * @param field The field.
+ * @param item  The item.
+ *
+ * @return TRUE if the item is selected. FALSE otherwise.
+ */
+gboolean gaim_request_field_list_is_selected(const GaimRequestField *field,
+											 const char *item);
+
+/**
+ * Returns a list of selected items in a list field.
+ *
+ * To retrieve the data for each item, use
+ * gaim_request_field_list_get_data().
+ *
+ * @param field The field.
+ *
+ * @return The list of selected items.
+ */
+const GList *gaim_request_field_list_get_selected(
+	const GaimRequestField *field);
+
+/**
+ * Returns a list of items in a list field.
+ *
+ * @param field The field.
+ *
+ * @return The list of items.
+ */
+const GList *gaim_request_field_list_get_items(const GaimRequestField *field);
+
+/*@}*/
+
+/**************************************************************************/
+/** @name Label Field API                                                 */
+/**************************************************************************/
+/*@{*/
+
+/**
+ * Creates a label field.
+ *
+ * @param id   The field ID.
+ * @param text The label of the field.
+ *
+ * @return The new field.
+ */
+GaimRequestField *gaim_request_field_label_new(const char *id,
+											   const char *text);
+
+/*@}*/
+
+/**************************************************************************/
+/** @name Account Field API                                               */
+/**************************************************************************/
+/*@{*/
+
+/**
+ * Creates an account field.
+ *
+ * By default, this field will not show offline accounts.
+ *
+ * @param id      The field ID.
+ * @param text    The text label of the field.
+ * @param account The optional default account.
+ *
+ * @return The new field.
+ */
+GaimRequestField *gaim_request_field_account_new(const char *id,
+												 const char *text,
+												 GaimAccount *account);
+
+/**
+ * Sets the default account on an account field.
+ *
+ * @param field         The account field.
+ * @param default_value The default account.
+ */
+void gaim_request_field_account_set_default_value(GaimRequestField *field,
+												  GaimAccount *default_value);
+
+/**
+ * Sets the account in an account field.
+ *
+ * @param field The account field.
+ * @param value The account.
+ */
+void gaim_request_field_account_set_value(GaimRequestField *field,
+										  GaimAccount *value);
+
+/**
+ * Sets whether or not to show all accounts in an account field.
+ *
+ * If TRUE, all accounts, online or offline, will be shown. If FALSE,
+ * only online accounts will be shown.
+ *
+ * @param field    The account field.
+ * @param show_all Whether or not to show all accounts.
+ */
+void gaim_request_field_account_set_show_all(GaimRequestField *field,
+											 gboolean show_all);
+
+/**
+ * Sets the account filter function in an account field.
+ *
+ * This function will determine which accounts get displayed and which
+ * don't.
+ *
+ * @param field       The account field.
+ * @param filter_func The account filter function.
+ */
+void gaim_request_field_account_set_filter(GaimRequestField *field,
+										   GaimFilterAccountFunc filter_func);
+
+/**
+ * Returns the default account in an account field.
+ *
+ * @param field The field.
+ *
+ * @return The default account.
+ */
+GaimAccount *gaim_request_field_account_get_default_value(
+		const GaimRequestField *field);
+
+/**
+ * Returns the user-entered account in an account field.
+ *
+ * @param field The field.
+ *
+ * @return The user-entered account.
+ */
+GaimAccount *gaim_request_field_account_get_value(
+		const GaimRequestField *field);
+
+/**
+ * Returns whether or not to show all accounts in an account field.
+ *
+ * If TRUE, all accounts, online or offline, will be shown. If FALSE,
+ * only online accounts will be shown.
+ *
+ * @param field    The account field.
+ * @return Whether or not to show all accounts.
+ */
+gboolean gaim_request_field_account_get_show_all(
+		const GaimRequestField *field);
+
+/**
+ * Returns the account filter function in an account field.
+ *
+ * This function will determine which accounts get displayed and which
+ * don't.
+ *
+ * @param field       The account field.
+ *
+ * @return The account filter function.
+ */
+GaimFilterAccountFunc gaim_request_field_account_get_filter(
+		const GaimRequestField *field);
+
+/*@}*/
+
 /**************************************************************************/
 /** @name Request API                                                     */
 /**************************************************************************/
@@ -643,6 +1072,9 @@ GList *gaim_request_field_choice_get_labels(const GaimRequestField *field);
  * @param default_value The default value.
  * @param multiline     TRUE if the inputted text can span multiple lines.
  * @param masked        TRUE if the inputted text should be masked in some way.
+ * @param hint          Optionally suggest how the input box should appear.
+ *                      Use "html," for example, to allow the user to enter
+ *                      HTML.
  * @param ok_text       The text for the OK button.
  * @param ok_cb         The callback for the OK button.
  * @param cancel_text   The text for the cancel button.
@@ -654,7 +1086,7 @@ GList *gaim_request_field_choice_get_labels(const GaimRequestField *field);
 void *gaim_request_input(void *handle, const char *title,
 						 const char *primary, const char *secondary,
 						 const char *default_value,
-						 gboolean multiline, gboolean masked,
+						 gboolean multiline, gboolean masked, gchar *hint,
 						 const char *ok_text, GCallback ok_cb,
 						 const char *cancel_text, GCallback cancel_cb,
 						 void *user_data);
@@ -673,7 +1105,7 @@ void *gaim_request_input(void *handle, const char *title,
  * @param cancel_cb     The callback for the cancel button.
  * @param user_data     The data to pass to the callback.
  * @param choice_count  The number of choices.
- * @param choice        The choices.
+ * @param ...           The choices.
  *
  * @return A UI-specific handle.
  */
@@ -722,7 +1154,7 @@ void *gaim_request_choice_varg(void *handle, const char *title,
  * @param default_action The default value.
  * @param user_data      The data to pass to the callback.
  * @param action_count   The number of actions.
- * @param action         The first action.
+ * @param ...            A list of actions.
  *
  * @return A UI-specific handle.
  */
@@ -779,9 +1211,6 @@ void *gaim_request_fields(void *handle, const char *title,
 /**
  * Closes a request.
  *
- * This should be used only by the UI operation functions and part of the
- * core.
- *
  * @param type     The request type.
  * @param uihandle The request UI handle.
  */
@@ -824,6 +1253,24 @@ void gaim_request_close_with_handle(void *handle);
 
 /*@}*/
 
+/**
+ * Displays file selector request dialog.  Returns the selected filename into
+ * the callback.
+ *
+ * @param handle      The plugin or connection handle.
+ * @param title       The title for the dialog (may be NULL)
+ * @param filename    The default filename (may be NULL)
+ * @param ok_cb       The callback for the OK button.
+ * @param cancel_cb   The callback for the cancel button.
+ * @param user_data   The data to pass to the callback.
+ *
+ * @return A UI-specific handle.
+ */
+void *gaim_request_file(void *handle, const char *title, const char *filename,
+			GCallback ok_cb, GCallback cancel_cb,
+			void *user_data);
+
+
 /**************************************************************************/
 /** @name UI Operations API                                               */
 /**************************************************************************/
@@ -835,15 +1282,15 @@ void gaim_request_close_with_handle(void *handle);
  *
  * @param ops The UI operations structure.
  */
-void gaim_set_request_ui_ops(GaimRequestUiOps *ops);
+void gaim_request_set_ui_ops(GaimRequestUiOps *ops);
 
 /**
  * Returns the UI operations structure to be used when displaying a
  * request.
  *
- * @param ops The UI operations structure.
+ * @return The UI operations structure.
  */
-GaimRequestUiOps *gaim_get_request_ui_ops(void);
+GaimRequestUiOps *gaim_request_get_ui_ops(void);
 
 /*@}*/
 
