@@ -3,7 +3,9 @@
  *
  * gaim
  *
- * Copyright (C) 2003, Nathan Walp <faceprint@faceprint.com>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,14 +74,14 @@ extern "C" {
  *
  * @param ops The UI sound operations structure.
  */
-void gaim_set_sound_ui_ops(GaimSoundUiOps *ops);
+void gaim_sound_set_ui_ops(GaimSoundUiOps *ops);
 
 /**
  * Gets the UI sound operations
  *
  * @return The UI sound operations structure.
  */
-GaimSoundUiOps *gaim_get_sound_ui_ops(void);
+GaimSoundUiOps *gaim_sound_get_ui_ops(void);
 
 /**
  * Initializes the sound subsystem
